C****************************************************************
C   Modue Name		Program		Responsible Organization
C----------------------------------------------------------------
C   ETHERNET_MONITOR	ETHERMON	GE-GESD, Syracuse, NY
C
C   Description:
C
C   Calling List:
C	A list of all units that are called by this unit.
C
C   Inputs:
C	Name		Type			Description
C----------------------------------------------------------------
C	<user-specified> Ethernet device	the interface device that
C						the Ethernet will be
C						monitored from
C
C   Outputs:
C	Name		Type			Description
C----------------------------------------------------------------
C	SYS$OUTPUT:	file
C
C   NOTES
C	Limitations:
C		This program will only work when the specified device
C		is a DEC-compatible Ethernet port, although it only
C		checks that the device is a serial communications
C		device.
C	Error Handling:
C
C   Revision History:
C	date	version		programmer	SCR #
C	6-6-89	  1A		Andrew Pavlin	Initial turnover
C
C****************************************************************
      PROGRAM ETHERNET_MONITOR
C
CP DATA DECLARATIONS for ETHERNET_MONITOR
C
      IMPLICIT NONE
C
C   declare local variables
C
      CHARACTER*132 CLV_CMDLIN
      INTEGER*2 NLV_IOSB(0:3), LN
      INTEGER*4 ILV_STATUS, ILV_ITMLST(7), ILV_DEVCLASS
C
C   declare local COMMON's
C
      LOGICAL*1 LCV_EXIT
      COMMON /LCV_EXIT/ LCV_EXIT
      CHARACTER*32 CCVNAM_DEVNAM, CCVNAM_SYSNAM
      INTEGER*2 NCVNAM_DEVNAMLEN, NCVNAM_SYSNAMLEN
      COMMON /NAMES/ CCVNAM_DEVNAM, CCVNAM_SYSNAM,
     +		NCVNAM_DEVNAMLEN, NCVNAM_SYSNAMLEN
      INTEGER*4 ICVSMG_KEYID, ICVSMG_KTBLID, ICVSMG_PASTEID,
     +		ICVSMG_DSPID
      COMMON /SMG/ ICVSMG_KEYID, ICVSMG_KTBLID, ICVSMG_PASTEID,
     +		ICVSMG_DSPID
C
C   declare system procedures
C
      INTEGER*4 LIB$GET_FOREIGN, SYS$GETDVIW, SYS$SETPRV,
     +		SMG$CREATE_VIRTUAL_KEYBOARD,	SMG$CREATE_KEY_TABLE,
     +		SMG$READ_COMPOSED_LINE
      EXTERNAL	LIB$GET_FOREIGN, SYS$GETDVIW, SYS$SETPRV,
     +		SMG$CREATE_VIRTUAL_KEYBOARD,	SMG$CREATE_KEY_TABLE,
     +		SMG$READ_COMPOSED_LINE
C
C   declare system symbolic codes
C
      INCLUDE '($DCDEF)		/NOLIST'
      INCLUDE '($DVIDEF)	/NOLIST'
      INCLUDE '($PRVDEF)	/NOLIST'
      EXTERNAL	RMS$_EOF
      EXTERNAL	SMG$_EOF
      EXTERNAL	SS$_NORMAL, SS$_NOTNETDEV
C
CP BEGIN ETHERNET_MONITOR
CP.  get name of Ethernet interface
C
      ILV_STATUS=LIB$GET_FOREIGN(CLV_CMDLIN,'Ethernet_port: ',LN)
      IF(ILV_STATUS.EQ.%LOC(RMS$_EOF))THEN
	CALL EXIT
      ELSEIF(.NOT.ILV_STATUS)THEN
	CALL LIB$STOP(%VAL(ILV_STATUS))
      ENDIF
C
CP.  strip off unit number (if any)
C
      ILV_ITMLST(1)=64+JISHFT(DVI$_DEVNAM,16)
      ILV_ITMLST(2)=%LOC(CCVNAM_DEVNAM)
      ILV_ITMLST(3)=%LOC(NCVNAM_DEVNAMLEN)
      ILV_ITMLST(4)=4+JISHFT(DVI$_DEVCLASS,16)
      ILV_ITMLST(5)=%LOC(ILV_DEVCLASS)
      ILV_ITMLST(6)=0
      ILV_ITMLST(7)=0
      ILV_STATUS=SYS$GETDVIW(,
     +			     ,CLV_CMDLIN(1:LN),
     +			     ILV_ITMLST,
     +			     NLV_IOSB,,,)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$STOP(%VAL(ILV_STATUS))
      ELSEIF(.NOT.NLV_IOSB(0))THEN
	CALL LIB$STOP(%VAL(NLV_IOSB(0)))
      ELSEIF(ILV_DEVCLASS.NE.DC$_SCOM)THEN
	CALL LIB$STOP(SS$_NOTNETDEV)
      ENDIF
      NCVNAM_DEVNAMLEN=NCVNAM_DEVNAMLEN-1
      DO WHILE (NCVNAM_DEVNAMLEN.GT.0.AND.
     +		CCVNAM_DEVNAM(NCVNAM_DEVNAMLEN:NCVNAM_DEVNAMLEN).GE.'0'.AND.
     +		CCVNAM_DEVNAM(NCVNAM_DEVNAMLEN:NCVNAM_DEVNAMLEN).LE.'9')
	NCVNAM_DEVNAMLEN=NCVNAM_DEVNAMLEN-1
      ENDDO
      NCVNAM_DEVNAMLEN=NCVNAM_DEVNAMLEN+1
      CCVNAM_DEVNAM(NCVNAM_DEVNAMLEN:NCVNAM_DEVNAMLEN)=':'
C
CP.  enable PHY_IO privilege
C
      ILV_ITMLST(1)=PRV$M_PHY_IO+PRV$M_NETMBX
      ILV_ITMLST(2)=0
      ILV_STATUS=SYS$SETPRV(%VAL(1),
     +			    ILV_ITMLST,
     +			    ,)
      IF(ILV_STATUS.NE.%LOC(SS$_NORMAL))THEN
	CALL LIB$STOP(%VAL(ILV_STATUS))
      ENDIF
C
CP.  connect to and initialize Ethernet interface
C
      CALL INIT_ETHERNET
C
CP.  initialize SMG$ interface
C
      ILV_STATUS=SMG$CREATE_VIRTUAL_KEYBOARD(ICVSMG_KEYID)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$STOP(%VAL(ILV_STATUS))
      ENDIF
      ILV_STATUS=SMG$CREATE_KEY_TABLE(ICVSMG_KTBLID)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$STOP(%VAL(ILV_STATUS))
      ENDIF
C
CP.  REPEAT
C
      LCV_EXIT=.FALSE.
      DO WHILE (.NOT.LCV_EXIT)
C
CP.  .  prompt for command
C
	ILV_STATUS=SMG$READ_COMPOSED_LINE(ICVSMG_KEYID,ICVSMG_KTBLID,
     +					  CLV_CMDLIN,'ETHERMON> ',LN)
C
CP.  .  IF no command entry error THEN
C
	IF(ILV_STATUS)THEN
C
CP.  .  .  parse command and execute it
C
	  IF(LN.GT.0)THEN
	    CALL STR$UPCASE(CLV_CMDLIN(1:LN),
     +			    CLV_CMDLIN(1:LN))
	    CALL EVALUATE_COMMAND(CLV_CMDLIN(1:LN))
	  ENDIF
C
CP.  .  ELSEIF error is control-Z THEN
C
	ELSEIF(ILV_STATUS.EQ.%LOC(SMG$_EOF))THEN
C
CP.  .  .  simulate EXIT command
C
	  LCV_EXIT=.TRUE.
C
CP.  .  ELSE
C
	ELSE
C
CP.  .  .  report error
C
	  CALL LIB$STOP(%VAL(ILV_STATUS))
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  UNTIL exit command entered
C
      ENDDO
C
CP END ETHERNET_MONITOR
C
      END
      INTEGER*4 FUNCTION FIND_ADDRSLOT(SLA_ADDR,LLA_TYPE)
C
CP DATA DECLARATIONS for FIND_ADDRSLOT
C
      IMPLICIT NONE
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ADDR.INC'
C
C   declare subroutine arguments
C
      RECORD /ETH_ADR/ SLA_ADDR
      LOGICAL*1 LLA_TYPE
C
C   declare local variables
C
      INTEGER*4 ILV_COUNT
C
CP BEGIN FIND_ADDRSLOT
CP.  calculate initial hash value
C
      FIND_ADDRSLOT=IAND(SLA_ADDR.NUM(2),ICPADR_MAXADDRS)
      ILV_COUNT=0
C
CP.  block AST's from messing up the scan
C
      IF (LLA_TYPE) CALL SYS$SETAST(%VAL(0))
C
CP.  DO WHILE current slot doesn't match and current slot is used and
CP		retry count not exceeded
C
      DO WHILE ((SCVADR_ADDR(FIND_ADDRSLOT).LNUM .NE. SLA_ADDR.LNUM .OR.
     +		 SCVADR_ADDR(FIND_ADDRSLOT).NUM(2) .NE. SLA_ADDR.NUM(2))
     +    .AND. LCVADR_USED(FIND_ADDRSLOT) .AND.
     +		ILV_COUNT.LT.ICPADR_MAXADDRS/2)
C
CP.  .  step to next slot
C
	FIND_ADDRSLOT=IAND(FIND_ADDRSLOT+1,ICPADR_MAXADDRS)
	ILV_COUNT=ILV_COUNT+1
C
CP.  ENDDO
C
      ENDDO
C
CP.  IF matching slot not found THEN
C
      IF(SCVADR_ADDR(FIND_ADDRSLOT).LNUM .NE.SLA_ADDR.LNUM .OR.
     +	 SCVADR_ADDR(FIND_ADDRSLOT).NUM(2) .NE.SLA_ADDR.NUM(2))THEN
C
CP.  .  IF "create new entry" not specified or return slot not empty THEN
C
	IF(.NOT.LLA_TYPE.OR.
     +	   LCVADR_USED(FIND_ADDRSLOT))THEN
C
CP.  .  .  change return status to error
C
	  FIND_ADDRSLOT=-1
C
CP.  .  ELSE {must be "create new slot"}
C
	ELSE
C
CP.  .  .  increment table entry count
C
	  ICVADR_NUMADDRS=ICVADR_NUMADDRS+1
C
CP.  .  .  add address to table
C
	  SCVADR_ADDR(FIND_ADDRSLOT).LNUM=SLA_ADDR.LNUM
	  SCVADR_ADDR(FIND_ADDRSLOT).NUM(2)=SLA_ADDR.NUM(2)
	  LCVADR_USED(FIND_ADDRSLOT)=.TRUE.
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ENDIF
C
      ENDIF
C
CP.  unblock AST's
C
      IF (LLA_TYPE) CALL SYS$SETAST(%VAL(1))
C
CP.  RETURN
C
      RETURN
C
CP END FIND_ADDRSLOT
C
      END
      INTEGER*4 FUNCTION FIND_ADDRSLOT_AST(SLA_ADDR)
C
CP DATA DECLARATIONS for FIND_ADDRSLOT_AST
C
      IMPLICIT NONE
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ADDR.INC'
C
C   declare subroutine arguments
C
      RECORD /ETH_ADR/ SLA_ADDR
C
C   declare local variables
C
      INTEGER*4 ILV_COUNT
C
CP BEGIN FIND_ADDRSLOT_AST
CP.  calculate initial hash value
C
      FIND_ADDRSLOT_AST=IAND(SLA_ADDR.NUM(2),ICPADR_MAXADDRS)
      ILV_COUNT=0
C
CP.  DO WHILE current slot doesn't match and current slot is used and
CP		retry count not exceeded
C
      DO WHILE ((SCVADR_ADDR(FIND_ADDRSLOT_AST).LNUM .NE.
     +				SLA_ADDR.LNUM .OR.
     +		 SCVADR_ADDR(FIND_ADDRSLOT_AST).NUM(2) .NE.
     +				SLA_ADDR.NUM(2))
     +    .AND. LCVADR_USED(FIND_ADDRSLOT_AST) .AND.
     +		ILV_COUNT.LT.ICPADR_MAXADDRS/2)
C
CP.  .  step to next slot
C
	FIND_ADDRSLOT_AST=IAND(FIND_ADDRSLOT_AST+1,ICPADR_MAXADDRS)
	ILV_COUNT=ILV_COUNT+1
C
CP.  ENDDO
C
      ENDDO
C
CP.  IF matching slot not found THEN
C
      IF(SCVADR_ADDR(FIND_ADDRSLOT_AST).LNUM .NE. SLA_ADDR.LNUM .OR.
     +	 SCVADR_ADDR(FIND_ADDRSLOT_AST).NUM(2) .NE.
     +					SLA_ADDR.NUM(2))THEN
C
CP.  .  IF return slot not empty THEN
C
	IF(LCVADR_USED(FIND_ADDRSLOT_AST))THEN
C
CP.  .  .  change return status to error
C
	  FIND_ADDRSLOT_AST=-1
C
CP.  .  ELSE {must be "create new slot"}
C
	ELSE
C
CP.  .  .  increment table entry count
C
	  ICVADR_NUMADDRS=ICVADR_NUMADDRS+1
C
CP.  .  .  add address to table
C
	  SCVADR_ADDR(FIND_ADDRSLOT_AST).LNUM=SLA_ADDR.LNUM
	  SCVADR_ADDR(FIND_ADDRSLOT_AST).NUM(2)=SLA_ADDR.NUM(2)
	  LCVADR_USED(FIND_ADDRSLOT_AST)=.TRUE.
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ENDIF
C
      ENDIF
C
CP.  RETURN
C
      RETURN
C
CP END FIND_ADDRSLOT_AST
C
      END
      SUBROUTINE INIT_ETHERNET
C
CP DATA DECLARATIONS for INIT_ETHERNET
C
      IMPLICIT NONE
C
C   declare local variables
C
      CHARACTER*30 CLV_SCRATCH
      INTEGER*2 NLV_INITETH_P2(18), NLV_IOSB(0:3), LN
      INTEGER*4 ILV_STATUS, ILV_ITMLST(10), ILV_TEMP, ILV_SYSID(2),
     +		ILV_DESCR(0:1), I
C
C   declare local COMMON's
C
      CHARACTER*32 CCVNAM_DEVNAM, CCVNAM_SYSNAM
      INTEGER*2 NCVNAM_DEVNAMLEN, NCVNAM_SYSNAMLEN
      COMMON /NAMES/ CCVNAM_DEVNAM, CCVNAM_SYSNAM,
     +		NCVNAM_DEVNAMLEN, NCVNAM_SYSNAMLEN
      INCLUDE 'ETHERMON_ADDR.INC'
      INCLUDE 'ETHERMON_ETH.INC'
C
C   declare local procedures
C
      INTEGER*4 FIND_ADDRSLOT
      EXTERNAL	FIND_ADDRSLOT
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__NODECNET
C
C   declare system procedures
C
      INTEGER*4 SYS$ASSIGN, SYS$QIOW, SYS$GETSYIW
      EXTERNAL	SYS$ASSIGN, SYS$QIOW, SYS$GETSYIW
C
C   declare system symbolic codes
C
      INCLUDE '($IODEF)		/NOLIST'
      INCLUDE '($SYIDEF)	/NOLIST'
      EXTERNAL	NMA$C_PCLI_PRM, NMA$C_PCLI_MLT, NMA$C_PCLI_PTY,
     +		NMA$C_PCLI_PAD, NMA$C_PCLI_BUS, NMA$C_PCLI_BFN
      EXTERNAL	SS$_NOMORENODE
C
CP BEGIN INIT_ETHERNET
CP.  assign channel to Ethernet port device
C
      ILV_STATUS=SYS$ASSIGN(CCVNAM_DEVNAM(1:NCVNAM_DEVNAMLEN),
     +			    NCVETH_CHAN,,)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$STOP(%VAL(ILV_STATUS))
      ENDIF
C
CP.  assign channel to DECnet interface device
C
      ILV_STATUS=SYS$ASSIGN('_NET:',NCVETH_DECNETCHAN,,)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(ETH__NODECNET,,%VAL(ILV_STATUS))
	NCVETH_DECNETCHAN=0
      ENDIF
      NCVETH_NETIOSB(0)=1	! to permit first access
      LCVETH_DECNETBUSY=.FALSE.
C
CP.  set this program's data mode to null
C
      ICVETH_DISPLAYMODE=0
C
CP.  fill in pre-defined constant addresses into the address table
C
      SCVADR_ADDR(ICPADR_MAXADDRS).LNUM='FFFFFFFF'X
      SCVADR_ADDR(ICPADR_MAXADDRS).NUM(2)='FFFF'X
      ICVADR_INETADR(ICPADR_MAXADDRS)='FFFFFFFF'X
      NCVADR_MISCNAMLEN(ICPADR_MAXADDRS)=9
      LCVADR_USED(ICPADR_MAXADDRS)=.TRUE.
      CCVADR_MISCNAM(ICPADR_MAXADDRS)='Broadcast'
      SCVADR_ADDR(0).LNUM='000000CF'X
      SCVADR_ADDR(0).NUM(2)='0000'X
      NCVADR_MISCNAMLEN(0)=19
      LCVADR_USED(0)=.TRUE.
      CCVADR_MISCNAM(0)='Loopback Assistance'
      SCVADR_ADDR(1).LNUM='010000AB'X
      SCVADR_ADDR(1).NUM(2)='0000'X
      NCVADR_MISCNAMLEN(1)=24
      LCVADR_USED(1)=.TRUE.
      CCVADR_MISCNAM(1)='DEC:Dump/Load Assistance'
      SCVADR_ADDR(2).LNUM='020000AB'X
      SCVADR_ADDR(2).NUM(2)='0000'X
      NCVADR_MISCNAMLEN(2)=18
      LCVADR_USED(2)=.TRUE.
      CCVADR_MISCNAM(2)='DEC:Remote Console'
      SCVADR_ADDR(3).LNUM='030000AB'X
      SCVADR_ADDR(3).NUM(2)='0000'X
      NCVADR_MISCNAMLEN(3)=19
      LCVADR_USED(3)=.TRUE.
      CCVADR_MISCNAM(3)='DEC:Lvl 1,2 Routers'
      SCVADR_ADDR(4).LNUM='040000AB'X
      SCVADR_ADDR(4).NUM(2)='0000'X
      NCVADR_MISCNAMLEN(4)=16
      LCVADR_USED(4)=.TRUE.
      CCVADR_MISCNAM(4)='DEC:All Endnodes'
      SCVADR_ADDR(5).LNUM='000300AB'X
      SCVADR_ADDR(5).NUM(2)='0000'X
      NCVADR_MISCNAMLEN(5)=13
      LCVADR_USED(5)=.TRUE.
      CCVADR_MISCNAM(5)='DEC:LAT (old)'
      SCVADR_ADDR('300'X).LNUM='002B0009'X
      SCVADR_ADDR('300'X).NUM(2)='0300'X
      NCVADR_MISCNAMLEN('300'X)=23
      LCVADR_USED('300'X)=.TRUE.
      CCVADR_MISCNAM('300'X)='DEC:LAN Traffic Monitor'
      SCVADR_ADDR('0301'X).LNUM='002B0009'X	! fix index if enlarging
      SCVADR_ADDR('0301'X).NUM(2)='0F00'X	! address table
      NCVADR_MISCNAMLEN('0301'X)=15
      LCVADR_USED('0301'X)=.TRUE.
      CCVADR_MISCNAM('0301'X)='DEC:LAT (LANCE)'
      SCVADR_ADDR(6).LNUM='012B0009'X
      SCVADR_ADDR(6).NUM(2)='0000'X
      NCVADR_MISCNAMLEN(6)=15
      LCVADR_USED(6)=.TRUE.
      CCVADR_MISCNAM(6)='DEC:Bridge Mgmt'
      SCVADR_ADDR('0100'X).LNUM='012B0009'X
      SCVADR_ADDR('0100'X).NUM(2)='0100'X
      NCVADR_MISCNAMLEN('0100'X)=16
      LCVADR_USED('0100'X)=.TRUE.
      CCVADR_MISCNAM('0100'X)='DEC:Bridge hello'
      SCVADR_ADDR(7).LNUM='022B0009'X
      SCVADR_ADDR(7).NUM(2)='0000'X
      NCVADR_MISCNAMLEN(7)=17
      LCVADR_USED(7)=.TRUE.
      CCVADR_MISCNAM(7)='DEC:Lvl 2 Routers'
      SCVADR_ADDR(8).LNUM='00870009'X
      SCVADR_ADDR(8).NUM(2)='FFFF'X
      NCVADR_MISCNAMLEN(8)=10
      LCVADR_USED(8)=.TRUE.
      CCVADR_MISCNAM(8)='Xyplex:???'
      SCVADR_ADDR('00DE'X).LNUM='6800DD01'X
      SCVADR_ADDR('00DE'X).NUM(2)='00DE'X
      NCVADR_MISCNAMLEN('00DE'X)=10
      LCVADR_USED('00DE'X)=.TRUE.
      CCVADR_MISCNAM('00DE'X)='U-B:??? #1'
      SCVADR_ADDR('00BC'X).LNUM='A100DD01'X
      SCVADR_ADDR('00BC'X).NUM(2)='00BC'X
      NCVADR_MISCNAMLEN('00BC'X)=10
      LCVADR_USED('00BC'X)=.TRUE.
      CCVADR_MISCNAM('00BC'X)='U-B:??? #2'
      SCVADR_ADDR(9).LNUM='FF00DD01'X
      SCVADR_ADDR(9).NUM(2)='FFFF'X
      NCVADR_MISCNAMLEN(9)=10
      LCVADR_USED(9)=.TRUE.
      CCVADR_MISCNAM(9)='U-B:??? #3'
C
CP.  get system name
C
      ILV_ITMLST(1)=32+JISHFT(SYI$_NODENAME,16)
      ILV_ITMLST(2)=%LOC(CCVNAM_SYSNAM)
      ILV_ITMLST(3)=%LOC(NCVNAM_SYSNAMLEN)
      ILV_ITMLST(4)=1+JISHFT(SYI$_CLUSTER_MEMBER,16)
      ILV_ITMLST(5)=%LOC(ILV_TEMP)
      ILV_ITMLST(6)=0
      ILV_ITMLST(7)=4+JISHFT(SYI$_SCSSYSTEMID,16)
      ILV_ITMLST(8)=%LOC(ILV_SYSID(2))
      ILV_ITMLST(9)=0
      ILV_ITMLST(10)=0
      ILV_SYSID(1)='000400AA'X
      ILV_STATUS=SYS$GETSYIW(,,,ILV_ITMLST,NLV_IOSB,,)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$STOP(%VAL(ILV_STATUS))
      ELSEIF(.NOT.NLV_IOSB(0))THEN
	CALL LIB$STOP(%VAL(NLV_IOSB(0)))
      ENDIF
C
CP.  IF system name is not blank THEN
C
      IF(CCVNAM_SYSNAM(1:NCVNAM_SYSNAMLEN).NE.' ')THEN
C
CP.  .  IF DECnet is available and system ID defined THEN
C
	IF(NCVETH_DECNETCHAN.NE.0.AND.
     +	   ILV_SYSID(2).NE.0)THEN
C
CP.  .  .  add physical address to known list
C
	  I=FIND_ADDRSLOT(ILV_SYSID,.TRUE.)
	  IF(I.GE.0)THEN
	    CCVADR_DECNETNAM(I)=CCVNAM_SYSNAM(1:NCVNAM_SYSNAMLEN)
	    NCVADR_DECNETNAMLEN(I)=NCVNAM_SYSNAMLEN
	    CCVADR_MISCNAM(I)='(phy)'
	    NCVADR_MISCNAMLEN(I)=5
	  ENDIF
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ENDIF
C
      ENDIF
C
CP.  IF node is a member of a VAXcluster THEN
C
      IF(ILV_TEMP)THEN
C
CP.  .  REPEAT
C
	ILV_STATUS=1
	NLV_IOSB(0)=1
	ILV_TEMP=-1	! wildcard CSID
	DO WHILE (ILV_STATUS.AND.NLV_IOSB(0))
C
CP.  .  .  get node number and name
C
	  ILV_ITMLST(1)=30+JISHFT(SYI$_NODENAME,16)
	  ILV_ITMLST(2)=%LOC(CLV_SCRATCH)
	  ILV_ITMLST(3)=%LOC(LN)
	  ILV_ITMLST(4)=4+JISHFT(SYI$_NODE_SYSTEMID,16)
	  ILV_ITMLST(5)=%LOC(ILV_SYSID(2))
	  ILV_ITMLST(6)=0
	  ILV_ITMLST(7)=0
	  ILV_STATUS=SYS$GETSYIW(,ILV_TEMP,,ILV_ITMLST,NLV_IOSB,,)
	  IF(.NOT.ILV_STATUS.AND.
     +	     ILV_STATUS.NE.%LOC(SS$_NOMORENODE))THEN
	    CALL LIB$STOP(%VAL(ILV_STATUS))
	  ELSEIF(.NOT.NLV_IOSB(0).AND.
     +		 NLV_IOSB(0).NE.%LOC(SS$_NOMORENODE))THEN
	    CALL LIB$STOP(%VAL(NLV_IOSB(0)))
	  ENDIF
C
CP.  .  .  add entry to address table (assuming cluster DECnet is Ethernet)
C
	  IF(CLV_SCRATCH(1:LN).NE.CCVNAM_SYSNAM(1:NCVNAM_SYSNAMLEN))THEN
	    I=FIND_ADDRSLOT(ILV_SYSID,.TRUE.)
	    IF(I.GE.0)THEN
	      CCVADR_DECNETNAM(I)=CLV_SCRATCH(1:LN)
	      NCVADR_DECNETNAMLEN(I)=LN
	      CCVADR_MISCNAM(I)='(VC)'
	      NCVADR_MISCNAMLEN(I)=4
	    ENDIF
	  ENDIF
C
CP.  .  UNTIL all nodes in cluster interrogated
C
	ENDDO
C
CP.  ENDIF
C
      ENDIF
C
CP.  initialize device as promiscuous Ethernet monitor
C
      NLV_INITETH_P2(1)=%LOC(NMA$C_PCLI_PRM)
      NLV_INITETH_P2(2)=0
      NLV_INITETH_P2(3)=0
      NLV_INITETH_P2(4)=%LOC(NMA$C_PCLI_MLT)
      NLV_INITETH_P2(5)=0
      NLV_INITETH_P2(6)=0
      NLV_INITETH_P2(7)=%LOC(NMA$C_PCLI_PTY)
      NLV_INITETH_P2(8)='FFFF'X
      NLV_INITETH_P2(9)=0
      NLV_INITETH_P2(10)=%LOC(NMA$C_PCLI_PAD)
      NLV_INITETH_P2(11)=1
      NLV_INITETH_P2(12)=0
      NLV_INITETH_P2(13)=%LOC(NMA$C_PCLI_BUS)
      NLV_INITETH_P2(14)=1500
      NLV_INITETH_P2(15)=0
      NLV_INITETH_P2(16)=%LOC(NMA$C_PCLI_BFN)
      NLV_INITETH_P2(17)=12
      NLV_INITETH_P2(18)=0
      ILV_DESCR(0)=36
      ILV_DESCR(1)=%LOC(NLV_INITETH_P2)
      ILV_STATUS=SYS$QIOW(,
     +			  %VAL(NCVETH_CHAN),
     +			  %VAL(IO$_SETCHAR+IO$M_CTRL+IO$M_STARTUP),
     +			  NCVETH_IOSB,,,
     +			  ,
     +			  %REF(ILV_DESCR),,,,)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$STOP(%VAL(ILV_STATUS))
      ELSEIF(.NOT.NCVETH_IOSB(0))THEN
	CALL LIB$STOP(%VAL(NCVETH_IOSB(0)))
      ENDIF
C
CP.  get initial port status
C
      CALL GET_PORT_STATUS
C
CP.  IF system name is not blank THEN
C
      IF(CCVNAM_SYSNAM(1:NCVNAM_SYSNAMLEN).NE.' ')THEN
C
CP.  .  IF hardware Ethernet address defined and not the same as physical THEN
C
	IF(.NOT.NCVETH_HWA(0).AND.
     +	   NCVETH_HWA(1).NE.NCVETH_PHA(1).AND.
     +	   NCVETH_HWA(2).NE.NCVETH_PHA(2))THEN
C
CP.  .  .  add hardware address to known list
C
	  I=FIND_ADDRSLOT(NCVETH_HWA,.TRUE.)
	  IF(I.GE.0)THEN
	    CCVADR_DECNETNAM(I)=CCVNAM_SYSNAM(1:NCVNAM_SYSNAMLEN)
	    NCVADR_DECNETNAMLEN(I)=NCVNAM_SYSNAMLEN
	    CCVADR_MISCNAM(I)='(hdwr)'
	    NCVADR_MISCNAMLEN(I)=6
	  ENDIF
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ENDIF
C
      ENDIF
C
CP.  start read AST's
C
      CALL QUEUE_ETHER_READ
C
CP.  RETURN
C
      RETURN
C
CP END INIT_ETHERNET
C
      END
      SUBROUTINE GET_PORT_STATUS
C
CP DATA DECLARATIONS for GET_PORT_STATUS
C
      IMPLICIT NONE
C
C   declare local variables
C
      INTEGER*2 NLV_IOSB(0:3)
      INTEGER*4 ILV_STATUS, ILV_DESCRIPTOR(0:1), I, J
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ETH.INC'
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__UNKETHCHR
C
C   declare system procedures
C
      INTEGER*4 SYS$QIOW,	LIB$EXTZV
      EXTERNAL	SYS$QIOW,	LIB$EXTZV
C
C   declare system symbolic codes
C
      INCLUDE '($IODEF)'
      EXTERNAL	NMA$C_PCLI_HWA,	NMA$C_PCLI_BUS,	NMA$C_PCLI_BFN,
     +		NMA$C_PCLI_BSZ,	NMA$C_PCLI_PHA,	NMA$C_PCLI_PRM,
     +		NMA$C_PCLI_MLT,	NMA$C_PCLI_PAD,	NMA$C_PCLI_EKO,
     +		NMA$C_PCLI_DCH,	NMA$C_PCLI_CRC,	NMA$C_PCLI_PTY,
     +		NMA$C_PCLI_CON,	NMA$C_PCLI_ACC,	NMA$C_PCLI_DES,
     +		NMA$C_PCLI_MCA
      EXTERNAL	SS$_NORMAL
C
CP BEGIN GET_PORT_STATUS
CP.  build extended characteristics buffer
C
      ILV_DESCRIPTOR(0)=156
      ILV_DESCRIPTOR(1)=%LOC(NCVETH_EXTCHARBUF(1))
C
CP.  query DEUNA/DEQNA for its characteristics
C
      ILV_STATUS=SYS$QIOW(%VAL(0),		! use scratch EFN,
     +			  %VAL(NCVETH_CHAN),	! specify channel to device,
     +			  %VAL(IO$_SENSEMODE+IO$M_CTRL),! specify function,
     +			  %REF(NLV_IOSB),	! specify I/O status block,
     +			  ,,			! specify no AST,
     +			  %REF(SCVETH_CHARBUF),	! specify primary characteristics buf,
     +			  %REF(ILV_DESCRIPTOR), ! specify extended char. buf,
     +			  ,,,)			! specify no more parameters.
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ELSEIF(NLV_IOSB(0).NE.%LOC(SS$_NORMAL))THEN
	CALL LIB$SIGNAL(%VAL(NLV_IOSB(0)))
      ELSE
	NCVETH_NUMMCA=-1
	DO I=0,5
	  BCVETH_HWA(I)=-1
	  BCVETH_PHA(I)=-1
	  BCVETH_DES(I)=-1
	  BCVETH_MCA(I)=-1
	ENDDO
	ICVETH_BUS=-1
	ICVETH_BFN=-1
	ICVETH_BSZ=-1
	ICVETH_PRM=-1
	ICVETH_MLT=-1
	ICVETH_PAD=-1
	ICVETH_EKO=-1
	ICVETH_DCH=-1
	ICVETH_CRC=-1
	ICVETH_CON=-1
	ICVETH_ACC=-1
	I=1
	DO WHILE (I.LT.NLV_IOSB(1)/2)
	  IF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_HWA)+'1000'X)THEN
	    DO J=0,5
	      BCVETH_HWA(J)=BCSETH_CHARBUF(2*I+3+J)
	    ENDDO
	    I=I+5
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_BUS))THEN
	    ICVETH_BUS=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_BFN))THEN
	    ICVETH_BFN=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_BSZ))THEN
	    ICVETH_BSZ=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_PHA)+'1000'X)THEN
	    DO J=0,5
	      BCVETH_PHA(J)=BCSETH_CHARBUF(2*I+3+J)
	    ENDDO
	    I=I+5
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_PRM))THEN
	    ICVETH_PRM=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_MLT))THEN
	    ICVETH_MLT=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_PAD))THEN
	    ICVETH_PAD=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_EKO))THEN
	    ICVETH_EKO=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_DCH))THEN
	    ICVETH_DCH=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_CRC))THEN
	    ICVETH_CRC=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_PTY))THEN
	    DO J=1,4
	      BCVETH_PTY(J)=BCSETH_CHARBUF(2*I+J)
	    ENDDO
	    I=I+3
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_CON))THEN
	    ICVETH_CON=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_ACC))THEN
	    ICVETH_ACC=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_DES)+'1000'X)THEN
	    DO J=0,5
	      BCVETH_DES(J)=BCSETH_CHARBUF(2*I+3+J)
	    ENDDO
	    I=I+5
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.%LOC(NMA$C_PCLI_MCA)+'1000'X)THEN
	    NCVETH_NUMMCA=NCVETH_EXTCHARBUF(I+1)
	    DO J=0,NCVETH_NUMMCA-1
	      BCVETH_MCA(J)=BCSETH_CHARBUF(2*I+3+J)
	    ENDDO
	    I=I+2+NCVETH_NUMMCA/2
	  ELSEIF(IAND(NCVETH_EXTCHARBUF(I),'1000'X).NE.0)THEN
D	    CALL LIB$SIGNAL(ETH__UNKETHCHR,%VAL(1),
D    +				%VAL(NCVETH_EXTCHARBUF(I)))
	    I=I+2+NCVETH_EXTCHARBUF(I+1)
	  ELSE
D	    CALL LIB$SIGNAL(ETH__UNKETHCHR,%VAL(1),
D    +				%VAL(NCVETH_EXTCHARBUF(I)))
	    I=I+3
	  ENDIF
	ENDDO
      ENDIF
C
CP.  RETURN
C
      RETURN
C
CP END GET_PORT_STATUS
C
      END
      SUBROUTINE GET_PORT_COUNTERS(LLA_CLEAR)
C
CP DATA DECLARATIONS for GET_PORT_COUNTERS
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      LOGICAL*1 LLA_CLEAR
C
C   declare local variables
C
      INTEGER*2 NLV_IOSB(0:3), NLV_FUNC
      INTEGER*4 ILV_STATUS, ILV_DESCRIPTOR(0:1), I
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ETH.INC'
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__UNKETHCNT
C
C   declare system procedures
C
      INTEGER*4 SYS$QIOW,	LIB$EXTZV
      EXTERNAL	SYS$QIOW,	LIB$EXTZV
C
C   declare system symbolic codes
C
      INCLUDE '($IODEF)'
      EXTERNAL	NMA$C_CTLIN_ZER,NMA$C_CTLIN_DBR,
     +		NMA$C_CTLIN_MBL,NMA$C_CTLIN_RFL,NMA$C_CTLIN_BRC,
     +		NMA$C_CTLIN_MBY,NMA$C_CTLIN_OVR,NMA$C_CTLIN_LBE,
     +		NMA$C_CTLIN_DBS,NMA$C_CTLIN_MBS,NMA$C_CTLIN_BSM,
     +		NMA$C_CTLIN_BS1,NMA$C_CTLIN_BID,NMA$C_CTLIN_BSN,
     +		NMA$C_CTLIN_MSN,NMA$C_CTLIN_SFL,NMA$C_CTLIN_CDC,
     +		NMA$C_CTLIN_UFD,NMA$C_CTLIN_SBU,NMA$C_CTLIN_UBU
      EXTERNAL	SS$_NORMAL
C
CP BEGIN GET_PORT_COUNTERS
CP.  build extended characteristics buffer
C
      ILV_DESCRIPTOR(0)=156
      ILV_DESCRIPTOR(1)=%LOC(NCVETH_EXTCHARBUF(1))
C
CP.  query DEUNA/DEQNA for its counters
C
      IF(LLA_CLEAR)THEN
	NLV_FUNC=IO$_SENSEMODE+IO$M_CTRL+IO$M_RD_COUNT+IO$M_CLR_COUNT
      ELSE
	NLV_FUNC=IO$_SENSEMODE+IO$M_CTRL+IO$M_RD_COUNT
      ENDIF
      ILV_STATUS=SYS$QIOW(%VAL(0),		! use scratch EFN,
     +			  %VAL(NCVETH_CHAN),	! specify channel to device,
     +			  %VAL(NLV_FUNC),	! specify function,
     +			  %REF(NLV_IOSB),	! specify I/O status block,
     +			  ,,			! specify no AST,
     +			  %REF(SCVETH_CHARBUF),	! specify primary characteristics buf,
     +			  %REF(ILV_DESCRIPTOR), ! specify extended char. buf,
     +			  ,,,)			! specify no more parameters.
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ELSEIF(NLV_IOSB(0).NE.%LOC(SS$_NORMAL))THEN
	CALL LIB$SIGNAL(%VAL(NLV_IOSB(0)))
      ELSE
	ICVETH_C_DBR=0
	ICVETH_C_MBL=0
	ICVETH_C_BRC=0
	ICVETH_C_MBY=0
	ICVETH_C_DBS=0
	ICVETH_C_MBS=0
	NCVETH_C_ZER=0
	NCVETH_C_RFL(0)=0
	NCVETH_C_RFL(1)=0
	NCVETH_C_OVR=0
	NCVETH_C_LBE=0
	NCVETH_C_SFL(0)=0
	NCVETH_C_SFL(1)=0
	NCVETH_C_CDC=0
	NCVETH_C_UFD=0
	NCVETH_C_SBU=0
	NCVETH_C_UBU=0
	I=1
	DO WHILE (I.LT.NLV_IOSB(1)/2)
	  IF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_ZER)+'C000'X)THEN
	    NCVETH_C_ZER=NCVETH_EXTCHARBUF(I+1)
	    I=I+2
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_DBR)+'E000'X)THEN
	    ICVETH_C_DBR=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_MBL)+'E000'X)THEN
	    ICVETH_C_MBL=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_RFL)+'D000'X)THEN
	    NCVETH_C_RFL(0)=NCVETH_EXTCHARBUF(I+1)
	    NCVETH_C_RFL(1)=NCVETH_EXTCHARBUF(I+2)
	    I=I+3
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_BRC)+'E000'X)THEN
	    ICVETH_C_BRC=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_MBY)+'E000'X)THEN
	    ICVETH_C_MBY=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_OVR)+'C000'X)THEN
	    NCVETH_C_OVR=NCVETH_EXTCHARBUF(I+1)
	    I=I+2
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_LBE)+'C000'X)THEN
	    NCVETH_C_LBE=NCVETH_EXTCHARBUF(I+1)
	    I=I+2
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_DBS)+'E000'X)THEN
	    ICVETH_C_DBS=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_MBS)+'E000'X)THEN
	    ICVETH_C_MBS=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_BSM)+'E000'X)THEN
	    ICVETH_C_BSM=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_BS1)+'E000'X)THEN
	    ICVETH_C_BS1=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_BID)+'E000'X)THEN
	    ICVETH_C_BID=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_BSN)+'E000'X)THEN
	    ICVETH_C_BSN=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_MSN)+'E000'X)THEN
	    ICVETH_C_MSN=LIB$EXTZV(0,32,NCVETH_EXTCHARBUF(I+1))
	    I=I+3
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_SFL)+'D000'X)THEN
	    NCVETH_C_SFL(0)=NCVETH_EXTCHARBUF(I+1)
	    NCVETH_C_SFL(1)=NCVETH_EXTCHARBUF(I+2)
	    I=I+3
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_CDC)+'C000'X)THEN
	    NCVETH_C_CDC=NCVETH_EXTCHARBUF(I+1)
	    I=I+2
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_UFD)+'C000'X)THEN
	    NCVETH_C_UFD=NCVETH_EXTCHARBUF(I+1)
	    I=I+2
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_SBU)+'C000'X)THEN
	    NCVETH_C_SBU=NCVETH_EXTCHARBUF(I+1)
	    I=I+2
	  ELSEIF(JZEXT(NCVETH_EXTCHARBUF(I)).EQ.
     +			%LOC(NMA$C_CTLIN_UBU)+'C000'X)THEN
	    NCVETH_C_UBU=NCVETH_EXTCHARBUF(I+1)
	    I=I+2
	  ELSEIF(NCVETH_EXTCHARBUF(I).EQ.0)THEN
	    I=NLV_IOSB(1)/2 + 1
	  ELSEIF(IAND(NCVETH_EXTCHARBUF(I),'F000'X).EQ.'C000'X)THEN
D	    CALL LIB$SIGNAL(ETH__UNKETHCNT,%VAL(1),
D    +				%VAL(NCVETH_EXTCHARBUF(I)))
	    I=I+2
	  ELSEIF(IAND(NCVETH_EXTCHARBUF(I),'F000'X).EQ.'D000'X.OR.
     +		 IAND(NCVETH_EXTCHARBUF(I),'F000'X).EQ.'E000'X)THEN
D	    CALL LIB$SIGNAL(ETH__UNKETHCNT,%VAL(1),
D    +				%VAL(NCVETH_EXTCHARBUF(I)))
	    I=I+3
	  ELSE
	    CALL LIB$STOP(ETH__UNKETHCNT,%VAL(1),
     +				%VAL(NCVETH_EXTCHARBUF(I)))
	  ENDIF
	ENDDO
      ENDIF
C
CP.  RETURN
C
      RETURN
C
CP END GET_PORT_COUNTERS
C
      END
      SUBROUTINE QUEUE_ETHER_READ
C
CP DATA DECLARATIONS for QUEUE_ETHER_READ
C
      IMPLICIT NONE
C
C   declare local variables
C
      INTEGER*4 ILV_STATUS
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ETH.INC'
C
C   declare local procedures
C
      EXTERNAL	ETHER_READ
C
C   declare system procedures
C
      INTEGER*4 SYS$QIO
      EXTERNAL	SYS$QIO
C
C   declare system symbolic codes
C
      EXTERNAL	IO$_READVBLK
C
CP BEGIN QUEUE_ETHER_READ
CP.  zero the packet header buffer's protocol area
C
      SCVETH_ADDRBUF.CLR_PROT=0
C
CP.  queue another read request to the Ethernet port
C
      ILV_STATUS=SYS$QIO(%VAL(0),		! use scratch EFN,
     +			 %VAL(NCVETH_CHAN),	! specify channel to device,
     +			 IO$_READVBLK,		! specify function,
     +			 %REF(NCVETH_IOSB),	! specify I/O status block,
     +			 ETHER_READ,,		! specify AST,
     +			 %REF(BCVETH_RCVBUF),	! specify buffer,
     +			 %VAL(1500),,,		! specify buffer size,
     +			 %REF(SCVETH_ADDRBUF),)	! specify no more parameters.
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
C
CP.  RETURN
C
      RETURN
C
CP END QUEUE_ETHER_READ
C
      END
      SUBROUTINE ETHER_READ
C
CP DATA DECLARATIONS for ETHER_READ
C
      IMPLICIT NONE
C
C   declare local variables
C
      CHARACTER*23 CLV_TIME
      CHARACTER*40 CLV_SCR
      INTEGER*2 LN
      INTEGER*4 I, J, QLV_TIME(2), ILV_NFBDSC(0:1), ILV_KEYDSC(0:1),
     +		ILV_BUFDSC(0:1), ILV_KEY(0:1), ILV_STATUS
      LOGICAL*1 LLV_REJECT
      STRUCTURE /ARPMSG/
	UNION
	  MAP
	    CHARACTER*28 BUF
	  END MAP
	  MAP
	    INTEGER*2	HW_SPACE
	    INTEGER*2	PROT_SPACE
	    BYTE	H_LEN
	    BYTE	P_LEN
	    INTEGER*2	OPCODE
	    INTEGER*2	S_HWA(0:2)
	    UNION
	      MAP
		BYTE	S_PTA(0:3)
	      END MAP
	      MAP
		INTEGER*4 LS_PTA
	      END MAP
	    END UNION
	    INTEGER*2	T_HWA(0:2)
	    UNION
	      MAP
		BYTE	T_PTA(0:3)
	      END MAP
	      MAP
		INTEGER*4 LT_PTA
	      END MAP
	    END UNION
	  END MAP
	END UNION
      END STRUCTURE ! /ARPMSG/
      RECORD /ARPMSG/ SLV_ARP
      STRUCTURE /LAVC_ARP/
	UNION
	 MAP
	  CHARACTER*56 BUF
	 END MAP
	 MAP
	  INTEGER*2 PAD_LEN
	  UNION
	    MAP
	      INTEGER*2 DEST(0:2)
	    END MAP
	    MAP
	      INTEGER*4 LDEST
	      INTEGER*2 LDEST4
	    END MAP
	  END UNION
	  INTEGER*2 DUMMY1
	  UNION
	    MAP
	      INTEGER*2 SRC(0:2)
	    END MAP
	    MAP
	      INTEGER*4 LSRC
	      INTEGER*2 LSRC4
	    END MAP
	  END UNION
	  INTEGER*4 DUMMY2(0:1)
	  INTEGER*2 DUMMY25
	  BYTE	NAMELEN
	  CHARACTER*6 NAME
	  INTEGER*2 %FILL
	  BYTE	DUMMY3(0:4)
	  INTEGER*4 %FILL(2)
	  INTEGER*4 DUMMY4
	  INTEGER*4 NODE_CLASS
	 END MAP
	END UNION
      END STRUCTURE ! /LAVC_ARP/
      RECORD /LAVC_ARP/ SLV_LAVC
      STRUCTURE /IP/
	UNION
	  MAP
	    CHARACTER*20 BUF
	  END MAP
	  MAP
	    BYTE	VERS_IHL
	    BYTE	SVCTYP
	    INTEGER*2	LENGTH
	    INTEGER*2	ID
	    INTEGER*2	FRAG_OFF
	    BYTE	TTL
	    BYTE	PROTOCOL
	    INTEGER*2	CHECKSUM
	    UNION
	      MAP
		BYTE	SRC(0:3)
		BYTE	DEST(0:3)
	      END MAP
	      MAP
		INTEGER*4 LSRC
		INTEGER*4 LDEST
	      END MAP
	    END UNION
	  END MAP
	END UNION
      END STRUCTURE ! /IP/
      RECORD /IP/ SLV_IP
      STRUCTURE /NFB/
	BYTE		FCT
	BYTE		FLAGS
	BYTE		DATABASE
	BYTE		OPER
	INTEGER*4	SRCH_KEY
	INTEGER*4	SRCH2_KEY
	INTEGER*2	DUMMY
	INTEGER*2	CELL_SIZE
	INTEGER*4	ITMLST(2)
      END STRUCTURE ! /NFB/
      RECORD /NFB/ SLV_NFB
      PARAMETER NFB$C_FC_SHOW=34,
     +		NFB$M_NOUPD='4'X,
     +		NFB$C_DB_NDI=2,
     +		NFB$C_OP_EQL=0,
     +		NFB$C_WILDCARD=1,
     +		NFB$C_NDI_ADD=33619986,
     +		NFB$C_NDI_NNA=33685571
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ADDR.INC'
      INCLUDE 'ETHERMON_ETH.INC'
      INCLUDE 'ETHERMON_FILTER.INC'
      INCLUDE 'ETHERMON_STATS.INC'
C
C   declare local procedures
C
      INTEGER*4 FIND_ADDRSLOT_AST
      EXTERNAL	FIND_ADDRSLOT_AST, NETACP_QUERY
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__CMPETHRD, ETH__ADRTBLOVF, ETH__PROTBLOVF,
     +		ETH__QIONETACP
C
C   declare system procedures
C
      INTEGER*4 SYS$QIO
      EXTERNAL	SYS$QIO
C
C   declare system symbolic codes
C
      EXTERNAL	IO$_ACPCONTROL
C
CP BEGIN ETHER_READ
CP.  check status of read
C
      IF(.NOT.NCVETH_IOSB(0))THEN
	CALL LIB$SIGNAL(ETH__CMPETHRD,,%VAL(NCVETH_IOSB(0)))
      ENDIF
C
CP.  IF automatic address determination disabled THEN
C
      IF(.NOT.LCVETH_AUTOADDR)THEN
C
CP.  .  do nothing
C
	CONTINUE
C
CP.  ELSEIF protocol type is DECnet routers (60-03) THEN
C
      ELSEIF(SCVETH_ADDRBUF.W_PROT .EQ.'0360'X)THEN
C
CP.  .  IF we have access to the DECnet ACP and request is not outstanding THEN
C
	IF(NCVETH_DECNETCHAN.NE.0.AND.
     +	   .NOT.LCVETH_DECNETBUSY)THEN
C
CP.  .  .  IF source address is a DECnet address THEN
C
	  IF(SCVETH_ADDRBUF.LSRC .EQ.'000400AA'X)THEN
C
CP.  .  .  .  check if address already in address table
C
	    J=FIND_ADDRSLOT_AST(SCVETH_ADDRBUF.LSRC)
C
CP.  .  .  .  IF entry needs to be made for this address THEN
C
	    IF(J.GE.0.AND.
     +	       NCVADR_DECNETNAMLEN(J).EQ.0)THEN
C
CP.  .  .  .  .  ask NETACP for node name
C
	      LCVETH_DECNETBUSY=.TRUE.	! mark that no more $QIO's can be done
	      ILV_NFBDSC(0)=24
	      ILV_NFBDSC(1)=%LOC(SLV_NFB)
	      SLV_NFB.FCT=NFB$C_FC_SHOW
	      SLV_NFB.FLAGS=NFB$M_NOUPD
	      SLV_NFB.DATABASE=NFB$C_DB_NDI
	      SLV_NFB.OPER=NFB$C_OP_EQL
	      SLV_NFB.SRCH_KEY=NFB$C_NDI_ADD
	      SLV_NFB.SRCH2_KEY=NFB$C_WILDCARD
	      SLV_NFB.DUMMY=0
	      SLV_NFB.CELL_SIZE=0
	      SLV_NFB.ITMLST(1)=NFB$C_NDI_NNA
	      SLV_NFB.ITMLST(2)=0
	      ILV_BUFDSC(0)=LEN(CCVETH_NETACPBUF)
	      ILV_BUFDSC(1)=%LOC(CCVETH_NETACPBUF)
	      ILV_KEYDSC(0)=8
	      ILV_KEYDSC(1)=%LOC(ILV_KEY)
	      ILV_KEY(0)=0
	      ILV_KEY(1)=ZEXT(SCVETH_ADDRBUF.LSRC4)
	      ILV_STATUS=SYS$QIO(%VAL(0),		! use scratch EFN,
     +			 %VAL(NCVETH_DECNETCHAN), ! specify channel to device,
     +			 IO$_ACPCONTROL,	! specify function,
     +			 %REF(NCVETH_NETIOSB),	! specify I/O status block,
     +			 NETACP_QUERY,		! specify AST,
     +			 %VAL(J),		! specify table slot# as param,
     +			 %REF(ILV_NFBDSC),	! specify NFB,
     +			 %REF(ILV_KEYDSC),,	! specify search key,
     +			 %REF(ILV_BUFDSC),,)	! specify return buffer.
	      IF(.NOT.ILV_STATUS)THEN
		CALL LIB$SIGNAL(ETH__QIONETACP,,%VAL(ILV_STATUS))
	      ENDIF
C
CP.  .  .  .  ENDIF
C
	    ENDIF
C
CP.  .  .  ENDIF
C
	  ENDIF
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ELSEIF protocol type is DEC LAVC (60-07) THEN
C
      ELSEIF(SCVETH_ADDRBUF.W_PROT .EQ.'0760'X)THEN
C
CP.  .  IF message addressed to LAVC Node ID for some LAVC group THEN
C
	IF(SCVETH_ADDRBUF.LDEST .EQ.'010400AB'X.AND.
     +	   NCVETH_IOSB(1).GE.62)THEN
C
CP.  .  .  copy message into buffer
C
	  SLV_LAVC.BUF=CCVETH_RCVBUF(1:56)
C
CP.  .  .  IF Ethernet addresses match and other message stuff looks "OK"(?) THEN
C
	  IF(SLV_LAVC.LSRC .EQ.SCVETH_ADDRBUF.LSRC .AND.
     +	     SLV_LAVC.LSRC4 .EQ.SCVETH_ADDRBUF.LSRC4 .AND.
     +	     SLV_LAVC.PAD_LEN .EQ.60.AND.
     +	     SLV_LAVC.DUMMY2(0) .EQ.'000800A0'X)THEN
C
CP.  .  .  .  try to insert group multicast into address table
C
	    J=FIND_ADDRSLOT_AST(SCVETH_ADDRBUF.W_DEST)
	    IF(J.GE.0.AND.
     +	       NCVADR_MISCNAMLEN(J).LT.LEN(CCVADR_MISCNAM(0))-1.AND.
     +	       (NCVADR_MISCNAMLEN(J).EQ.0.OR.
     +	        INDEX(CCVADR_MISCNAM(J)(1:NCVADR_MISCNAMLEN(J)),
     +		      'DEC:LAVC ID').EQ.0))THEN
	      IF(NCVADR_MISCNAMLEN(J).NE.0)THEN
		NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+1
		CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J):)=','
	      ENDIF
	      CALL SYS$FAO('DEC:LAVC ID(!UW)',LN,
     +			   CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J)+1:),
     +			   %VAL(SCVETH_ADDRBUF.W_DEST(2)))
	      NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+LN
	    ENDIF
C
CP.  .  .  .  try to insert source into address table
C
	    J=FIND_ADDRSLOT_AST(SLV_LAVC.SRC)
	    IF(J.GE.0.AND.
     +	       (NCVADR_DECNETNAMLEN(J).EQ.0.OR.
     +		INDEX(CCVADR_DECNETNAM(J)(1:NCVADR_DECNETNAMLEN(J)),
     +		      '.').NE.0))THEN
	      LN=6
	      DO WHILE (LN.GE.1.AND.
     +			SLV_LAVC.NAME(LN:LN) .EQ.' ')
		LN=LN-1
	      ENDDO
	      CCVADR_DECNETNAM(J)=SLV_LAVC.NAME(1:LN)
	      NCVADR_DECNETNAMLEN(J)=LN
	    ENDIF
	    IF(J.GE.0.AND.
     +	       NCVADR_MISCNAMLEN(J).LT.LEN(CCVADR_MISCNAM(0))-1.AND.
     +	       (NCVADR_MISCNAMLEN(J).EQ.0.OR.
     +	        INDEX(CCVADR_MISCNAM(J)(1:NCVADR_MISCNAMLEN(J)),
     +		      'LVC').EQ.0))THEN
	      IF(NCVADR_MISCNAMLEN(J).NE.0)THEN
		NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+1
		CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J):)=','
	      ENDIF
	      IF(SLV_LAVC.NODE_CLASS .EQ.0)THEN
		CALL SYS$FAO('(bootLVC!UW)',LN,
     +			     CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J)+1:),
     +			     %VAL(SCVETH_ADDRBUF.W_DEST(2)))
	      ELSEIF(SLV_LAVC.NODE_CLASS .EQ.2)THEN
		CALL SYS$FAO('(satLVC!UW)',LN,
     +			     CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J)+1:),
     +			     %VAL(SCVETH_ADDRBUF.W_DEST(2)))
	      ELSE	! unknown type
		CALL SYS$FAO('(???LVC!UW)',LN,
     +			     CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J)+1:),
     +			     %VAL(SCVETH_ADDRBUF.W_DEST(2)))
	      ENDIF
	      NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+LN
	    ENDIF
C
CP.  .  .  ENDIF
C
	  ENDIF
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ELSEIF protocol type is DOD Internet (08-00) THEN
C
      ELSEIF(SCVETH_ADDRBUF.W_PROT .EQ.'0008'X)THEN
C
CP.  .  IF message is Ethernet and long enough THEN
C
	IF(SCVETH_ADDRBUF.ECTL .EQ.0.AND.
     +	   NCVETH_IOSB(1).GE.20)THEN
C
CP.  .  .  copy IP message into buffer
C
	  SLV_IP.BUF=CCVETH_RCVBUF(1:20)
C
CP.  .  .  IF message header is a recognized IP header THEN
C
	  IF(SLV_IP.VERS_IHL .GE.'45'X.AND.
     +	     SLV_IP.VERS_IHL .LE.'4F'X)THEN
C
CP.  .  .  .  try to insert source into address table
C
	    J=FIND_ADDRSLOT_AST(SCVETH_ADDRBUF.W_SRC)
	    IF(J.GE.0.AND.
     +	       NCVADR_INETNAMLEN(J).EQ.0)THEN
	      CALL SYS$FAO('!UB.!UB.!UB.!UB',NCVADR_INETNAMLEN(J),
     +			   CCVADR_INETNAM(J),
     +			   %VAL(SLV_IP.SRC(0)),%VAL(SLV_IP.SRC(1)),
     +			   %VAL(SLV_IP.SRC(2)),%VAL(SLV_IP.SRC(3)))
	      ICVADR_INETADR(J)=SLV_IP.LSRC
	    ENDIF
C
CP.  .  .  .  try to insert destination into address table
C
	    J=FIND_ADDRSLOT_AST(SCVETH_ADDRBUF.W_DEST)
	    IF(J.GE.0.AND.
     +	       NCVADR_INETNAMLEN(J).EQ.0)THEN
	      CALL SYS$FAO('!UB.!UB.!UB.!UB',NCVADR_INETNAMLEN(J),
     +			   CCVADR_INETNAM(J),
     +			   %VAL(SLV_IP.DEST(0)),%VAL(SLV_IP.DEST(1)),
     +			   %VAL(SLV_IP.DEST(2)),%VAL(SLV_IP.DEST(3)))
	      ICVADR_INETADR(J)=SLV_IP.LDEST
	    ENDIF
C
CP.  .  .  ENDIF
C
	  ENDIF
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ELSEIF protocol type is ARP (08-06) or RARP (80-35) THEN
C
      ELSEIF(SCVETH_ADDRBUF.W_PROT .EQ.'0608'X.OR.
     +	     SCVETH_ADDRBUF.W_PROT .EQ.'3580'X)THEN
C
CP.  .  IF message is Ethernet type and long enough THEN
C
	IF(SCVETH_ADDRBUF.ECTL .EQ.0.AND.
     +	   NCVETH_IOSB(1).GE.28)THEN
C
CP.  .  .  copy ARP/RARP message into buffer
C
	  SLV_ARP.BUF=CCVETH_RCVBUF(1:28)
C
CP.  .  .  IF protocol address space specified in msg is DOD Internet THEN
C
	  IF(SLV_ARP.PROT_SPACE .EQ.'0008'X.AND.
     +	     SLV_ARP.H_LEN .EQ.6.AND.
     +	     SLV_ARP.P_LEN .EQ.4)THEN
C
CP.  .  .  .  IF message is ARP request, or ARP or RARP reply THEN
C
	    IF(SLV_ARP.OPCODE .EQ.'0100'X.OR.
     +	       SLV_ARP.OPCODE .EQ.'0200'X.OR.
     +	       SLV_ARP.OPCODE .EQ.'0400'X)THEN
C
CP.  .  .  .  .  try to insert sender into address table
C
	      J=FIND_ADDRSLOT_AST(SLV_ARP.S_HWA)
	      IF(J.GE.0.AND.
     +		 NCVADR_INETNAMLEN(J).EQ.0)THEN
		CALL SYS$FAO('!UB.!UB.!UB.!UB',NCVADR_INETNAMLEN(J),
     +			     CCVADR_INETNAM(J),
     +			     %VAL(SLV_ARP.S_PTA(0)),%VAL(SLV_ARP.S_PTA(1)),
     +			     %VAL(SLV_ARP.S_PTA(2)),%VAL(SLV_ARP.S_PTA(3)))
		ICVADR_INETADR(J)=SLV_ARP.LS_PTA
	      ENDIF
C
CP.  .  .  .  ENDIF
C
	    ENDIF
C
CP.  .  .  .  IF message is ARP or RARP reply THEN
C
	    IF(SLV_ARP.OPCODE .EQ.'0200'X.OR.
     +	       SLV_ARP.OPCODE .EQ.'0400'X)THEN
C
CP.  .  .  .  .  try to insert target into address table
C
	      J=FIND_ADDRSLOT_AST(SLV_ARP.T_HWA)
	      IF(J.GE.0.AND.
     +		 NCVADR_INETNAMLEN(J).EQ.0)THEN
		CALL SYS$FAO('!UB.!UB.!UB.!UB',NCVADR_INETNAMLEN(J),
     +			     CCVADR_INETNAM(J),
     +			     %VAL(SLV_ARP.T_PTA(0)),%VAL(SLV_ARP.T_PTA(1)),
     +			     %VAL(SLV_ARP.T_PTA(2)),%VAL(SLV_ARP.T_PTA(3)))
		ICVADR_INETADR(J)=SLV_ARP.LT_PTA
	      ENDIF
C
CP.  .  .  .  ENDIF
C
	    ENDIF
C
CP.  .  .  ENDIF
C
	  ENDIF
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ELSEIF protocol type is Xyplex (08-88) THEN
C
      ELSEIF(SCVETH_ADDRBUF.W_PROT .EQ.'8808'X)THEN
C
CP.  .  IF source Ethernet address looks "OK"(?) THEN
C
	IF(SCVETH_ADDRBUF.LSRC .EQ.'00870008'X)THEN
C
CP.  .  .  try to insert source into address table
C
	  J=FIND_ADDRSLOT_AST(SCVETH_ADDRBUF.W_SRC)
	  IF(J.GE.0.AND.
     +	     NCVADR_MISCNAMLEN(J).LT.LEN(CCVADR_MISCNAM(0))-1.AND.
     +	     (NCVADR_MISCNAMLEN(J).EQ.0.OR.
     +	      INDEX(CCVADR_MISCNAM(J)(1:NCVADR_MISCNAMLEN(J)),
     +		    'Xyplex').EQ.0))THEN
	    IF(NCVADR_MISCNAMLEN(J).NE.0)THEN
	      NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+1
	      CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J):)=','
	    ENDIF
	    ICVADR_SERNUM(J)=JZEXT(SCVETH_ADDRBUF.SRC(4))*256+
     +			     JZEXT(SCVETH_ADDRBUF.SRC(5))
	    CALL SYS$FAO('Xyplex SN#!UW',LN,
     +			 CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J)+1:),
     +			 %VAL(ICVADR_SERNUM(J)))
	    NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+LN
	  ENDIF
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ELSEIF protocol type is XEROX NS IDP (06-00) THEN
C
      ELSEIF(SCVETH_ADDRBUF.W_PROT .EQ.'0006'X)THEN
C
CP.  .  IF source Ethernet address looks like old Ungermann-Bass THEN
C
	IF(SCVETH_ADDRBUF.W_SRC(0) .EQ.'DD00'X.AND.
     +	   SCVETH_ADDRBUF.SRC(2) .EQ.'00'X)THEN
C
CP.  .  .  try to insert source into address table
C
	  J=FIND_ADDRSLOT_AST(SCVETH_ADDRBUF.W_SRC)
	  IF(J.GE.0.AND.
     +	     NCVADR_MISCNAMLEN(J).LT.LEN(CCVADR_MISCNAM(0))-1.AND.
     +	     (NCVADR_MISCNAMLEN(J).EQ.0.OR.
     +	      INDEX(CCVADR_MISCNAM(J)(1:NCVADR_MISCNAMLEN(J)),
     +		    'U-B#').EQ.0))THEN
	    IF(NCVADR_MISCNAMLEN(J).NE.0)THEN
	      NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+1
	      CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J):)=','
	    ENDIF
	    ICVADR_SERNUM(J)=JZEXT(SCVETH_ADDRBUF.SRC(3))*256+
     +			     JZEXT(SCVETH_ADDRBUF.SRC(4))
	    CALL SYS$FAO('U-B#!UL!AD',LN,
     +			 CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J)+1:),
     +			 %VAL(ICVADR_SERNUM(J)),%VAL(1),
     +			 %REF(ICHAR('A')+ZEXT(SCVETH_ADDRBUF.SRC(5))))
	    NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+LN
	  ENDIF
C
CP.  .  ELSEIF source Ethernet address looks like new Ungermann-Bass THEN
C
	ELSEIF(SCVETH_ADDRBUF.W_SRC(0) .EQ.'DD00'X.AND.
     +	       SCVETH_ADDRBUF.SRC(2) .EQ.'01'X)THEN
C
CP.  .  .  try to insert source into address table
C
	  J=FIND_ADDRSLOT_AST(SCVETH_ADDRBUF.W_SRC)
	  IF(J.GE.0.AND.
     +	     NCVADR_MISCNAMLEN(J).LT.LEN(CCVADR_MISCNAM(0))-1.AND.
     +	     (NCVADR_MISCNAMLEN(J).EQ.0.OR.
     +	      INDEX(CCVADR_MISCNAM(J)(1:NCVADR_MISCNAMLEN(J)),
     +		    'U-B#').EQ.0))THEN
	    IF(NCVADR_MISCNAMLEN(J).NE.0)THEN
	      NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+1
	      CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J):)=','
	    ENDIF
	    ICVADR_SERNUM(J)=JZEXT(SCVETH_ADDRBUF.SRC(3))*65536+
     +			     JZEXT(SCVETH_ADDRBUF.SRC(4))*256+
     +			     JZEXT(SCVETH_ADDRBUF.SRC(5))
	    CALL SYS$FAO('U-B#!UL',LN,
     +			 CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J)+1:),
     +			 %VAL(ICVADR_SERNUM(J)))
	    NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+LN
	  ENDIF
C
CP.  .  ELSEIF source Ethernet address looks like token-ring Ungermann-Bass THEN
C
	ELSEIF(SCVETH_ADDRBUF.W_SRC(0) .EQ.'DD00'X.AND.
     +	       SCVETH_ADDRBUF.SRC(2) .EQ.'0F'X)THEN
C
CP.  .  .  try to insert source into address table
C
	  J=FIND_ADDRSLOT_AST(SCVETH_ADDRBUF.W_SRC)
	  IF(J.GE.0.AND.
     +	     NCVADR_MISCNAMLEN(J).LT.LEN(CCVADR_MISCNAM(0))-1.AND.
     +	     (NCVADR_MISCNAMLEN(J).EQ.0.OR.
     +	      INDEX(CCVADR_MISCNAM(J)(1:NCVADR_MISCNAMLEN(J)),
     +		    'U-B tr#').EQ.0))THEN
	    IF(NCVADR_MISCNAMLEN(J).NE.0)THEN
	      NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+1
	      CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J):)=','
	    ENDIF
	    ICVADR_SERNUM(J)=JZEXT(SCVETH_ADDRBUF.SRC(3))*65536+
     +			     JZEXT(SCVETH_ADDRBUF.SRC(4))*256+
     +			     JZEXT(SCVETH_ADDRBUF.SRC(5))
	    CALL SYS$FAO('U-B tr#!UL',LN,
     +			 CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J)+1:),
     +			 %VAL(ICVADR_SERNUM(J)))
	    NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+LN
	  ENDIF
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ELSEIF protocol type is DEC Bridge (80-38) THEN
C
      ELSEIF(SCVETH_ADDRBUF.W_PROT .EQ.'3880'X)THEN
C
CP.  .  IF source Ethernet address looks like a DEC Bridge THEN
C
	IF(SCVETH_ADDRBUF.W_SRC(0) .EQ.'0008'X.AND.
     +	   SCVETH_ADDRBUF.SRC(2) .EQ.'2B'X)THEN
C
CP.  .  .  try to insert source into address table
C
	  J=FIND_ADDRSLOT_AST(SCVETH_ADDRBUF.W_SRC)
	  IF(J.GE.0.AND.
     +	     NCVADR_MISCNAMLEN(J).LT.LEN(CCVADR_MISCNAM(0))-1.AND.
     +	     (NCVADR_MISCNAMLEN(J).EQ.0.OR.
     +	      INDEX(CCVADR_MISCNAM(J)(1:NCVADR_MISCNAMLEN(J)),
     +		    'DECbrdg#').EQ.0))THEN
	    IF(NCVADR_MISCNAMLEN(J).NE.0)THEN
	      NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+1
	      CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J):)=','
	    ENDIF
	    ICVADR_SERNUM(J)=JZEXT(SCVETH_ADDRBUF.W_SRC(2))*256+
     +			     JZEXT(SCVETH_ADDRBUF.SRC(3))
	    CALL SYS$FAO('DECbrdg#!UL',LN,
     +			 CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J)+1:),
     +			 %VAL(ICVADR_SERNUM(J)))
	    NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+LN
	  ENDIF
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ELSEIF protocol type is DEC LAT (60-04) THEN
C
      ELSEIF(SCVETH_ADDRBUF.W_PROT .EQ.'0460'X)THEN
C
CP.  .  IF source Ethernet address looks like a DECserver or DECbridge THEN
C
	IF(SCVETH_ADDRBUF.W_SRC(0) .EQ.'0008'X.AND.
     +	   SCVETH_ADDRBUF.SRC(2) .EQ.'2B'X)THEN
C
CP.  .  .  try to insert source into address table
C
	  J=FIND_ADDRSLOT_AST(SCVETH_ADDRBUF.W_SRC)
	  IF(J.GE.0.AND.
     +	     NCVADR_MISCNAMLEN(J).LT.LEN(CCVADR_MISCNAM(0))-1.AND.
     +	     (NCVADR_MISCNAMLEN(J).EQ.0.OR.
     +	      (INDEX(CCVADR_MISCNAM(J)(1:NCVADR_MISCNAMLEN(J)),
     +		     'DECserv#').EQ.0.AND.
     +	       INDEX(CCVADR_MISCNAM(J)(1:NCVADR_MISCNAMLEN(J)),
     +		     'DECbrdg#').EQ.0)))THEN
	    IF(NCVADR_MISCNAMLEN(J).NE.0)THEN
	      NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+1
	      CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J):)=','
	    ENDIF
	    ICVADR_SERNUM(J)=JZEXT(SCVETH_ADDRBUF.W_SRC(2))*256+
     +			     JZEXT(SCVETH_ADDRBUF.SRC(3))
	    CALL SYS$FAO('DECserv#!UL',LN,
     +			 CCVADR_MISCNAM(J)(NCVADR_MISCNAMLEN(J)+1:),
     +			 %VAL(ICVADR_SERNUM(J)))
	    NCVADR_MISCNAMLEN(J)=NCVADR_MISCNAMLEN(J)+LN
	  ENDIF
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ENDIF
C
      ENDIF
C
CP.  IF non-null display mode THEN
C
      IF(ICVETH_DISPLAYMODE.NE.0)THEN
C
CP.  .  add statistics to pre-filter counts
C
	ICVSTT_BYTES_S_PRE(ICVSTT_BUFNUM)=
     +			ICVSTT_BYTES_S_PRE(ICVSTT_BUFNUM)+NCVETH_IOSB(1)
	ICVSTT_MSGS_S_PRE(ICVSTT_BUFNUM)=
     +			ICVSTT_MSGS_S_PRE(ICVSTT_BUFNUM)+1
C
CP.  .  DO WHILE end of filter list not reached and message not rejected
C
	I=0
 	LLV_REJECT=.FALSE.
	IF(LCVFLT_ENB_SIZE .AND.		! must be in size range
     +	   (NCVETH_IOSB(1).LT.NCVFLT_MINSIZE .OR.
     +	    NCVETH_IOSB(1).GT.NCVFLT_MAXSIZE))THEN
	  LLV_REJECT=.TRUE.
	ENDIF
	DO WHILE (I.LE.ICVFLT_NUMFILTERS.AND.
     +		  .NOT.LLV_REJECT)
C
CP.  .  .  check if message is acceptable under current filters
C
	  IF(.NOT. SCVFLT_FILTERS(I).ENABLE)THEN	! enable must be=1(odd) 
	    I=I+1
	  ELSEIF(((SCVFLT_FILTERS(I).PROT_ACT .EQ.0.OR.
     +	      (SCVFLT_FILTERS(I).PROT_ACT .EQ.1.AND.
     +	       SCVFLT_FILTERS(I).W_PROTOCOL .NE.
     +				SCVETH_ADDRBUF.W_PROT).OR.
     +	      (SCVFLT_FILTERS(I).PROT_ACT .EQ.2.AND.
     +	       SCVFLT_FILTERS(I).W_PROTOCOL .EQ.
     +				SCVETH_ADDRBUF.W_PROT)).AND.
     +	     (SCVFLT_FILTERS(I).SRC_ACT .EQ.0.OR.
     +	      (SCVFLT_FILTERS(I).SRC_ACT .EQ.1.AND.
     +	       (SCVFLT_FILTERS(I).LSOURCE .NE.
     +			SCVETH_ADDRBUF.LSRC .OR.
     +	        SCVFLT_FILTERS(I).LSOURCE4 .NE.
     +			SCVETH_ADDRBUF.LSRC4)).OR.
     +	      (SCVFLT_FILTERS(I).SRC_ACT .EQ.2.AND.
     +	       SCVFLT_FILTERS(I).LSOURCE .EQ.
     +			SCVETH_ADDRBUF.LSRC .AND.
     +	       SCVFLT_FILTERS(I).LSOURCE4 .EQ.
     +			SCVETH_ADDRBUF.LSRC4)).AND.
     +	     (SCVFLT_FILTERS(I).DST_ACT .EQ.0.OR.
     +	      (SCVFLT_FILTERS(I).DST_ACT .EQ.1.AND.
     +	       (SCVFLT_FILTERS(I).LDESTINATION .NE.
     +			SCVETH_ADDRBUF.LDEST .OR.
     +	        SCVFLT_FILTERS(I).LDESTINATION4 .NE.
     +			SCVETH_ADDRBUF.LDEST4)).OR.
     +	      (SCVFLT_FILTERS(I).DST_ACT .EQ.2.AND.
     +	       SCVFLT_FILTERS(I).LDESTINATION .EQ.
     +			SCVETH_ADDRBUF.LDEST .AND.
     +	       SCVFLT_FILTERS(I).LDESTINATION4 .EQ.
     +			SCVETH_ADDRBUF.LDEST4))))THEN
	    I=I+1
	  ELSE
	    LLV_REJECT=.TRUE.
	  ENDIF
C
CP.  .  ENDDO
C
	ENDDO
C
CP.  .  IF message not rejected THEN
C
	IF(.NOT.LLV_REJECT)THEN
C
CP.  .  .  add message to post-filter counts
C
	  ICVSTT_BYTES_S_POST(ICVSTT_BUFNUM)=
     +			ICVSTT_BYTES_S_POST(ICVSTT_BUFNUM)+NCVETH_IOSB(1)
	  ICVSTT_MSGS_S_POST(ICVSTT_BUFNUM)=
     +			ICVSTT_MSGS_S_POST(ICVSTT_BUFNUM)+1
C
CP.  .  .  DO CASE (display mode) OF
C
	  GOTO (1000,2000,3000,4000,9999,6000,7000), ICVETH_DISPLAYMODE
C
CP.  .  .  CASE 1 (destinations): increment count of msgs with this addr
C
 1000	  CONTINUE
	    I=FIND_ADDRSLOT_AST(SCVETH_ADDRBUF.W_DEST)
	    IF(I.LT.0)THEN
	      CALL LIB$STOP(ETH__ADRTBLOVF)
	    ENDIF
	    ICVADR_ADDRCNT(I,ICVSTT_BUFNUM)=
     +			ICVADR_ADDRCNT(I,ICVSTT_BUFNUM)+1
	    GOTO 9999
C
CP.  .  .  CASE 2 (protocols): increment count of msgs with this protocol
C
 2000	  CONTINUE
	    I=IEOR(ZEXT(SCVETH_ADDRBUF.PROT(0)),
     +		   ZEXT(SCVETH_ADDRBUF.PROT(1)))
	    J=1
	    DO WHILE (J.LE.127.AND.
     +		      (NCVSTT_PROT(I).NE.
     +				SCVETH_ADDRBUF.W_PROT .AND.
     +		       NCVSTT_PROT(I).NE.0))
	      I=MOD(I+1,255)
	      J=J+1
	    ENDDO
	    IF(J.GT.127)THEN
	      CALL LIB$STOP(ETH__PROTBLOVF)
	    ENDIF
	    ICVSTT_PROTCNT(I,ICVSTT_BUFNUM)=
     +			ICVSTT_PROTCNT(I,ICVSTT_BUFNUM)+1
	    NCVSTT_PROT(I)=SCVETH_ADDRBUF.W_PROT
	    GOTO 9999
C
CP.  .  .  CASE 3 (sizes): increment count of msgs in this size range
C
 3000	  CONTINUE
	    ICVSTT_SIZECNT(MAX(NCVETH_IOSB(1)-1,0)/100,ICVSTT_BUFNUM)=
     +		ICVSTT_SIZECNT(MAX(NCVETH_IOSB(1)-1,0)/100,ICVSTT_BUFNUM)+1
	    GOTO 9999
C
CP.  .  .  CASE 4 (sources): increment count of msgs with this addr
C
 4000	  CONTINUE
	    I=FIND_ADDRSLOT_AST(SCVETH_ADDRBUF.W_SRC)
	    IF(I.LT.0)THEN
	      CALL LIB$STOP(ETH__ADRTBLOVF)
	    ENDIF
	    ICVADR_ADDRCNT(I,ICVSTT_BUFNUM)=
     +			ICVADR_ADDRCNT(I,ICVSTT_BUFNUM)+1
	    GOTO 9999
C
CP.  .  .  CASE 6 (trace data to disk file):
C
 6000	  CONTINUE
	    IF(BCVETH_TRACEFLAG.EQ.0)THEN
	      CALL SYS$GETTIM(QLV_TIME)
	      WRITE(70) QLV_TIME, SCVETH_ADDRBUF, NCVETH_IOSB(1),
     +			(BCVETH_RCVBUF(I),I=1,NCVETH_IOSB(1))
	    ELSE
	      CALL SYS$ASCTIM(,CLV_TIME,,)
	      WRITE(70,9000) CLV_TIME, SCVETH_ADDRBUF.SRC,
     +		SCVETH_ADDRBUF.DEST, SCVETH_ADDRBUF.PROT,
     +		NCVETH_IOSB(1)
 9000	      FORMAT(A23,' s:',Z2.2,5('-',Z2.2),' d:',Z2.2,
     +			5('-',Z2.2),' p:',Z2.2,'-',Z2.2,I5)
	      IF(NCVETH_IOSB(1).GT.0)THEN
		WRITE(70,9001) (BCVETH_RCVBUF(I),I=1,MIN(NCVETH_IOSB(1),40))
 9001		FORMAT(40(Z2.2:))
		CALL SYS$FAO('!AF',,CLV_SCR,%VAL(MIN(NCVETH_IOSB(1),40)),
     +				%REF(BCVETH_RCVBUF))
		WRITE(70,9002) (CLV_SCR(I:I),I=1,MIN(NCVETH_IOSB(1),40))
 9002		FORMAT(40(X,A1:))
	      ENDIF
	    ENDIF
	    GOTO 9999
C
CP.  .  .  CASE 7 (increment count of Internet IP messages with protocol):
C
 7000	  CONTINUE
	    IF(SCVETH_ADDRBUF.W_PROT .EQ.'0008'X)THEN
	      SLV_IP.BUF=CCVETH_RCVBUF(1:20)
	      ICVSTT_PROTCNT(ZEXT(SLV_IP.PROTOCOL),ICVSTT_BUFNUM)=
     +			ICVSTT_PROTCNT(ZEXT(SLV_IP.PROTOCOL),ICVSTT_BUFNUM)+1
	    ENDIF
	    GOTO 9999
C
CP.  .  .  END CASE
C
 9999	  CONTINUE
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ENDIF
C
      ENDIF
C
CP.  queue up another read
C
      CALL QUEUE_ETHER_READ
C
CP.  RETURN
C
      RETURN
C
CP END ETHER_READ
C
      END
      OPTIONS /CHECK=NOOVERFLOW
      SUBROUTINE NETACP_QUERY(ILA_ASTPRM)
C
CP DATA DECLARATIONS for NETACP_QUERY
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_ASTPRM
C
C   declare local variables
C
      INTEGER*4 I
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ADDR.INC'
      INCLUDE 'ETHERMON_ETH.INC'
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__CMPNETACP, ETH__NETACPDAT, ETH__UNDDECNOD
C
CP BEGIN NETACP_QUERY
CP.  test if any errors in query completion
C
      IF(.NOT.NCVETH_NETIOSB(0))THEN
	CALL LIB$SIGNAL(ETH__CMPNETACP,,%VAL(NCVETH_NETIOSB(0)))
      ENDIF
      IF(NCVETH_NETIOSB(1).LT.2)THEN
	CALL LIB$SIGNAL(ETH__NETACPDAT)
      ENDIF
C
CP.  IF entry does not already has DECnet name THEN
C
      IF(NCVADR_DECNETNAMLEN(%LOC(ILA_ASTPRM)).EQ.0)THEN
C
CP.  .  IF returned nodename is zero length THEN
C
	IF(ICHAR(CCVETH_NETACPBUF(1:1)).EQ.0)THEN
C
CP.  .  .  replace name with warning message
C
	  CALL SYS$FAO('!UW.!UW',%REF(CCVETH_NETACPBUF(1:2)),
     +		       CCVETH_NETACPBUF(3:),
     +		       %VAL(SCVADR_ADDR(%LOC(ILA_ASTPRM)).NUM(2)/1024),
     +		       %VAL(MOD(SCVADR_ADDR(%LOC(ILA_ASTPRM)).NUM(2),1024)))
C
CP.  .  .  report error to user (regarding obsolete DECnet database)
C
	  CALL LIB$SIGNAL(ETH__UNDDECNOD,%VAL(2),
     +			  %VAL(SCVADR_ADDR(%LOC(ILA_ASTPRM)).NUM(2)/1024),
     +			  %VAL(MOD(SCVADR_ADDR(%LOC(ILA_ASTPRM)).NUM(2),1024)))
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  .  add name to entry
C
	NCVADR_DECNETNAMLEN(%LOC(ILA_ASTPRM))=ICHAR(CCVETH_NETACPBUF(1:1))
	CCVADR_DECNETNAM(%LOC(ILA_ASTPRM))=
     +		CCVETH_NETACPBUF(3:ICHAR(CCVETH_NETACPBUF(1:1))+2)
C
CP.  ENDIF
C
      ENDIF
C
CP.  unlock the ACP request mechanism
C
      LCVETH_DECNETBUSY=.FALSE.
C
CP.  RETURN
C
      RETURN
C
CP END NETACP_QUERY
C
      END
      INTEGER*4 FUNCTION ADD_FILTER(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for ADD_FILTER
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      INTEGER*4 I, ILV_MSGVEC(0:3)
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_FILTER.INC'
      INCLUDE 'ETHERMON_PRS.INC'
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__FLTTBLFUL, ETH__FLTADDENA, ETH__FLTADDDIS
C
CP BEGIN ADD_FILTER
CP.  scan for an empty slot in the filter table
C
      I=1
      DO WHILE (I.LE.ICVFLT_NUMFILTERS.AND.
     +		SCVFLT_FILTERS(I).ENABLE .NE.0)
	I=I+1
      ENDDO
C
CP.  IF no open slots in existing table THEN
C
      IF(I.GT.ICVFLT_NUMFILTERS)THEN
C
CP.  .  try to add a new slot on end of table
C
	ICVFLT_NUMFILTERS=ICVFLT_NUMFILTERS+1
C
CP.  .  IF no room left THEN
C
	IF(ICVFLT_NUMFILTERS.GT.ICPFLT_MAXFILTERS)THEN
C
CP.  .  .  report error
C
	  ICVFLT_NUMFILTERS=ICPFLT_MAXFILTERS
	  ILV_MSGVEC(0)=1
	  ILV_MSGVEC(1)=%LOC(ETH__FLTTBLFUL)
	  CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  .  ELSE
C
	ELSE
C
CP.  .  .  clear out new slot
C
	  I=ICVFLT_NUMFILTERS
	  SCVFLT_FILTERS(I).ENABLE=0
	  SCVFLT_FILTERS(I).PROT_ACT=0
	  SCVFLT_FILTERS(I).SRC_ACT=0
	  SCVFLT_FILTERS(I).DST_ACT=0
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ENDIF
C
      ENDIF
C
CP.  IF slot found THEN
C
      IF(SCVFLT_FILTERS(I).ENABLE .EQ.0)THEN
C
CP.  .  IF protocol number specified THEN
C
	IF(BCVPRS_PRTMOD.NE.0)THEN
C
CP.  .  .  store it
C
	  SCVFLT_FILTERS(I).W_PROTOCOL=NCVPRS_PROTNUM
	  SCVFLT_FILTERS(I).PROT_ACT=BCVPRS_PRTMOD
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  .  IF source address specified THEN
C
	IF(BCVPRS_SRCMOD.NE.0)THEN
C
CP.  .  .  store it
C
	  SCVFLT_FILTERS(I).W_SOURCE(0)=NCVPRS_SRCADR(0)
	  SCVFLT_FILTERS(I).W_SOURCE(1)=NCVPRS_SRCADR(1)
	  SCVFLT_FILTERS(I).W_SOURCE(2)=NCVPRS_SRCADR(2)
	  SCVFLT_FILTERS(I).SRC_ACT=BCVPRS_SRCMOD
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  .  IF destination address specified THEN
C
	IF(BCVPRS_DSTMOD.NE.0)THEN
C
CP.  .  .  store it
C
	  SCVFLT_FILTERS(I).W_DESTINATION(0)=NCVPRS_DSTADR(0)
	  SCVFLT_FILTERS(I).W_DESTINATION(1)=NCVPRS_DSTADR(1)
	  SCVFLT_FILTERS(I).W_DESTINATION(2)=NCVPRS_DSTADR(2)
	  SCVFLT_FILTERS(I).DST_ACT=BCVPRS_DSTMOD
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  .  mark as enabled or disabled, based on flag setting
C
	IF(BCVPRS_FLTMOD.EQ.0) BCVPRS_FLTMOD=1
	SCVFLT_FILTERS(I).ENABLE=BCVPRS_FLTMOD
C
CP.  .  report entry number of newly created filter
C
	ILV_MSGVEC(0)=3
	IF(BCVPRS_FLTMOD)THEN
	  ILV_MSGVEC(1)=%LOC(ETH__FLTADDENA)
	ELSE
	  ILV_MSGVEC(1)=%LOC(ETH__FLTADDDIS)
	ENDIF
	ILV_MSGVEC(2)=1
	ILV_MSGVEC(3)=I
	CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  ENDIF
C
      ENDIF
C
CP.  RETURN
C
      ADD_FILTER=1
      RETURN
C
CP END ADD_FILTER
C
      END
      INTEGER*4 FUNCTION CLEAR_FILTER(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for CLEAR_FILTER
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      INTEGER*4 ILV_MSGVEC(0:3)
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_FILTER.INC'
      INCLUDE 'ETHERMON_PRS.INC'
C
C   declare local symbolic codes
C
      EXTERNAL	ETH__ALLFLTDEL, ETH__INVFLTNUM, ETH__FLTDEL
C
CP BEGIN CLEAR_FILTER
CP.  IF parameter code says /ALL was specified THEN
C
      IF(%LOC(ILA_PARAM).EQ.-1)THEN
C
CP.  .  zero the active filter count
C
	ICVFLT_NUMFILTERS=0
C
CP.  .  report deletion
C
	ILV_MSGVEC(0)=1
	ILV_MSGVEC(1)=%LOC(ETH__ALLFLTDEL)
	CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  ELSE
C
      ELSE
C
CP.  .  IF the specified filter number is undefined THEN
C
	IF(%LOC(ILA_NUMBER).LE.0.OR.
     +	   %LOC(ILA_NUMBER).GT.ICVFLT_NUMFILTERS.OR.
     +	   SCVFLT_FILTERS(%LOC(ILA_NUMBER)).ENABLE .EQ.0)THEN
C
CP.  .  .  report error
C
	  ILV_MSGVEC(0)=1
	  ILV_MSGVEC(1)=%LOC(ETH__INVFLTNUM)
	  CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  .  ELSE
C
	ELSE
C
CP.  .  .  mark filter table entry as unused
C
	  SCVFLT_FILTERS(%LOC(ILA_NUMBER)).ENABLE=0
	  SCVFLT_FILTERS(%LOC(ILA_NUMBER)).PROT_ACT=0
	  SCVFLT_FILTERS(%LOC(ILA_NUMBER)).SRC_ACT=0
	  SCVFLT_FILTERS(%LOC(ILA_NUMBER)).DST_ACT=0
C
CP.  .  .  report deletion
C
	  ILV_MSGVEC(0)=3
	  ILV_MSGVEC(1)=%LOC(ETH__FLTDEL)
	  ILV_MSGVEC(2)=1
	  ILV_MSGVEC(3)=%LOC(ILA_NUMBER)
	  CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  .  .  IF filter was last filter defined in table THEN
C
	  IF(%LOC(ILA_NUMBER).EQ.ICVFLT_NUMFILTERS)THEN
C
CP.  .  .  .  backwind the filter count until a defined filter is last or
CP			table is empty
C
	    DO WHILE (ICVFLT_NUMFILTERS.GT.0.AND.
     +		      SCVFLT_FILTERS(ICVFLT_NUMFILTERS).ENABLE .EQ.0)
	      ICVFLT_NUMFILTERS=ICVFLT_NUMFILTERS-1
	    ENDDO
C
CP.  .  .  ENDIF
C
	  ENDIF
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ENDIF
C
      ENDIF
C
CP.  RETURN
C
      CLEAR_FILTER=1
      RETURN
C
CP END CLEAR_FILTER
C
      END
      INTEGER*4 FUNCTION DISPLAY(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for DISPLAY
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local COMMON's
C
      INTEGER*4 ICVDSP_INTERVAL/4/, QCVDSP_INTERVAL(2)/-40000000,-1/
      LOGICAL*1 LCVDSP_EXIT
      COMMON /DISPLAY/ ICVDSP_INTERVAL, QCVDSP_INTERVAL, LCVDSP_EXIT
      CHARACTER*32 CCVNAM_DEVNAM, CCVNAM_SYSNAM
      INTEGER*2 NCVNAM_DEVNAMLEN, NCVNAM_SYSNAMLEN
      COMMON /NAMES/ CCVNAM_DEVNAM, CCVNAM_SYSNAM,
     +		NCVNAM_DEVNAMLEN, NCVNAM_SYSNAMLEN
      INTEGER*4 ICVSMG_KEYID, ICVSMG_KTBLID, ICVSMG_PASTEID,
     +		ICVSMG_DSPID
      COMMON /SMG/ ICVSMG_KEYID, ICVSMG_KTBLID, ICVSMG_PASTEID,
     +		ICVSMG_DSPID
      INCLUDE 'ETHERMON_ADDR.INC'
      INCLUDE 'ETHERMON_ETH.INC'
      INCLUDE 'ETHERMON_PROT.INC'
      INCLUDE 'ETHERMON_PRS.INC'
      INCLUDE 'ETHERMON_STATS.INC'
C
C   declare local variables
C
      CHARACTER*8 CLV_SCR
      CHARACTER*18 CLV_DISPLAY_TITLE(5)
      CHARACTER*23 CLV_SYSTIM
      CHARACTER*80 CLV_LINE
      INTEGER*2 NLV_DSPTTLLEN(5), LN
      INTEGER*4 ILV_STATUS, ILV_DSPTTLXPOS(5), I, J, D, B,
     +		IDX(0:ICPADR_MAXADDRS), TIDX, QLV_SYSTIM(2),
     +		QLV_OLDSYSTIM(2), ILV_OLD_C_DBR, ILV_OLD_C_BRC,
     +		ILV_OLD_C_MBY, ILV_OLD_C_MBL, ILV_OLD_C_BS1,
     +		ILV_OLD_C_BSM, ILV_OLD_C_BID, ILV_OLD_C_OVR,
     +		ILV_OLD_C_LBE, ILV_OLD_C_XBU
      LOGICAL*1 LLV_FIRST
      REAL*8	DLV_INTERVAL, DLV_AVG_C_DBR, DLV_AVG_C_BRC,
     +		DLV_AVG_C_MBY, DLV_AVG_C_MBL, DLV_AVG_C_BS1,
     +		DLV_AVG_C_BSM, DLV_AVG_C_BID, DLV_AVG_C_OVR,
     +		DLV_AVG_C_LBE, DLV_AVG_C_XBU, DLV_MIN_C_DBR, DLV_MIN_C_BRC,
     +		DLV_MIN_C_MBY, DLV_MIN_C_MBL, DLV_MIN_C_BS1,
     +		DLV_MIN_C_BSM, DLV_MIN_C_BID, DLV_MIN_C_OVR,
     +		DLV_MIN_C_LBE, DLV_MIN_C_XBU, DLV_MAX_C_DBR, DLV_MAX_C_BRC,
     +		DLV_MAX_C_MBY, DLV_MAX_C_MBL, DLV_MAX_C_BS1,
     +		DLV_MAX_C_BSM, DLV_MAX_C_BID, DLV_MAX_C_OVR,
     +		DLV_MAX_C_LBE, DLV_MAX_C_XBU, DLV_TEMP
      DATA CLV_DISPLAY_TITLE/'TOP DESTINATIONS','TOP PROTOCOLS',
     +		'TOP MESSAGE SIZES','TOP SOURCES','GENERAL STATISTICS'/
      DATA NLV_DSPTTLLEN/16,13,17,11,18/
      DATA ILV_DSPTTLXPOS/33,34,32,35,32/
C
C   declare local procedures
C
      EXTERNAL	DISPLAY_EXIT_AST
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__DISPJAMMD
C
C   declare system procedures
C
      INTEGER*4 SMG$CREATE_PASTEBOARD,	SMG$CREATE_VIRTUAL_DISPLAY,
     +		SMG$PASTE_VIRTUAL_DISPLAY,	SMG$DELETE_PASTEBOARD,
     +		SMG$DELETE_VIRTUAL_DISPLAY,	SMG$ENABLE_UNSOLICITED_INPUT,
     +		SMG$DISABLE_UNSOLICITED_INPUT,	SMG$PUT_CHARS,
     +		SMG$DRAW_LINE,	SMG$BEGIN_DISPLAY_UPDATE, SMG$SNAPSHOT,
     +		SMG$END_DISPLAY_UPDATE,	SMG$PUT_LINE,	SMG$SET_CURSOR_ABS
      EXTERNAL	SMG$CREATE_PASTEBOARD,	SMG$CREATE_VIRTUAL_DISPLAY,
     +		SMG$PASTE_VIRTUAL_DISPLAY,	SMG$DELETE_PASTEBOARD,
     +		SMG$DELETE_VIRTUAL_DISPLAY,	SMG$ENABLE_UNSOLICITED_INPUT,
     +		SMG$DISABLE_UNSOLICITED_INPUT,	SMG$PUT_CHARS,
     +		SMG$DRAW_LINE,	SMG$BEGIN_DISPLAY_UPDATE, SMG$SNAPSHOT,
     +		SMG$END_DISPLAY_UPDATE,	SMG$PUT_LINE,	SMG$SET_CURSOR_ABS
C
C   declare system symbolic codes
C
      INCLUDE '($SMGDEF)	/NOLIST'
C
CP BEGIN DISPLAY
CP.  set up the screen with SMG$
C
      ILV_STATUS=SMG$CREATE_PASTEBOARD(ICVSMG_PASTEID)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
      ILV_STATUS=SMG$CREATE_VIRTUAL_DISPLAY(24,80,ICVSMG_DSPID)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
      ILV_STATUS=SMG$PASTE_VIRTUAL_DISPLAY(ICVSMG_DSPID,ICVSMG_PASTEID,
     +					   1,1)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
C
CP.  enable control-C and control-Z exit traps
C
      LCVDSP_EXIT=.FALSE.
      ILV_STATUS=SMG$ENABLE_UNSOLICITED_INPUT(ICVSMG_PASTEID,
     +					      DISPLAY_EXIT_AST)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
C
CP.  start display setup buffering
C
      ILV_STATUS=SMG$BEGIN_DISPLAY_UPDATE(ICVSMG_DSPID)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
C
CP.  fill in background of selected display
C
      ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +			       'VAX/VMS Ethernet Monitor Display',
     +			       1,25)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
      ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +			       'on node '//CCVNAM_SYSNAM(1:NCVNAM_SYSNAMLEN)//
     +				'  port '//CCVNAM_DEVNAM(1:NCVNAM_DEVNAMLEN),
     +			       2,27)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
      ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +		CLV_DISPLAY_TITLE(%LOC(ILA_PARAM))(1:
     +				  NLV_DSPTTLLEN(%LOC(ILA_PARAM))),
     +			       3,ILV_DSPTTLXPOS(%LOC(ILA_PARAM)),,
     +			       SMG$M_REVERSE,0)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
      IF(%LOC(ILA_PARAM).GE.1.AND.
     +	 %LOC(ILA_PARAM).LE.4)THEN
	ILV_STATUS=SMG$DRAW_LINE(ICVSMG_DSPID,7,39,7,79)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$DRAW_LINE(ICVSMG_DSPID,23,39,23,79)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$DRAW_LINE(ICVSMG_DSPID,7,39,23,39)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$DRAW_LINE(ICVSMG_DSPID,7,49,23,49)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$DRAW_LINE(ICVSMG_DSPID,7,59,23,59)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$DRAW_LINE(ICVSMG_DSPID,7,69,23,69)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$DRAW_LINE(ICVSMG_DSPID,7,79,23,79)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +		'0         50       100       150       200',
     +				 6,39)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
      ELSEIF(%LOC(ILA_PARAM).EQ.5)THEN
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +				 'CUR         AVG         MIN         MAX',
     +				 6,40)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +				 'Packets/second',
     +				 8,4)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +				 'Kbytes/second',
     +				 9,4)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +				 'Packet size',
     +				 10,4)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +				 'Multicast packets/second',
     +				 12,4)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +				 'Multicast kbytes/second',
     +				 13,4)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +				 'Multicast packet size',
     +				 14,4)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +				 'Trans Single Collision',
     +				 16,4)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +				 'Trans Multiple Collision',
     +				 17,4)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +				 'Trans Initially Deferred',
     +				 18,4)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +				 'Internal Buffer Error',
     +				 20,4)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +				 'Local Buffer Error',
     +				 21,4)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,
     +				 'Buffer Unavailable',
     +				 22,4)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
      ENDIF
C
CP.  end display setup buffering
C
      ILV_STATUS=SMG$END_DISPLAY_UPDATE(ICVSMG_DSPID)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
      ILV_STATUS=SMG$SNAPSHOT(ICVSMG_PASTEID)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
C
CP.  store the data processing mode for the Ethernet read routines
C
      CALL GET_PORT_COUNTERS(.FALSE.)
      LLV_FIRST=.TRUE.
      ILV_OLD_C_DBR=ICVETH_C_DBR
      ILV_OLD_C_BRC=ICVETH_C_BRC
      ILV_OLD_C_MBL=ICVETH_C_MBL
      ILV_OLD_C_MBY=ICVETH_C_MBY
      ILV_OLD_C_BS1=ICVETH_C_BS1
      ILV_OLD_C_BSM=ICVETH_C_BSM
      ILV_OLD_C_BID=ICVETH_C_BID
      ILV_OLD_C_OVR=ZEXT(NCVETH_C_OVR)
      ILV_OLD_C_LBE=ZEXT(NCVETH_C_LBE)
      ILV_OLD_C_XBU=ZEXT(NCVETH_C_SBU)+ZEXT(NCVETH_C_UBU)
      DLV_MIN_C_DBR=ICVETH_C_DBR
      DLV_MIN_C_BRC=ICVETH_C_BRC
      DLV_MIN_C_MBL=ICVETH_C_MBL
      DLV_MIN_C_MBY=ICVETH_C_MBY
      DLV_MIN_C_BS1=ICVETH_C_BS1
      DLV_MIN_C_BSM=ICVETH_C_BSM
      DLV_MIN_C_BID=ICVETH_C_BID
      DLV_MIN_C_OVR=ZEXT(NCVETH_C_OVR)
      DLV_MIN_C_LBE=ZEXT(NCVETH_C_LBE)
      DLV_MIN_C_XBU=ZEXT(NCVETH_C_SBU)+ZEXT(NCVETH_C_UBU)
      DLV_MAX_C_DBR=0
      DLV_MAX_C_BRC=0
      DLV_MAX_C_MBL=0
      DLV_MAX_C_MBY=0
      DLV_MAX_C_BS1=0
      DLV_MAX_C_BSM=0
      DLV_MAX_C_BID=0
      DLV_MAX_C_OVR=0
      DLV_MAX_C_LBE=0
      DLV_MAX_C_XBU=0
      CALL SYS$GETTIM(QLV_OLDSYSTIM)
      ICVETH_DISPLAYMODE=%LOC(ILA_PARAM)
C
CP.  start the display interval timer
C
      CALL SYS$SCHDWK(,,QCVDSP_INTERVAL,QCVDSP_INTERVAL)
C
CP.  REPEAT
C
      DO WHILE (.NOT.LCVDSP_EXIT)
C
CP.  .  wait for timer timeout
C
	CALL SYS$HIBER
C
CP.  .  switch buffer flag
C
	B=ICVSTT_BUFNUM
	ICVSTT_BUFNUM=1-ICVSTT_BUFNUM
C
CP.  .  get current counters
C
	CALL GET_PORT_COUNTERS(.FALSE.)
C
CP.  .  get current system time
C
	CALL SYS$GETTIM(QLV_SYSTIM)
C
CP.  .  start display buffering
C
	ILV_STATUS=SMG$BEGIN_DISPLAY_UPDATE(ICVSMG_DSPID)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
C
CP.  .  display clock on screen
C
	CALL SYS$ASCTIM(,CLV_SYSTIM,QLV_SYSTIM,)
	CALL LIB$SUBX(QLV_SYSTIM,QLV_OLDSYSTIM,QLV_OLDSYSTIM)
	IF(QLV_OLDSYSTIM(2).NE.0)THEN
	  CALL LIB$SIGNAL(ETH__DISPJAMMD)
	ENDIF
	DLV_INTERVAL=MAX(QLV_OLDSYSTIM(1)*1.0D-7,0.01D0)
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_SYSTIM,3,58)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
C
CP.  .  calculate message loss factor and display it on screen
C
	CLV_LINE(1:20)='xxx.xx% message loss'
	CALL FOR$CVT_D_TF(DFLOAT(100*((ICVETH_C_DBR-ILV_OLD_C_DBR)-
     +						ICVSTT_MSGS_S_PRE(B)))/
     +				DFLOAT(MAX(ICVETH_C_DBR-ILV_OLD_C_DBR,1)),
     +			  CLV_LINE(1:6),%VAL(2))
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:20),4,1)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
C
CP.  .  write pre-filter and post-filter statistics to screen
C
	CALL SYS$FAO('pre-filter:!6UL bytes/sec!6UL msg/sec xxx.xx%',,
     +		     CLV_LINE,%VAL(NINT(ICVSTT_BYTES_S_PRE(B)/DLV_INTERVAL)),
     +		     %VAL(NINT(ICVSTT_MSGS_S_PRE(B)/DLV_INTERVAL)))
	CALL FOR$CVT_D_TF(DFLOAT((ICVSTT_BYTES_S_PRE(B)+
     +				  (ICVSTT_MSGS_S_PRE(B)*29))*8)/
     +				(1.0D+5*DLV_INTERVAL),
     +			  CLV_LINE(43:48),%VAL(2))
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:49),4,31)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	CALL SYS$FAO('post-filter:!6UL bytes/sec!6UL msg/sec xxx.xx%',,
     +		     CLV_LINE,%VAL(NINT(ICVSTT_BYTES_S_POST(B)/DLV_INTERVAL)),
     +		     %VAL(NINT(ICVSTT_MSGS_S_POST(B)/DLV_INTERVAL)))
	CALL FOR$CVT_D_TF(DFLOAT((ICVSTT_BYTES_S_POST(B)+
     +				  (ICVSTT_MSGS_S_POST(B)*29))*8)/
     +				(1.0D+5*DLV_INTERVAL),
     +			  CLV_LINE(44:49),%VAL(2))
	ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:50),5,30)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
C
CP.  .  DO CASE (displaymode) OF
C
	GOTO (1000,2000,3000,1000,5000), ICVETH_DISPLAYMODE
C
CP.  .  CASE 1 or 4 (destinations or sources):
C
 1000	CONTINUE
C
CP.  .  .  sort the address statistics by descending size (indexed shellsort)
C
	  DO I=0,ICPADR_MAXADDRS
	    IDX(I)=I
	  ENDDO
	  D=(ICPADR_MAXADDRS+1)/2
	  DO WHILE (D.GT.0)
	    DO I=0,ICPADR_MAXADDRS-D
	      IF(ICVADR_ADDRCNT(IDX(I),B).LT.ICVADR_ADDRCNT(IDX(I+D),B))THEN
		TIDX=IDX(I+D)
		IDX(I+D)=IDX(I)
		IF(I.GT.D)THEN
		  J=I-D
		  DO WHILE (J.GE.0.AND.
     +			    ICVADR_ADDRCNT(TIDX,B).GT.
     +					ICVADR_ADDRCNT(IDX(J),B))
		    IDX(J+D)=IDX(J)
		    J=J-D
		  ENDDO
		  IDX(J+D)=TIDX
		ELSE
		  IDX(I)=TIDX
		ENDIF
	      ENDIF
	    ENDDO
	    D=D/2
	  ENDDO
C
CP.  .  .  position in virtual display
C
	  ILV_STATUS=SMG$SET_CURSOR_ABS(ICVSMG_DSPID,8,1)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
C
CP.  .  .  DO for top 15 addresses (or as many as found, whichever smaller)
C
	  D=0
	  DO WHILE (D.LT.15.AND.
     +		    ICVADR_ADDRCNT(IDX(D),B).GT.0)
	    D=D+1
	  ENDDO
	  DO I=0,D-1
C
CP.  .  .  .  format line for display and write to SMG
C
	    J=IDX(I)
	    CLV_LINE=' '
	    LN=0
	    IF(NCVADR_DECNETNAMLEN(J).NE.0)THEN
	      CLV_LINE(1:16)=CCVADR_DECNETNAM(J)(1:NCVADR_DECNETNAMLEN(J))
	      LN=MIN(NCVADR_DECNETNAMLEN(J),16)
	    ENDIF
	    IF(LN.LT.15.AND.
     +	       NCVADR_INETNAMLEN(J).NE.0)THEN
	      IF(LN.NE.0)THEN
		LN=LN+1
		CLV_LINE(LN:LN)=','
	      ENDIF
	      CLV_LINE(LN+1:16)=CCVADR_INETNAM(J)(1:NCVADR_INETNAMLEN(J))
	      LN=MIN(LN+NCVADR_INETNAMLEN(J),16)
	    ENDIF
	    IF(LN.LT.15.AND.
     +	       NCVADR_MISCNAMLEN(J).NE.0)THEN
	      IF(LN.NE.0)THEN
		LN=LN+1
		CLV_LINE(LN:LN)=','
	      ENDIF
	      CLV_LINE(LN+1:16)=CCVADR_MISCNAM(J)(1:NCVADR_MISCNAMLEN(J))
	      LN=MIN(LN+NCVADR_MISCNAMLEN(J),16)
	    ENDIF
	    CALL OTS$CVT_L_TZ(SCVADR_ADDR(J).LNUM,CLV_SCR,%VAL(8))
	    CLV_LINE(18:28)=CLV_SCR(7:8)//'-'//CLV_SCR(5:6)//
     +			    '-'//CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	    CALL OTS$CVT_L_TZ(SCVADR_ADDR(J).NUM(2),CLV_SCR(1:4),
     +			      %VAL(4),%VAL(2))
	    CLV_LINE(29:34)='-'//CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	    CALL OTS$CVT_L_TI(NINT(ICVADR_ADDRCNT(J,B)/DLV_INTERVAL),
     +			      CLV_LINE(35:38),%VAL(1))
	    CALL STR$DUPL_CHAR(CLV_LINE(40:),
     +		MIN(NINT((ICVADR_ADDRCNT(J,B)+4)/DLV_INTERVAL)/5,39),
     +			       %REF('*'))
	    ILV_STATUS=SMG$PUT_LINE(ICVSMG_DSPID,CLV_LINE,1)
	    IF(.NOT.ILV_STATUS)THEN
	      CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	    ENDIF
C
CP.  .  .  ENDDO
C
	  ENDDO
C
CP.  .  .  clear remaining lines of plot (if < 15 addresses)
C
	  DO I=D,14
	    ILV_STATUS=SMG$PUT_LINE(ICVSMG_DSPID,' ',1)
	    IF(.NOT.ILV_STATUS)THEN
	      CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	    ENDIF
	  ENDDO
	  GOTO 9999
C
CP.  .  CASE 2 (protocols):
C
 2000	CONTINUE
C
CP.  .  .  sort the protocol statistics by descending size (indexed shellsort)
C
	  DO I=0,255
	    IDX(I)=I
	  ENDDO
	  D=128
	  DO WHILE (D.GT.0)
	    DO I=0,255-D
	      IF(ICVSTT_PROTCNT(IDX(I),B).LT.ICVSTT_PROTCNT(IDX(I+D),B))THEN
		TIDX=IDX(I+D)
		IDX(I+D)=IDX(I)
		IF(I.GT.D)THEN
		  J=I-D
		  DO WHILE (J.GE.0.AND.
     +			    ICVSTT_PROTCNT(TIDX,B).GT.
     +					ICVSTT_PROTCNT(IDX(J),B))
		    IDX(J+D)=IDX(J)
		    J=J-D
		  ENDDO
		  IDX(J+D)=TIDX
		ELSE
		  IDX(I)=TIDX
		ENDIF
	      ENDIF
	    ENDDO
	    D=D/2
	  ENDDO
C
CP.  .  .  position in virtual display
C
	  ILV_STATUS=SMG$SET_CURSOR_ABS(ICVSMG_DSPID,8,1)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
C
CP.  .  .  DO for top 8 protocols (or as many as found, whichever smaller)
C
	  D=0
	  DO WHILE (D.LT.8.AND.
     +		    ICVSTT_PROTCNT(IDX(D),B).GT.0)
	    D=D+1
	  ENDDO
	  DO I=0,D-1
C
CP.  .  .  .  try to find protocol in protocol name table
C
	    J=1
	    DO WHILE (J.LE.ICVPRT_NUMNAMES.AND.
     +		      NCVSTT_PROT(IDX(I)).NE.NCVPRT_PROT_NUM(J))
	      J=J+1
	    ENDDO
C
CP.  .  .  .  format line for display and write to SMG
C
	    CLV_LINE=' '
	    IF(J.LE.ICVPRT_NUMNAMES)THEN
	      CLV_LINE(2:25)=CCVPRT_PROTNAM(J)(1:NCVPRT_PROTNAMLEN(J))
	    ENDIF
	    CALL OTS$CVT_L_TZ(NCVSTT_PROT(IDX(I)),
     +			      CLV_SCR(1:4),%VAL(4),%VAL(2))
	    CLV_LINE(27:31)=CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	    CALL OTS$CVT_L_TI(NINT(ICVSTT_PROTCNT(IDX(I),B)/DLV_INTERVAL),
     +			      CLV_LINE(32:37),%VAL(1))
	    CALL STR$DUPL_CHAR(CLV_LINE(40:),
     +		MIN(NINT((ICVSTT_PROTCNT(IDX(I),B)+4)/DLV_INTERVAL)/5,39),
     +			       %REF('*'))
	    ILV_STATUS=SMG$PUT_LINE(ICVSMG_DSPID,CLV_LINE,2)
	    IF(.NOT.ILV_STATUS)THEN
	      CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	    ENDIF
C
CP.  .  .  ENDDO
C
	  ENDDO
C
CP.  .  .  clear remaining lines of plot (if < 8 protocols)
C
	  DO I=D,7
	    ILV_STATUS=SMG$PUT_LINE(ICVSMG_DSPID,' ',2)
	    IF(.NOT.ILV_STATUS)THEN
	      CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	    ENDIF
	  ENDDO
	  GOTO 9999
C
CP.  .  CASE 3 (sizes):
C
 3000	CONTINUE
C
CP.  .  .  position in virtual display
C
	  ILV_STATUS=SMG$SET_CURSOR_ABS(ICVSMG_DSPID,8,1)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
C
CP.  .  .  DO for all cell size ranges
C
	  DO I=0,14
C
CP.  .  .  .  format line for display and write to SMG
C
	    CLV_LINE=' '
	    CALL SYS$FAO('!UL - !UL',,CLV_LINE(19:29),%VAL(I*100+1),
     +			 %VAL((I+1)*100))
	    CALL OTS$CVT_L_TI(NINT(ICVSTT_SIZECNT(I,B)/DLV_INTERVAL),
     +			      CLV_LINE(32:37),%VAL(1))
	    CALL STR$DUPL_CHAR(CLV_LINE(40:),
     +		MIN(NINT((ICVSTT_SIZECNT(I,B)+4)/DLV_INTERVAL)/5,39),
     +			       %REF('*'))
	    ILV_STATUS=SMG$PUT_LINE(ICVSMG_DSPID,CLV_LINE,1)
	    IF(.NOT.ILV_STATUS)THEN
	      CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	    ENDIF
C
CP.  .  .  ENDDO
C
	  ENDDO
	  GOTO 9999
C
CP.  .  CASE 5 (general statistics):
C
 5000	CONTINUE
C
CP.  .  .  display current statistics
C
	  DLV_TEMP=DFLOAT(ICVETH_C_DBR-ILV_OLD_C_DBR)/DLV_INTERVAL
	  DLV_AVG_C_DBR=0.9*DLV_AVG_C_DBR + 0.1*DLV_TEMP
	  IF(LLV_FIRST)DLV_AVG_C_DBR=DLV_TEMP
	  IF(DLV_TEMP.GT.DLV_MAX_C_DBR) DLV_MAX_C_DBR=DLV_TEMP
	  IF(DLV_TEMP.LT.DLV_MIN_C_DBR) DLV_MIN_C_DBR=DLV_TEMP
	  CALL FOR$CVT_D_TF(DLV_TEMP,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),8,35)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_AVG_C_DBR,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),8,47)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MIN_C_DBR,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),8,59)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MAX_C_DBR,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),8,71)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  DLV_TEMP=DFLOAT(ICVETH_C_BRC-ILV_OLD_C_BRC)/
     +			(1024.D0*DLV_INTERVAL)
	  DLV_AVG_C_BRC=0.9*DLV_AVG_C_BRC + 0.1*DLV_TEMP
	  IF(LLV_FIRST)DLV_AVG_C_BRC=DLV_TEMP
	  IF(DLV_TEMP.GT.DLV_MAX_C_BRC) DLV_MAX_C_BRC=DLV_TEMP
	  IF(DLV_TEMP.LT.DLV_MIN_C_BRC) DLV_MIN_C_BRC=DLV_TEMP
	  CALL FOR$CVT_D_TF(DLV_TEMP,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),9,35)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_AVG_C_BRC,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),9,47)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MIN_C_BRC,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),9,59)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MAX_C_BRC,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),9,71)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DFLOAT(ICVETH_C_BRC-ILV_OLD_C_BRC)/
     +			    DFLOAT(MAX(ICVETH_C_DBR-ILV_OLD_C_DBR,1)),
     +			  CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),10,35)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  DLV_TEMP=DFLOAT(ICVETH_C_MBL-ILV_OLD_C_MBL)/DLV_INTERVAL
	  DLV_AVG_C_MBL=0.9*DLV_AVG_C_MBL + 0.1*DLV_TEMP
	  IF(LLV_FIRST)DLV_AVG_C_MBL=DLV_TEMP
	  IF(DLV_TEMP.GT.DLV_MAX_C_MBL) DLV_MAX_C_MBL=DLV_TEMP
	  IF(DLV_TEMP.LT.DLV_MIN_C_MBL) DLV_MIN_C_MBL=DLV_TEMP
	  CALL FOR$CVT_D_TF(DLV_TEMP,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),12,35)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_AVG_C_MBL,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),12,47)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MIN_C_MBL,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),12,59)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MAX_C_MBL,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),12,71)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  DLV_TEMP=DFLOAT(ICVETH_C_MBY-ILV_OLD_C_MBY)/
     +					(1024.D0*DLV_INTERVAL)
	  DLV_AVG_C_MBY=0.9*DLV_AVG_C_MBY + 0.1*DLV_TEMP
	  IF(LLV_FIRST)DLV_AVG_C_MBY=DLV_TEMP
	  IF(DLV_TEMP.GT.DLV_MAX_C_MBY) DLV_MAX_C_MBY=DLV_TEMP
	  IF(DLV_TEMP.LT.DLV_MIN_C_MBY) DLV_MIN_C_MBY=DLV_TEMP
	  CALL FOR$CVT_D_TF(DLV_TEMP,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),13,35)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_AVG_C_MBY,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),13,47)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MIN_C_MBY,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),13,59)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MAX_C_MBY,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),13,71)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DFLOAT(ICVETH_C_MBY-ILV_OLD_C_MBY)/
     +			    DFLOAT(MAX(ICVETH_C_MBL-ILV_OLD_C_MBL,1)),
     +			  CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),14,35)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  DLV_TEMP=DFLOAT(ICVETH_C_BS1-ILV_OLD_C_BS1)/DLV_INTERVAL
	  DLV_AVG_C_BS1=0.9*DLV_AVG_C_BS1 + 0.1*DLV_TEMP
	  IF(LLV_FIRST)DLV_AVG_C_BS1=DLV_TEMP
	  IF(DLV_TEMP.GT.DLV_MAX_C_BS1) DLV_MAX_C_BS1=DLV_TEMP
	  IF(DLV_TEMP.LT.DLV_MIN_C_BS1) DLV_MIN_C_BS1=DLV_TEMP
	  CALL FOR$CVT_D_TF(DLV_TEMP,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),16,35)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_AVG_C_BS1,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),16,47)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MIN_C_BS1,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),16,59)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MAX_C_BS1,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),16,71)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  DLV_TEMP=DFLOAT(ICVETH_C_BSM-ILV_OLD_C_BSM)/DLV_INTERVAL
	  DLV_AVG_C_BSM=0.9*DLV_AVG_C_BSM + 0.1*DLV_TEMP
	  IF(LLV_FIRST)DLV_AVG_C_BSM=DLV_TEMP
	  IF(DLV_TEMP.GT.DLV_MAX_C_BSM) DLV_MAX_C_BSM=DLV_TEMP
	  IF(DLV_TEMP.LT.DLV_MIN_C_BSM) DLV_MIN_C_BSM=DLV_TEMP
	  CALL FOR$CVT_D_TF(DLV_TEMP,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),17,35)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_AVG_C_BSM,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),17,47)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MIN_C_BSM,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),17,59)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MAX_C_BSM,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),17,71)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  DLV_TEMP=DFLOAT(ICVETH_C_BID-ILV_OLD_C_BID)/DLV_INTERVAL
	  DLV_AVG_C_BID=0.9*DLV_AVG_C_BID + 0.1*DLV_TEMP
	  IF(LLV_FIRST)DLV_AVG_C_BID=DLV_TEMP
	  IF(DLV_TEMP.GT.DLV_MAX_C_BID) DLV_MAX_C_BID=DLV_TEMP
	  IF(DLV_TEMP.LT.DLV_MIN_C_BID) DLV_MIN_C_BID=DLV_TEMP
	  CALL FOR$CVT_D_TF(DLV_TEMP,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),18,35)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_AVG_C_BID,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),18,47)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MIN_C_BID,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),18,59)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MAX_C_BID,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),18,71)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  DLV_TEMP=DFLOAT(JZEXT(NCVETH_C_OVR)-ILV_OLD_C_OVR)/DLV_INTERVAL
	  DLV_AVG_C_OVR=0.9*DLV_AVG_C_OVR + 0.1*DLV_TEMP
	  IF(LLV_FIRST)DLV_AVG_C_OVR=DLV_TEMP
	  IF(DLV_TEMP.GT.DLV_MAX_C_OVR) DLV_MAX_C_OVR=DLV_TEMP
	  IF(DLV_TEMP.LT.DLV_MIN_C_OVR) DLV_MIN_C_OVR=DLV_TEMP
	  CALL FOR$CVT_D_TF(DLV_TEMP,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),20,35)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_AVG_C_OVR,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),20,47)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MIN_C_OVR,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),20,59)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MAX_C_OVR,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),20,71)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  DLV_TEMP=DFLOAT(JZEXT(NCVETH_C_LBE)-ILV_OLD_C_LBE)/DLV_INTERVAL
	  DLV_AVG_C_LBE=0.9*DLV_AVG_C_LBE + 0.1*DLV_TEMP
	  IF(LLV_FIRST)DLV_AVG_C_LBE=DLV_TEMP
	  IF(DLV_TEMP.GT.DLV_MAX_C_LBE) DLV_MAX_C_LBE=DLV_TEMP
	  IF(DLV_TEMP.LT.DLV_MIN_C_LBE) DLV_MIN_C_LBE=DLV_TEMP
	  CALL FOR$CVT_D_TF(DLV_TEMP,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),21,35)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_AVG_C_LBE,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),21,47)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MIN_C_LBE,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),21,59)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MAX_C_LBE,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),21,71)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  DLV_TEMP=DFLOAT(JZEXT(NCVETH_C_SBU)+
     +			  JZEXT(NCVETH_C_UBU)-ILV_OLD_C_XBU)/
     +		   DLV_INTERVAL
	  DLV_AVG_C_XBU=0.9*DLV_AVG_C_XBU + 0.1*DLV_TEMP
	  IF(LLV_FIRST)DLV_AVG_C_XBU=DLV_TEMP
	  IF(DLV_TEMP.GT.DLV_MAX_C_XBU) DLV_MAX_C_XBU=DLV_TEMP
	  IF(DLV_TEMP.LT.DLV_MIN_C_XBU) DLV_MIN_C_XBU=DLV_TEMP
	  CALL FOR$CVT_D_TF(DLV_TEMP,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),22,35)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_AVG_C_XBU,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),22,47)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MIN_C_XBU,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),22,59)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  CALL FOR$CVT_D_TF(DLV_MAX_C_XBU,CLV_LINE(1:9),%VAL(2))
	  ILV_STATUS=SMG$PUT_CHARS(ICVSMG_DSPID,CLV_LINE(1:9),22,71)
	  IF(.NOT.ILV_STATUS)THEN
	    CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	  ENDIF
	  LLV_FIRST=.FALSE.
	  GOTO 9999
C
CP.  .  END CASE
C
 9999	CONTINUE
C
CP.  .  flush buffer to terminal
C
	ILV_STATUS=SMG$END_DISPLAY_UPDATE(ICVSMG_DSPID)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
	ILV_STATUS=SMG$SNAPSHOT(ICVSMG_PASTEID)
	IF(.NOT.ILV_STATUS)THEN
	  CALL LIB$SIGNAL(%VAL(ILV_STATUS))
	ENDIF
C
CP.  .  clear buffer counts for next buffer switch
C
	QLV_OLDSYSTIM(1)=QLV_SYSTIM(1)
	QLV_OLDSYSTIM(2)=QLV_SYSTIM(2)
	DO I=0,ICPADR_MAXADDRS
	  ICVADR_ADDRCNT(I,B)=0
	ENDDO
	DO I=0,255
	  ICVSTT_PROTCNT(I,B)=0
	ENDDO
	DO I=0,150
	  ICVSTT_SIZECNT(I,B)=0
	ENDDO
	ICVSTT_BYTES_S_PRE(B)=0
	ICVSTT_MSGS_S_PRE(B)=0
	ICVSTT_BYTES_S_POST(B)=0
	ICVSTT_MSGS_S_POST(B)=0
C
CP.  .  save counters for next run
C
	ILV_OLD_C_DBR=ICVETH_C_DBR
	ILV_OLD_C_BRC=ICVETH_C_BRC
	ILV_OLD_C_MBL=ICVETH_C_MBL
	ILV_OLD_C_MBY=ICVETH_C_MBY
	ILV_OLD_C_BS1=ICVETH_C_BS1
	ILV_OLD_C_BSM=ICVETH_C_BSM
	ILV_OLD_C_BID=ICVETH_C_BID
	ILV_OLD_C_OVR=ZEXT(NCVETH_C_OVR)
	ILV_OLD_C_LBE=ZEXT(NCVETH_C_LBE)
	ILV_OLD_C_XBU=ZEXT(NCVETH_C_SBU)+ZEXT(NCVETH_C_UBU)
C
CP.  UNTIL exit trap triggered
C
      ENDDO
C
CP.  turn off display mode
C
      ICVETH_DISPLAYMODE=0
C
CP.  flush any extraneous pending wake-ups that might screw up the next DISPLAY
C
      CALL SYS$WAKE(,)
      CALL SYS$HIBER
C
CP.  undo SMG screen control
C
      ILV_STATUS=SMG$DISABLE_UNSOLICITED_INPUT(ICVSMG_PASTEID)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
      ILV_STATUS=SMG$DELETE_VIRTUAL_DISPLAY(ICVSMG_DSPID)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
      ILV_STATUS=SMG$DELETE_PASTEBOARD(ICVSMG_PASTEID,0)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ENDIF
C
CP.  RETURN
C
      DISPLAY=1
      RETURN
C
CP END DISPLAY
C
      END
      SUBROUTINE DISPLAY_EXIT_AST
C
CP DATA DECLARATIONS for DISPLAY_EXIT_AST
C
      IMPLICIT NONE
C
C   declare local COMMON's
C
      INTEGER*4 ICVDSP_INTERVAL/4/, QCVDSP_INTERVAL(2)/-40000000,-1/
      LOGICAL*1 LCVDSP_EXIT
      COMMON /DISPLAY/ ICVDSP_INTERVAL, QCVDSP_INTERVAL, LCVDSP_EXIT
C
CP BEGIN DISPLAY_EXIT_AST
CP.  flag that display is done
C
      LCVDSP_EXIT=.TRUE.
C
CP.  stop display interval timer
C
      CALL SYS$CANWAK(,)
C
CP.  force main program to exit wait state now
C
      CALL SYS$WAKE(,)
C
CP.  RETURN
C
      RETURN
C
CP END DISPLAY_EXIT_AST
C
      END
      INTEGER*4 FUNCTION HELP(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for HELP
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      CHARACTER*128 CLV_IMGFIL
      INTEGER*2 LN, NLV_IOSB(0:3)
      INTEGER*4 ILV_DESCRIPTOR(0:1), ILV_ITMLST(4), ILV_STATUS
C
C   declare system procedures
C
      INTEGER*4 LBR$OUTPUT_HELP, SYS$GETJPIW
      EXTERNAL	LBR$OUTPUT_HELP, LIB$PUT_OUTPUT, LIB$GET_INPUT,
     +		SYS$GETJPIW
C
C   declare system symbolic codes
C
      INCLUDE '($FSCNDEF)	/NOLIST'
      INCLUDE '($HLPDEF)	/NOLIST'
      INCLUDE '($JPIDEF)	/NOLIST'
C
CP BEGIN HELP
CP.  obtain the directory location of the .EXE file (and, presumably, the
CP		help library)
C
      ILV_ITMLST(1)=LEN(CLV_IMGFIL)+JISHFT(JPI$_IMAGNAME,16)
      ILV_ITMLST(2)=%LOC(CLV_IMGFIL)
      ILV_ITMLST(3)=%LOC(LN)
      ILV_ITMLST(4)=0
      ILV_STATUS=SYS$GETJPIW(,,,ILV_ITMLST,NLV_IOSB,,)
      IF(.NOT.ILV_STATUS)THEN
	CALL LIB$SIGNAL(%VAL(ILV_STATUS))
      ELSEIF(.NOT.NLV_IOSB(0))THEN
	CALL LIB$SIGNAL(%VAL(NLV_IOSB(0)))
      ENDIF
      ILV_ITMLST(1)=JISHFT(FSCN$_NAME,16)
      ILV_ITMLST(2)=0
      ILV_ITMLST(3)=0
      CALL SYS$FILESCAN(CLV_IMGFIL(1:LN),ILV_ITMLST,)
      LN=ILV_ITMLST(2)-%LOC(CLV_IMGFIL)
C
CP.  build descriptor of "token" (HELP keywords)
C
      ILV_DESCRIPTOR(0)=%LOC(ILA_TOKENCNT)
      ILV_DESCRIPTOR(1)=%LOC(ILA_TOKENPTR)
C
CP.  extract the results from the help library
C
      HELP=LBR$OUTPUT_HELP(LIB$PUT_OUTPUT,,ILV_DESCRIPTOR,
     +			   CLV_IMGFIL(1:LN)//'ETHERMON.HLB',
     +			   HLP$M_PROMPT,LIB$GET_INPUT)
C
CP.  RETURN
C
      RETURN
C
CP END HELP
C
      END
      INTEGER*4 FUNCTION LOAD(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for LOAD
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ADDR.INC'
      INCLUDE 'ETHERMON_FILTER.INC'
      INCLUDE 'ETHERMON_PROT.INC'
      INCLUDE 'ETHERMON_PRS.INC'
C
C   declare local variables
C
      BYTE	BLV_INET(0:3)
      CHARACTER*2 CLV_FLTMOD/'EO'/
      CHARACTER*132 CLV_SCR, CLV_FILE
      INTEGER*2 LN, LN2
      INTEGER*4 ILV_STATUS, ILV_ENTRIES, I, ILV_INET, ILV_MSGVEC(0:3)
      RECORD /ETH_ADR/ SLV_ADDR
      EQUIVALENCE (ILV_INET, BLV_INET)
C
C   declare local procedures
C
      INTEGER*4 FIND_ADDRSLOT
      EXTERNAL	FIND_ADDRSLOT
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__ADRTBLFUL,	ETH__FLTTBLFUL,	ETH__PRTTBLFUL,
     +		ETH__NOFILE,	ETH__FILOPNERR
C
C   declare system symbolic message codes
C
      INCLUDE '($FORIOSDEF)	/NOLIST'
C
CP BEGIN LOAD
CP.  IF target is unspecified or forced as addresses THEN
C
      ILV_ENTRIES=0
      IF(BCVPRS_FLTMOD.EQ.0.OR.
     +	 BCVPRS_FLTMOD.EQ.1)THEN
C
CP.  .  attempt to open the input file
C
	OPEN(UNIT=47,FILE=CCVPRS_FILESPEC(1:NCVPRS_FILESPECLEN),
     +	     STATUS='OLD',DEFAULTFILE='ETHERMON.ETH_ADDRESSES',
     +	     FORM='FORMATTED',READONLY,IOSTAT=ILV_STATUS)
C
CP.  .  IF file open failed THEN
C
	IF(ILV_STATUS.NE.0)THEN
C
CP.  .  .  report appropriate error
C
	  IF(ILV_STATUS .EQ. FOR$IOS_FILNOTFOU)THEN
	    ILV_MSGVEC(0)=1
	    ILV_MSGVEC(1)=%LOC(ETH__NOFILE)
	  ELSE
	    ILV_MSGVEC(0)=3
	    ILV_MSGVEC(1)=%LOC(ETH__FILOPNERR)
	    ILV_MSGVEC(2)=1
	    ILV_MSGVEC(3)=ILV_STATUS
	  ENDIF
	  CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  .  ELSE
C
	ELSE
C
CP.  .  .  DO for all entries in file
C
	  DO WHILE(ILV_STATUS.EQ.0)
C
CP.  .  .  .  attempt to read another record
C
	    READ(47,'(Z4,Z8,X,Q,A)',IOSTAT=ILV_STATUS) SLV_ADDR.NUM(2),
     +			SLV_ADDR.LNUM, LN, CLV_SCR
C
CP.  .  .  .  IF read succeeded THEN
C
	    IF(ILV_STATUS.EQ.0)THEN
	      ILV_ENTRIES=ILV_ENTRIES+1
C
CP.  .  .  .  .  add entry to address table
C
	      I=FIND_ADDRSLOT(SLV_ADDR,.TRUE.)
	      IF(I.LT.0)THEN
		ILV_MSGVEC(0)=1
		ILV_MSGVEC(1)=%LOC(ETH__ADRTBLFUL)
		CALL SYS$PUTMSG(ILV_MSGVEC,,,)
		ILV_ENTRIES=ILV_ENTRIES-1
		ILV_STATUS=-2
	      ELSE
		IF(CLV_SCR(1:1).EQ.' ')THEN
		  LN2=0
		  DO WHILE (CLV_SCR(1+LN2:1+LN2).EQ.' ')
		    LN2=LN2+1
		  ENDDO
		  CLV_SCR=CLV_SCR(LN2:)
		  LN=LN-LN2+1
		ENDIF
		DO WHILE (LN.GT.0)
		  IF(CLV_SCR(1:2).EQ.'D:')THEN
		    LN2=0
		    DO WHILE (CLV_SCR(3+LN2:3+LN2).NE.' '.AND.
     +			      LN2+3.LE.LN)
		      LN2=LN2+1
		    ENDDO
		    CCVADR_DECNETNAM(I)=CLV_SCR(3:2+LN2)
		    NCVADR_DECNETNAMLEN(I)=MIN(LN2,7)
		    LN2=LN2+3
		  ELSEIF(CLV_SCR(1:2).EQ.'I:')THEN
		    LN2=3
		    CALL OTS$CVT_TU_L(CLV_SCR(LN2:LN2+
     +					INDEX(CLV_SCR(LN2:),'.')-2),
     +				    BLV_INET(0),%VAL(1))
		    LN2=LN2+INDEX(CLV_SCR(LN2:),'.')
		    CALL OTS$CVT_TU_L(CLV_SCR(LN2:LN2+
     +					INDEX(CLV_SCR(LN2:),'.')-2),
     +				    BLV_INET(1),%VAL(1))
		    LN2=LN2+INDEX(CLV_SCR(LN2:),'.')
		    CALL OTS$CVT_TU_L(CLV_SCR(LN2:LN2+
     +					INDEX(CLV_SCR(LN2:),'.')-2),
     +				    BLV_INET(2),%VAL(1))
		    LN2=LN2+INDEX(CLV_SCR(LN2:),'.')
		    CALL OTS$CVT_TU_L(CLV_SCR(LN2:LN2+
     +					INDEX(CLV_SCR(LN2:),'''')-2),
     +				    BLV_INET(3),%VAL(1))
		    LN2=LN2+INDEX(CLV_SCR(LN2:),'''')
		    ICVADR_INETADR(I)=ILV_INET
		    ILV_INET=INDEX(CLV_SCR(LN2:),'''')+LN2-2
		    CCVADR_INETNAM(I)=CLV_SCR(LN2:ILV_INET)
		    NCVADR_INETNAMLEN(I)=MIN(ILV_INET-LN2+1,24)
		    LN2=ILV_INET+2
		  ELSEIF(CLV_SCR(1:2).EQ.'M:')THEN
		    CCVADR_MISCNAM(I)=CLV_SCR(3:LN)
		    NCVADR_MISCNAMLEN(I)=MIN(LN-2,24)
		    LN2=LN+1
		  ELSEIF(CLV_SCR(1:1).NE.' ')THEN
		    CALL LIB$PUT_OUTPUT(
     +			'%ETHERMON-W-BADFIL, invalid record in file')
		    LN2=LN+1
		  ENDIF
		  IF(CLV_SCR(LN2:LN2).EQ.' '.AND.
     +		     LN2.LT.LN)THEN
		    DO WHILE (CLV_SCR(LN2:LN2).EQ.' '.AND.
     +			      LN2.LE.LN)
		      LN2=LN2+1
		    ENDDO
		  ENDIF
		  CLV_SCR=CLV_SCR(LN2:)
		  LN=LN-LN2+1
		ENDDO
	      ENDIF
C
CP.  .  .  .  ENDIF
C
	    ENDIF
C
CP.  .  .  ENDDO
C
	  ENDDO
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ELSEIF target is forced as filters THEN
C
      ELSEIF(BCVPRS_FLTMOD.EQ.2)THEN
C
CP.  .  attempt to open the input file
C
	OPEN(UNIT=47,FILE=CCVPRS_FILESPEC(1:NCVPRS_FILESPECLEN),
     +	     STATUS='OLD',DEFAULTFILE='ETHERMON.ETH_FILTERS',
     +	     FORM='FORMATTED',READONLY,IOSTAT=ILV_STATUS)
C
CP.  .  IF file open failed THEN
C
	IF(ILV_STATUS.NE.0)THEN
C
CP.  .  .  report appropriate error
C
	  IF(ILV_STATUS .EQ. FOR$IOS_FILNOTFOU)THEN
	    ILV_MSGVEC(0)=1
	    ILV_MSGVEC(1)=%LOC(ETH__NOFILE)
	  ELSE
	    ILV_MSGVEC(0)=3
	    ILV_MSGVEC(1)=%LOC(ETH__FILOPNERR)
	    ILV_MSGVEC(2)=1
	    ILV_MSGVEC(3)=ILV_STATUS
	  ENDIF
	  CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  .  ELSE
C
	ELSE
C
CP.  .  .  DO for all entries in file
C
	  DO WHILE(ILV_STATUS.EQ.0)
C
CP.  .  .  .  attempt to read another record
C
	    READ(47,'(Q,A)',IOSTAT=ILV_STATUS) LN, CLV_SCR
C
CP.  .  .  .  IF read succeeded THEN
C
	    IF(ILV_STATUS.EQ.0)THEN
	      ILV_ENTRIES=ILV_ENTRIES+1
C
CP.  .  .  .  .  add entry to filter table
C
	      I=1
	      DO WHILE (SCVFLT_FILTERS(I).ENABLE .NE.0.AND.
     +			I.LE.ICPFLT_MAXFILTERS)
		I=I+1
	      ENDDO
	      IF(I.GT.ICPFLT_MAXFILTERS)THEN
		ILV_MSGVEC(0)=1
		ILV_MSGVEC(1)=%LOC(ETH__FLTTBLFUL)
		CALL SYS$PUTMSG(ILV_MSGVEC,,,)
		ILV_ENTRIES=ILV_ENTRIES-1
		ILV_STATUS=-2
	      ELSE
		SCVFLT_FILTERS(I).ENABLE=2		! initially disabled
		ICVFLT_NUMFILTERS=ICVFLT_NUMFILTERS+1
		SCVFLT_FILTERS(I).PROT_ACT=0
		SCVFLT_FILTERS(I).SRC_ACT=0
		SCVFLT_FILTERS(I).DST_ACT=0
		DO WHILE (LN.GT.0)
		  IF(CLV_SCR(2:3).EQ.'P:')THEN
		    CALL OTS$CVT_TZ_L(CLV_SCR(4:7),
     +				    SCVFLT_FILTERS(I).W_PROTOCOL,%VAL(2))
		    SCVFLT_FILTERS(I).PROT_ACT=INDEX(CLV_FLTMOD,CLV_SCR(1:1))
		    LN2=8
		  ELSEIF(CLV_SCR(2:3).EQ.'S:')THEN
		    CALL OTS$CVT_TZ_L(CLV_SCR(4:7),
     +				    SCVFLT_FILTERS(I).LSOURCE4,%VAL(2))
		    CALL OTS$CVT_TZ_L(CLV_SCR(8:15),
     +				    SCVFLT_FILTERS(I).LSOURCE)
		    SCVFLT_FILTERS(I).SRC_ACT=INDEX(CLV_FLTMOD,CLV_SCR(1:1))
		    LN2=16
		  ELSEIF(CLV_SCR(2:3).EQ.'D:')THEN
		    CALL OTS$CVT_TZ_L(CLV_SCR(4:7),
     +				    SCVFLT_FILTERS(I).LDESTINATION4,
     +				    %VAL(2))
		    CALL OTS$CVT_TZ_L(CLV_SCR(8:15),
     +				    SCVFLT_FILTERS(I).LDESTINATION)
		    SCVFLT_FILTERS(I).DST_ACT=INDEX(CLV_FLTMOD,CLV_SCR(1:1))
		    LN2=16
		  ELSEIF(CLV_SCR(1:1).NE.' ')THEN
		    CALL LIB$PUT_OUTPUT(
     +			'%ETHERMON-W-BADFIL, invalid record in file')
		    LN2=LN+1
		  ENDIF
		  IF(CLV_SCR(LN2:LN2).EQ.' '.AND.
     +		     LN2.LT.LN)THEN
		    DO WHILE (CLV_SCR(LN2:LN2).EQ.' '.AND.
     +			      LN2.LE.LN)
		      LN2=LN2+1
		    ENDDO
		  ENDIF
		  CLV_SCR=CLV_SCR(LN2:)
		  LN=LN-LN2+1
		ENDDO
	      ENDIF
C
CP.  .  .  .  ENDIF
C
	    ENDIF
C
CP.  .  .  ENDDO
C
	  ENDDO
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ELSEIF target is forced as protocols THEN
C
      ELSEIF(BCVPRS_FLTMOD.EQ.3)THEN
C
CP.  .  attempt to open the input file
C
	OPEN(UNIT=47,FILE=CCVPRS_FILESPEC(1:NCVPRS_FILESPECLEN),
     +	     STATUS='OLD',DEFAULTFILE='ETHERMON.ETH_PROTOCOLS',
     +	     FORM='FORMATTED',READONLY,IOSTAT=ILV_STATUS)
C
CP.  .  IF file open failed THEN
C
	IF(ILV_STATUS.NE.0)THEN
C
CP.  .  .  report appropriate error
C
	  IF(ILV_STATUS .EQ. FOR$IOS_FILNOTFOU)THEN
	    ILV_MSGVEC(0)=1
	    ILV_MSGVEC(1)=%LOC(ETH__NOFILE)
	  ELSE
	    ILV_MSGVEC(0)=3
	    ILV_MSGVEC(1)=%LOC(ETH__FILOPNERR)
	    ILV_MSGVEC(2)=1
	    ILV_MSGVEC(3)=ILV_STATUS
	  ENDIF
	  CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  .  ELSE
C
	ELSE
C
CP.  .  .  DO for all entries in file
C
	  DO WHILE(ILV_STATUS.EQ.0)
C
CP.  .  .  .  attempt to read another record
C
	    READ(47,'(Z4,X,Q,A)',IOSTAT=ILV_STATUS) LN, LN2, CLV_SCR
C
CP.  .  .  .  IF read succeeded THEN
C
	    IF(ILV_STATUS.EQ.0)THEN
	      ILV_ENTRIES=ILV_ENTRIES+1
C
CP.  .  .  .  .  find corresponding entry in protocol table
C
	      I=1
	      DO WHILE (I.LE.ICVPRT_NUMNAMES.AND.
     +			NCVPRT_PROT_NUM(I) .NE. LN)
		I=I+1
	      ENDDO
C
CP.  .  .  .  .  IF no such entry found THEN
C
	      IF(I.GT.ICVPRT_NUMNAMES)THEN
C
CP.  .  .  .  .  .  IF room left in table THEN
C
		IF(ICVPRT_NUMNAMES.LT.ICPPRT_MAXNAMES)THEN
C
CP.  .  .  .  .  .  .  add new entry
C
		  ICVPRT_NUMNAMES=ICVPRT_NUMNAMES+1
		  NCVPRT_PROT_NUM(ICVPRT_NUMNAMES)=LN
		  CCVPRT_PROTNAM(ICVPRT_NUMNAMES)=CLV_SCR(1:LN2)
		  NCVPRT_PROTNAMLEN(ICVPRT_NUMNAMES)=
     +				MIN(LN2,LEN(CCVPRT_PROTNAM(1)))
C
CP.  .  .  .  .  .  ELSE
C
		ELSE
C
CP.  .  .  .  .  .  .  report table overflow error
C
		  ILV_MSGVEC(0)=1
		  ILV_MSGVEC(1)=%LOC(ETH__PRTTBLFUL)
		  CALL SYS$PUTMSG(ILV_MSGVEC,,,)
		  ILV_STATUS=-2
		  ILV_ENTRIES=ILV_ENTRIES-1
C
CP.  .  .  .  .  .  ENDIF
C
		ENDIF
C
CP.  .  .  .  .  ELSE
C
	      ELSE
C
CP.  .  .  .  .  .  overwrite existing protocol name with new name
C
		CCVPRT_PROTNAM(I)=CLV_SCR(1:LN2)
		NCVPRT_PROTNAMLEN(I)=MIN(LN2,LEN(CCVPRT_PROTNAM(1)))
C
CP.  .  .  .  .  ENDIF
C
	      ENDIF
C
CP.  .  .  .  ENDIF
C
	    ENDIF
C
CP.  .  .  ENDDO
C
	  ENDDO
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ENDIF
C
      ENDIF
C
CP.  close file
C
      INQUIRE(UNIT=47,NAME=CLV_FILE)
      CALL STR$TRIM(CLV_FILE,CLV_FILE,LN)
      CLOSE(UNIT=47)
C
CP.  report number of entries read from file
C
      IF(ILV_ENTRIES.NE.0)THEN
	CALL SYS$FAO('!UL entries read from !AS',LN2,
     +		     CLV_SCR,%VAL(ILV_ENTRIES),CLV_FILE(1:LN))
	CALL LIB$PUT_OUTPUT(CLV_SCR(1:LN2))
      ENDIF
C
CP.  RETURN
C
      LOAD=1
      RETURN
C
CP END LOAD
C
      END
      INTEGER*4 FUNCTION SAVE(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for SAVE
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      CHARACTER*1 CLV_FLTMOD(2)/'E','O'/
      CHARACTER*132 CLV_SCR, CLV_FILE
      INTEGER*2 LN, LN2
      INTEGER*4 I, ILV_ENTRIES
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ADDR.INC'
      INCLUDE 'ETHERMON_FILTER.INC'
      INCLUDE 'ETHERMON_PROT.INC'
      INCLUDE 'ETHERMON_PRS.INC'
C
CP BEGIN SAVE
CP.  IF target is unspecified or forced as addresses THEN
C
      ILV_ENTRIES=0
      IF(BCVPRS_FLTMOD.EQ.0.OR.
     +	 BCVPRS_FLTMOD.EQ.1)THEN
C
CP.  .  open the output file
C
	OPEN(UNIT=47,FILE=CCVPRS_FILESPEC(1:NCVPRS_FILESPECLEN),
     +	     STATUS='NEW',DEFAULTFILE='ETHERMON.ETH_ADDRESSES',
     +	     FORM='FORMATTED',CARRIAGECONTROL='LIST')
C
CP.  .  DO for all entries in address table
C
	DO I=0,ICPADR_MAXADDRS
C
CP.  .  .  IF entry defined THEN
C
	  IF(LCVADR_USED(I))THEN
	    ILV_ENTRIES=ILV_ENTRIES+1
C
CP.  .  .  .  write entry to file
C
	    CALL OTS$CVT_L_TZ(SCVADR_ADDR(I).NUM(2),CLV_SCR(1:4),
     +			      %VAL(4),%VAL(2))
	    CALL OTS$CVT_L_TZ(SCVADR_ADDR(I).LNUM,CLV_SCR(5:12),%VAL(8))
	    LN=12
	    IF(NCVADR_DECNETNAMLEN(I).NE.0)THEN
	      CLV_SCR(13:15)=' D:'
	      CLV_SCR(16:)=CCVADR_DECNETNAM(I)
	      LN=15+NCVADR_DECNETNAMLEN(I)
	    ENDIF
	    IF(NCVADR_INETNAMLEN(I).NE.0)THEN
	      CLV_SCR(LN+1:LN+3)=' I:'
	      CALL SYS$FAO('!UB.!UB.!UB.!UB''',LN2,CLV_SCR(LN+4:),
     +			   %VAL(IAND(ICVADR_INETADR(I),'FF'X)),
     +			   %VAL(IAND(ISHFT(ICVADR_INETADR(I),-8),'FF'X)),
     +			   %VAL(IAND(ISHFT(ICVADR_INETADR(I),-16),'FF'X)),
     +			   %VAL(IAND(ISHFT(ICVADR_INETADR(I),-24),'FF'X)))
	      LN=LN+3+LN2
	      CLV_SCR(LN+1:)=CCVADR_INETNAM(I)
	      LN=LN+NCVADR_INETNAMLEN(I)+1
	      CLV_SCR(LN:LN)=''''
	    ENDIF
	    IF(NCVADR_MISCNAMLEN(I).NE.0)THEN
	      CLV_SCR(LN+1:LN+3)=' M:'
	      CLV_SCR(LN+4:)=CCVADR_MISCNAM(I)
	      LN=LN+NCVADR_MISCNAMLEN(I)+3
	    ENDIF
	    WRITE(47,'(A)') CLV_SCR(1:LN)
C
CP.  .  .  ENDIF
C
	  ENDIF
C
CP.  .  ENDDO
C
	ENDDO
C
CP.  ELSEIF target is forced as filters THEN
C
      ELSEIF(BCVPRS_FLTMOD.EQ.2)THEN
C
CP.  .  open the output file
C
	OPEN(UNIT=47,FILE=CCVPRS_FILESPEC(1:NCVPRS_FILESPECLEN),
     +	     STATUS='NEW',DEFAULTFILE='ETHERMON.ETH_FILTERS',
     +	     FORM='FORMATTED',CARRIAGECONTROL='LIST')
C
CP.  .  DO for all entries in filters table
C
	DO I=1,ICVFLT_NUMFILTERS
C
CP.  .  .  IF entry defined THEN
C
	  IF(SCVFLT_FILTERS(I).ENABLE .NE.0)THEN
	    ILV_ENTRIES=ILV_ENTRIES+1
C
CP.  .  .  .  write entry to file
C
	    LN=-1
	    CLV_SCR=' '
	    IF(SCVFLT_FILTERS(I).PROT_ACT .NE.0)THEN
	      CLV_SCR(1:1)=CLV_FLTMOD(SCVFLT_FILTERS(I).PROT_ACT)
	      CLV_SCR(2:3)='P:'
	      CALL OTS$CVT_L_TZ(SCVFLT_FILTERS(I).W_PROTOCOL,
     +				CLV_SCR(4:7),%VAL(4),%VAL(2))
	      LN=7
	    ENDIF
	    IF(SCVFLT_FILTERS(I).SRC_ACT .NE.0)THEN
	      CLV_SCR(LN+2:LN+2)=CLV_FLTMOD(SCVFLT_FILTERS(I).SRC_ACT)
	      CLV_SCR(LN+3:LN+4)='S:'
	      CALL OTS$CVT_L_TZ(SCVFLT_FILTERS(I).LSOURCE4,
     +				CLV_SCR(LN+5:LN+8),%VAL(4),%VAL(2))
	      CALL OTS$CVT_L_TZ(SCVFLT_FILTERS(I).LSOURCE,
     +				CLV_SCR(LN+9:LN+16),%VAL(8))
	      LN=LN+16
	    ENDIF
	    IF(SCVFLT_FILTERS(I).DST_ACT .NE.0)THEN
	      CLV_SCR(LN+2:LN+2)=CLV_FLTMOD(SCVFLT_FILTERS(I).DST_ACT)
	      CLV_SCR(LN+3:LN+4)='D:'
	      CALL OTS$CVT_L_TZ(SCVFLT_FILTERS(I).LDESTINATION4,
     +				CLV_SCR(LN+5:LN+8),%VAL(4),%VAL(2))
	      CALL OTS$CVT_L_TZ(SCVFLT_FILTERS(I).LDESTINATION,
     +				CLV_SCR(LN+9:LN+16),%VAL(8))
	      LN=LN+16
	    ENDIF
	    WRITE(47,'(A)') CLV_SCR(1:LN)
C
CP.  .  .  ENDIF
C
	  ENDIF
C
CP.  .  ENDDO
C
	ENDDO
C
CP.  ELSEIF target is forced as protocols THEN
C
      ELSEIF(BCVPRS_FLTMOD.EQ.3)THEN
C
CP.  .  open the output file
C
	OPEN(UNIT=47,FILE=CCVPRS_FILESPEC(1:NCVPRS_FILESPECLEN),
     +	     STATUS='NEW',DEFAULTFILE='ETHERMON.ETH_PROTOCOLS',
     +	     FORM='FORMATTED',CARRIAGECONTROL='LIST')
C
CP.  .  DO for all entries in protocol table
C
	DO I=1,ICVPRT_NUMNAMES
C
CP.  .  .  write entry to file
C
	  ILV_ENTRIES=ILV_ENTRIES+1
	  WRITE(47,'(Z4.4,X,A)') NCVPRT_PROT_NUM(I),
     +				 CCVPRT_PROTNAM(I)(1:NCVPRT_PROTNAMLEN(I))
C
CP.  .  ENDDO
C
	ENDDO
C
CP.  ENDIF
C
      ENDIF
C
CP.  close file
C
      INQUIRE(UNIT=47,NAME=CLV_FILE)
      CALL STR$TRIM(CLV_FILE,CLV_FILE,LN)
      CLOSE(UNIT=47)
C
CP.  report number of entries written to file
C
      CALL SYS$FAO('!UL entries written to !AS',LN2,
     +		   CLV_SCR,%VAL(ILV_ENTRIES),CLV_FILE(1:LN))
      CALL LIB$PUT_OUTPUT(CLV_SCR(1:LN2))
C
CP.  RETURN
C
      SAVE=1
      RETURN
C
CP END SAVE
C
      END
      INTEGER*4 FUNCTION SET_ADDRNAM(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for SET_ADDRNAM
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      CHARACTER*10 CLV_REPLY
      INTEGER*2 LN
      INTEGER*4 I, ILV_MSGVEC(0:1)
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ADDR.INC'
      INCLUDE 'ETHERMON_PRS.INC'
C
C   declare local procedures
C
      INTEGER*4 FIND_ADDRSLOT
      EXTERNAL	FIND_ADDRSLOT
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__ADRTBLFUL
C
CP BEGIN SET_ADDRNAM
CP.  search address table for pre-existing entry with this number
C
      I=FIND_ADDRSLOT(NCVPRS_ADDRNUM,.TRUE.)
C
CP.  IF room left in table THEN
C
      IF(I.GE.0)THEN
C
CP.  .  IF entry not filled in THEN
C
	IF(NCVADR_MISCNAMLEN(I).EQ.0)THEN
C
CP.  .  .  add new entry
C
	  IF(CCVPRS_FILESPEC(1:1).EQ.'"')THEN
	    CCVADR_MISCNAM(I)=
     +			CCVPRS_FILESPEC(2:NCVPRS_FILESPECLEN-1)
	    NCVADR_MISCNAMLEN(I)=MIN(NCVPRS_FILESPECLEN-2,24)
	  ELSE
	    CCVADR_MISCNAM(I)=CCVPRS_FILESPEC
	    NCVADR_MISCNAMLEN(I)=MIN(NCVPRS_FILESPECLEN,24)
	  ENDIF
C
CP.  .  ELSE
C
	ELSE
C
CP.  .  .  ask if user wants to modify existing entry
C
	  CALL LIB$GET_COMMAND(CLV_REPLY,
     +	'Entry already exists with that address; overwrite (Y/N) ? ',LN)
C
CP.  .  .  IF user says yes THEN
C
	  IF(LN.GE.1.AND.
     +	     (CLV_REPLY(1:1).EQ.'Y'.OR.
     +	      CLV_REPLY(1:1).EQ.'y'))THEN
C
CP.  .  .  .  replace name field of existing entry with user-supplied text
C
	    IF(CCVPRS_FILESPEC(1:1).EQ.'"')THEN
	      CCVADR_MISCNAM(I)=CCVPRS_FILESPEC(2:NCVPRS_FILESPECLEN-1)
	      NCVADR_MISCNAMLEN(I)=MIN(NCVPRS_FILESPECLEN-2,23)
	    ELSE
	      CCVADR_MISCNAM(I)=CCVPRS_FILESPEC
	      NCVADR_MISCNAMLEN(I)=MIN(NCVPRS_FILESPECLEN,23)
	    ENDIF
C
CP.  .  .  ENDIF
C
	  ENDIF
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ELSE
C
      ELSE
C
CP.  .  report table overflow error
C
	ILV_MSGVEC(0)=1
	ILV_MSGVEC(1)=%LOC(ETH__ADRTBLFUL)
	CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  ENDIF
C
      ENDIF
C
CP.  RETURN
C
      SET_ADDRNAM=1
      RETURN
C
CP END SET_ADDRNAM
C
      END
      INTEGER*4 FUNCTION SET_ADDRNAM_AUTO(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for SET_ADDRNAM_AUTO
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ETH.INC'
      INCLUDE 'ETHERMON_PRS.INC'
C
CP BEGIN SET_ADDRNAM_AUTO
CP.  set automatic address/name determination flag
C
      LCVETH_AUTOADDR=ICVPRS_NUMBER
C
CP.  RETURN
C
      SET_ADDRNAM_AUTO=1
      RETURN
C
CP END SET_ADDRNAM_AUTO
C
      END
      INTEGER*4 FUNCTION SET_FILTER(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for SET_FILTER
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      INTEGER*4 ILV_MSGVEC(0:3)
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_FILTER.INC'
      INCLUDE 'ETHERMON_PRS.INC'
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__INVFLTNUM, ETH__FLTMODENA, ETH__FLTMODDIS
C
CP BEGIN SET_FILTER
CP.  IF filter slot is undefined THEN
C
      IF(ICVPRS_NUMBER.GT.ICVFLT_NUMFILTERS.OR.
     +	 ICVPRS_NUMBER.LE.0.OR.
     +	 SCVFLT_FILTERS(ICVPRS_NUMBER).ENABLE .EQ.0)THEN
C
CP.  .  report error
C
	ILV_MSGVEC(0)=1
	ILV_MSGVEC(1)=%LOC(ETH__INVFLTNUM)
	CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  ELSE
C
      ELSE
C
CP.  .  IF enable/disable flag specified THEN
C
	IF(BCVPRS_FLTMOD.NE.0)THEN
C
CP.  .  .  modify the filter's flag
C
	  SCVFLT_FILTERS(ICVPRS_NUMBER).ENABLE=BCVPRS_FLTMOD
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  .  IF protocol type being added/modified THEN
C
	IF(BCVPRS_PRTMOD.GT.0)THEN
C
CP.  .  .  store new values
C
	  SCVFLT_FILTERS(ICVPRS_NUMBER).PROT_ACT=BCVPRS_PRTMOD
	  SCVFLT_FILTERS(ICVPRS_NUMBER).W_PROTOCOL=NCVPRS_PROTNUM
C
CP.  .  ELSEIF protocol type being cleared THEN
C
	ELSEIF(BCVPRS_PRTMOD.LT.0)THEN
C
CP.  .  .  mark protocol type as unused
C
	  SCVFLT_FILTERS(ICVPRS_NUMBER).PROT_ACT=0
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  .  IF source address being added/modified THEN
C
	IF(BCVPRS_SRCMOD.GT.0)THEN
C
CP.  .  .  store new values
C
	  SCVFLT_FILTERS(ICVPRS_NUMBER).SRC_ACT=BCVPRS_SRCMOD
	  SCVFLT_FILTERS(ICVPRS_NUMBER).W_SOURCE(0)=NCVPRS_SRCADR(0)
	  SCVFLT_FILTERS(ICVPRS_NUMBER).W_SOURCE(1)=NCVPRS_SRCADR(1)
	  SCVFLT_FILTERS(ICVPRS_NUMBER).W_SOURCE(2)=NCVPRS_SRCADR(2)
C
CP.  .  ELSEIF source address being cleared THEN
C
	ELSEIF(BCVPRS_SRCMOD.LT.0)THEN
C
CP.  .  .  mark source address as unused
C
	  SCVFLT_FILTERS(ICVPRS_NUMBER).SRC_ACT=0
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  .  IF destination address being added/modified THEN
C
	IF(BCVPRS_DSTMOD.GT.0)THEN
C
CP.  .  .  store new values
C
	  SCVFLT_FILTERS(ICVPRS_NUMBER).DST_ACT=BCVPRS_DSTMOD
	  SCVFLT_FILTERS(ICVPRS_NUMBER).W_DESTINATION(0)=NCVPRS_DSTADR(0)
	  SCVFLT_FILTERS(ICVPRS_NUMBER).W_DESTINATION(1)=NCVPRS_DSTADR(1)
	  SCVFLT_FILTERS(ICVPRS_NUMBER).W_DESTINATION(2)=NCVPRS_DSTADR(2)
C
CP.  .  ELSEIF destination address being cleared THEN
C
	ELSEIF(BCVPRS_DSTMOD.LT.0)THEN
C
CP.  .  .  mark destination address as unused
C
	  SCVFLT_FILTERS(ICVPRS_NUMBER).DST_ACT=0
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  .  IF some fields still left defined THEN
C
	IF(SCVFLT_FILTERS(ICVPRS_NUMBER).PROT_ACT .NE.0.OR.
     +	   SCVFLT_FILTERS(ICVPRS_NUMBER).SRC_ACT .NE.0.OR.
     +	   SCVFLT_FILTERS(ICVPRS_NUMBER).DST_ACT .NE.0)THEN
C
CP.  .  .  report filter as modified
C
	  ILV_MSGVEC(0)=3
	  IF(SCVFLT_FILTERS(ICVPRS_NUMBER).ENABLE .EQ.1)THEN
	    ILV_MSGVEC(1)=%LOC(ETH__FLTMODENA)
	  ELSE
	    ILV_MSGVEC(1)=%LOC(ETH__FLTMODDIS)
	  ENDIF
	  ILV_MSGVEC(2)=1
	  ILV_MSGVEC(3)=ICVPRS_NUMBER
	  CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  .  ELSE
C
	ELSE
C
CP.  .  .  delete the (now null function) filter
C
	  SCVFLT_FILTERS(ICVPRS_NUMBER).ENABLE=-1	! to bypass error trap
	  CALL CLEAR_FILTER(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,%VAL(ICVPRS_NUMBER),ILA_PARAM)
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ENDIF
C
      ENDIF
C
CP.  RETURN
C
      SET_FILTER=1
      RETURN
C
CP END SET_FILTER
C
      END
      INTEGER*4 FUNCTION SET_FILTER_SIZE(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for SET_FILTER_SIZE
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_FILTER.INC'
      INCLUDE 'ETHERMON_PRS.INC'
C
CP BEGIN SET_FILTER_SIZE
CP.  set ENABLE flag if a new value has been specified
C
      IF(NCVPRS_PROTNUM .NE.0)THEN
	LCVFLT_ENB_SIZE=NCVPRS_PROTNUM
      ENDIF
C
CP.  set MAXIMUM value if a new value has been specified
C
      IF(NCVPRS_SRCADR(0) .NE.0)THEN
	NCVFLT_MAXSIZE=NCVPRS_SRCADR(0)
      ENDIF
C
CP.  set MINIMUM value if a new value has been specified
C
      IF(NCVPRS_DSTADR(0) .NE.0)THEN
	NCVFLT_MINSIZE=NCVPRS_DSTADR(0)
      ENDIF
C
CP.  RETURN
C
      SET_FILTER_SIZE=1
      RETURN
C
CP END SET_FILTER_SIZE
C
      END
      INTEGER*4 FUNCTION SET_INTERVAL(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for SET_INTERVAL
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      INTEGER*4 ILV_MSGVEC(0:3)
C
C   declare local COMMON's
C
      INTEGER*4 ICVDSP_INTERVAL/4/, QCVDSP_INTERVAL(2)/-40000000,-1/
      LOGICAL*1 LCVDSP_EXIT
      COMMON /DISPLAY/ ICVDSP_INTERVAL, QCVDSP_INTERVAL, LCVDSP_EXIT
      INCLUDE 'ETHERMON_PRS.INC'
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__INVDSPINT
C
CP BEGIN SET_INTERVAL
CP.  IF specified value is valid THEN
C
      IF(%LOC(ILA_NUMBER).GE.2.AND.
     +   %LOC(ILA_NUMBER).LE.59)THEN
C
CP.  .  store value as display update interval
C
	ICVDSP_INTERVAL=%LOC(ILA_NUMBER)
	CALL LIB$EMUL(ICVDSP_INTERVAL,-10000000,0,QCVDSP_INTERVAL)
C
CP.  ELSE
C
      ELSE
C
CP.  .  report error
C
	ILV_MSGVEC(0)=3
	ILV_MSGVEC(1)=%LOC(ETH__INVDSPINT)
	ILV_MSGVEC(2)=1
	ILV_MSGVEC(3)=%LOC(ILA_NUMBER)
	CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  ENDIF
C
      ENDIF
C
CP.  RETURN
C
      SET_INTERVAL=1
      RETURN
C
CP END SET_INTERVAL
C
      END
      INTEGER*4 FUNCTION SET_PROTNAM(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for SET_PROTNAM
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      CHARACTER*10 CLV_REPLY
      INTEGER*2 LN
      INTEGER*4 I, ILV_MSGVEC(0:1)
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_PROT.INC'
      INCLUDE 'ETHERMON_PRS.INC'
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__PRTTBLFUL
C
CP BEGIN SET_PROTNAM
CP.  search protocol table for pre-existing entry with this number
C
      I=1
      DO WHILE (I.LE.ICVPRT_NUMNAMES.AND.
     +		NCVPRT_PROT_NUM(I) .NE. NCVPRS_PROTNUM)
	I=I+1
      ENDDO
C
CP.  IF no such entry found THEN
C
      IF(I.GT.ICVPRT_NUMNAMES)THEN
C
CP.  .  IF room left in table THEN
C
	IF(ICVPRT_NUMNAMES.LT.ICPPRT_MAXNAMES)THEN
C
CP.  .  .  add new entry
C
	  ICVPRT_NUMNAMES=ICVPRT_NUMNAMES+1
	  NCVPRT_PROT_NUM(ICVPRT_NUMNAMES)=NCVPRS_PROTNUM
	  IF(CCVPRS_FILESPEC(1:1).EQ.'"')THEN
	    CCVPRT_PROTNAM(ICVPRT_NUMNAMES)=
     +			CCVPRS_FILESPEC(2:NCVPRS_FILESPECLEN-1)
	    NCVPRT_PROTNAMLEN(ICVPRT_NUMNAMES)=MIN(NCVPRS_FILESPECLEN-2,24)
	  ELSE
	    CCVPRT_PROTNAM(ICVPRT_NUMNAMES)=CCVPRS_FILESPEC
	    NCVPRT_PROTNAMLEN(ICVPRT_NUMNAMES)=MIN(NCVPRS_FILESPECLEN,24)
	  ENDIF
C
CP.  .  ELSE
C
	ELSE
C
CP.  .  .  report table overflow error
C
	  ILV_MSGVEC(0)=1
	  ILV_MSGVEC(1)=%LOC(ETH__PRTTBLFUL)
	  CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ELSE
C
      ELSE
C
CP.  .  ask if user wants to modify existing entry
C
	CALL LIB$GET_COMMAND(CLV_REPLY,
     +	'Entry already exists with that protocol; replace (Y/N) ? ',LN)
C
CP.  .  IF user says yes THEN
C
	IF(LN.GE.1.AND.
     +	   (CLV_REPLY(1:1).EQ.'Y'.OR.
     +	    CLV_REPLY(1:1).EQ.'y'))THEN
C
CP.  .  .  replace name field of existing entry with user-supplied text
C
	  IF(CCVPRS_FILESPEC(1:1).EQ.'"')THEN
	    CCVPRT_PROTNAM(I)=CCVPRS_FILESPEC(2:NCVPRS_FILESPECLEN-1)
	    NCVPRT_PROTNAMLEN(I)=MIN(NCVPRS_FILESPECLEN-2,24)
	  ELSE
	    CCVPRT_PROTNAM(I)=CCVPRS_FILESPEC
	    NCVPRT_PROTNAMLEN(I)=MIN(NCVPRS_FILESPECLEN,24)
	  ENDIF
C
CP.  .  ENDIF
C
	ENDIF
C
CP.  ENDIF
C
      ENDIF
C
CP.  RETURN
C
      SET_PROTNAM=1
      RETURN
C
CP END SET_PROTNAM
C
      END
      INTEGER*4 FUNCTION SHOW_ADDRESS_NAMES(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for SHOW_ADDRESS_NAMES
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      CHARACTER*8 CLV_SCR
      CHARACTER*80 CLV_LINE
      INTEGER*2 LN
      INTEGER*4 I
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ADDR.INC'
C
CP BEGIN SHOW_ADDRESS_NAMES
CP.  report total count of names displayed
C
      CALL SYS$FAO('Total of !UL names defined.',LN,CLV_LINE,
     +			%VAL(ICVADR_NUMADDRS))
      CALL LIB$PUT_OUTPUT(CLV_LINE(1:LN))
C
CP.  DO for all defined address/name pairs
C
      DO I=0,ICPADR_MAXADDRS
C
CP.  .  format and print pair to terminal
C
	IF(LCVADR_USED(I))THEN
	  CLV_LINE=' '
	  CALL OTS$CVT_L_TZ(SCVADR_ADDR(I).LNUM,CLV_SCR,%VAL(8))
	  CLV_LINE(1:11)=CLV_SCR(7:8)//'-'//CLV_SCR(5:6)//
     +			'-'//CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	  CALL OTS$CVT_L_TZ(SCVADR_ADDR(I).NUM(2),CLV_SCR(1:4),
     +			      %VAL(4),%VAL(2))
	  CLV_LINE(12:17)='-'//CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	  LN=17
	  IF(NCVADR_DECNETNAMLEN(I).NE.0)THEN
	    CLV_LINE(19:25)=CCVADR_DECNETNAM(I)(1:NCVADR_DECNETNAMLEN(I))
	    LN=18+NCVADR_DECNETNAMLEN(I)
	  ENDIF
	  IF(NCVADR_INETNAMLEN(I).NE.0)THEN
	    CLV_LINE(26:49)=CCVADR_INETNAM(I)(1:NCVADR_INETNAMLEN(I))
	    LN=25+NCVADR_INETNAMLEN(I)
	  ENDIF
	  IF(NCVADR_MISCNAMLEN(I).NE.0)THEN
	    CLV_LINE(50:)=CCVADR_MISCNAM(I)(1:NCVADR_MISCNAMLEN(I))
	    LN=49+NCVADR_MISCNAMLEN(I)
	  ENDIF
	  CALL LIB$PUT_OUTPUT(CLV_LINE(1:LN))
	ENDIF
C
CP.  ENDDO
C
      ENDDO
C
CP.  report total count of names displayed
C
      CALL SYS$FAO('Total of !UL names defined.',LN,CLV_LINE,
     +			%VAL(ICVADR_NUMADDRS))
      CALL LIB$PUT_OUTPUT(CLV_LINE(1:LN))
C
CP.  RETURN
C
      SHOW_ADDRESS_NAMES=1
      RETURN
C
CP END SHOW_ADDRESS_NAMES
C
      END
      INTEGER*4 FUNCTION SHOW_FILTER(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for SHOW_FILTER
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      CHARACTER*3 CLV_MODE(0:2)/'dis','ENA','dis'/
      CHARACTER*8 CLV_SCR
      CHARACTER*8 CLV_FLT(2)/' EXCLUDE',' ONLY'/
      CHARACTER*80 CLV_LINE
      INTEGER*2 LN
      INTEGER*4 I, J
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ADDR.INC'
      INCLUDE 'ETHERMON_FILTER.INC'
      INCLUDE 'ETHERMON_PROT.INC'
C
C   declare local procedures
C
      INTEGER*4 FIND_ADDRSLOT
      EXTERNAL	FIND_ADDRSLOT
C
CP BEGIN SHOW_FILTER
CP.  print the size filter to terminal
C
      CALL SYS$FAO('size limit: !AS  min=!UW  max=!UW',LN,CLV_LINE,
     +		   CLV_MODE(LCVFLT_ENB_SIZE),%VAL(NCVFLT_MINSIZE),
     +		   %VAL(NCVFLT_MAXSIZE))
      CALL LIB$PUT_OUTPUT(CLV_LINE(1:LN))
C
CP.  DO for all defined filters
C
      DO I=1,ICVFLT_NUMFILTERS
C
CP.  .  print filter description to terminal
C
	IF(SCVFLT_FILTERS(I).ENABLE .EQ.0)THEN
	  CONTINUE	! skip deleted entries
	ELSE
	  CALL SYS$FAO('!2UL: !AS',,CLV_LINE,%VAL(I),
     +			 CLV_MODE(SCVFLT_FILTERS(I).ENABLE))
	  IF(SCVFLT_FILTERS(I).PROT_ACT .NE.0)THEN
	    CALL OTS$CVT_L_TZ(SCVFLT_FILTERS(I).W_PROTOCOL,CLV_SCR(1:4),
     +				%VAL(4),%VAL(2))
	    CLV_LINE(8:23)=CLV_FLT(SCVFLT_FILTERS(I).PROT_ACT)//
     +				' P:'//CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	    LN=23
	    J=1
	    DO WHILE (J.LE.ICVPRT_NUMNAMES.AND.
     +		      NCVPRT_PROT_NUM(J).NE. SCVFLT_FILTERS(I).W_PROTOCOL)
	      J=J+1
	    ENDDO
	    IF(J.LE.ICVPRT_NUMNAMES)THEN
	      CLV_LINE(24:)=' '//CCVPRT_PROTNAM(J)(1:
     +					NCVPRT_PROTNAMLEN(J))
	      LN=LN+1+NCVPRT_PROTNAMLEN(J)
	    ENDIF
	    CALL LIB$PUT_OUTPUT(CLV_LINE(1:LN))
	    CLV_LINE=' '
	  ENDIF
	  IF(SCVFLT_FILTERS(I).SRC_ACT .NE.0)THEN
	    CALL OTS$CVT_L_TZ(SCVFLT_FILTERS(I).LSOURCE,CLV_SCR,%VAL(8))
	    CLV_LINE(8:29)=CLV_FLT(SCVFLT_FILTERS(I).SRC_ACT)//
     +				' S:'//CLV_SCR(7:8)//'-'//CLV_SCR(5:6)//
     +				'-'//CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	    CALL OTS$CVT_L_TZ(SCVFLT_FILTERS(I).LSOURCE4,CLV_SCR(1:4),
     +				%VAL(4),%VAL(2))
	    CLV_LINE(30:35)='-'//CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	    LN=35
	    J=FIND_ADDRSLOT(SCVFLT_FILTERS(I).LSOURCE,.FALSE.)
	    IF(J.GE.0)THEN
	      IF(NCVADR_DECNETNAMLEN(J).NE.0)THEN
		CLV_LINE(36:)=' '//CCVADR_DECNETNAM(J)
		LN=LN+NCVADR_DECNETNAMLEN(J)+1
	      ENDIF
	      IF(NCVADR_INETNAMLEN(J).NE.0)THEN
		CLV_LINE(LN+1:)=' '//CCVADR_INETNAM(J)
		LN=LN+NCVADR_INETNAMLEN(J)+1
	      ENDIF
	      IF(NCVADR_MISCNAMLEN(J).NE.0)THEN
		CLV_LINE(LN+1:)=' '//CCVADR_MISCNAM(J)
		LN=LN+NCVADR_MISCNAMLEN(J)+1
	      ENDIF
	    ENDIF
	    CALL LIB$PUT_OUTPUT(CLV_LINE(1:LN))
	    CLV_LINE=' '
	  ENDIF
	  IF(SCVFLT_FILTERS(I).DST_ACT .NE.0)THEN
	    CALL OTS$CVT_L_TZ(SCVFLT_FILTERS(I).LDESTINATION,
     +			      CLV_SCR,%VAL(8))
	    CLV_LINE(8:29)=CLV_FLT(SCVFLT_FILTERS(I).DST_ACT)//
     +				' D:'//CLV_SCR(7:8)//'-'//CLV_SCR(5:6)//
     +				'-'//CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	    CALL OTS$CVT_L_TZ(SCVFLT_FILTERS(I).LDESTINATION4,
     +			      CLV_SCR(1:4),%VAL(4),%VAL(2))
	    CLV_LINE(30:35)='-'//CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	    LN=35
	    J=FIND_ADDRSLOT(SCVFLT_FILTERS(I).LDESTINATION,.FALSE.)
	    IF(J.GE.0)THEN
	      IF(NCVADR_DECNETNAMLEN(J).NE.0)THEN
		CLV_LINE(36:)=' '//CCVADR_DECNETNAM(J)
		LN=LN+NCVADR_DECNETNAMLEN(J)+1
	      ENDIF
	      IF(NCVADR_INETNAMLEN(J).NE.0)THEN
		CLV_LINE(LN+1:)=' '//CCVADR_INETNAM(J)
		LN=LN+NCVADR_INETNAMLEN(J)+1
	      ENDIF
	      IF(NCVADR_MISCNAMLEN(J).NE.0)THEN
		CLV_LINE(LN+1:)=' '//CCVADR_MISCNAM(J)
		LN=LN+NCVADR_MISCNAMLEN(J)+1
	      ENDIF
	    ENDIF
	    CALL LIB$PUT_OUTPUT(CLV_LINE(1:LN))
	    CLV_LINE=' '
	  ENDIF
	ENDIF
C
CP.  ENDDO
C
      ENDDO
C
CP.  RETURN
C
      SHOW_FILTER=1
      RETURN
C
CP END SHOW_FILTER
C
      END
      INTEGER*4 FUNCTION SHOW_PORT_STATUS(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for SHOW_PORT_STATUS
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      BYTE	BLV_DEVNUMS(8)
      CHARACTER*3 CLV_MODES(-1:1)/'???','ON','OFF'/
      CHARACTER*4 CLV_RFL(0:2)/'CRC','FRAM','MLEN'/,
     +		  CLV_SFL(0:5)/'RTRY','LCAR','???','???','MLEN','LCOL'/
      CHARACTER*7 CLV_DEVNAMS(8)
      CHARACTER*8 CLV_CON(-1:1)/'???','NORMAL','LOOPBACK'/
      CHARACTER*80 CLV_BUF
      INTEGER*2 LN
      INTEGER*4 I
      DATA BLV_DEVNUMS/13,14,22,24,25,29,33,0/
      DATA CLV_DEVNAMS/'NI','DEUNA','DEQNA','DESVA','DELUA','DEBNA',
     +		       'DELQA','unknown'/
C
C   declare local COMMON's
C
      CHARACTER*32 CCVNAM_DEVNAM, CCVNAM_SYSNAM
      INTEGER*2 NCVNAM_DEVNAMLEN, NCVNAM_SYSNAMLEN
      COMMON /NAMES/ CCVNAM_DEVNAM, CCVNAM_SYSNAM,
     +		NCVNAM_DEVNAMLEN, NCVNAM_SYSNAMLEN
      INCLUDE 'ETHERMON_ETH.INC'
      INCLUDE 'ETHERMON_PRS.INC'
C
CP BEGIN SHOW_PORT_STATUS
CP.  get the current port status
C
      CALL GET_PORT_STATUS
      CALL GET_PORT_COUNTERS(ICVPRS_NUMBER)
C
CP.  print port status
C
      CALL LIB$PUT_OUTPUT('Ethernet port '//
     +			  CCVNAM_DEVNAM(1:NCVNAM_DEVNAMLEN)//
     +			  ' status:')
      I=1
      DO WHILE (BLV_DEVNUMS(I).NE.0.AND.
     +		BLV_DEVNUMS(I).NE. SCVETH_CHARBUF.P1_TYPE)
	I=I+1
      ENDDO
      CALL SYS$FAO(
     +'Device=!AS  buffer size=!UW  status=!XB  errsum=!XB',LN,CLV_BUF,
     +		   CLV_DEVNAMS(I),%VAL(SCVETH_CHARBUF.P1_MSGBUF),
     +		   %VAL(SCVETH_CHARBUF.P1_STATUS),
     +		   %VAL(SCVETH_CHARBUF.P1_ERRSUM))
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:LN))
      WRITE(CLV_BUF,9010) BCVETH_HWA
 9010 FORMAT('hardware address: ',Z2.2,5('-',Z2.2))
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:35))
      WRITE(CLV_BUF,9012) BCVETH_PHA
 9012 FORMAT('physical port address: ',Z2.2,5('-',Z2.2))
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:40))
      WRITE(CLV_BUF,9011) ICVETH_BUS, ICVETH_BFN, ICVETH_BSZ
 9011 FORMAT('receive buffer size',I5,'(',I2,
     +		')   device buffer size',I5)
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:54))
      CALL LIB$PUT_OUTPUT('modes: echo='//CLV_MODES(ICVETH_EKO))
      CALL LIB$PUT_OUTPUT('       data chaining='//
     +				CLV_MODES(ICVETH_DCH))
      CALL LIB$PUT_OUTPUT('       CRC='//CLV_MODES(ICVETH_CRC))
      CALL LIB$PUT_OUTPUT('       controller='//CLV_CON(ICVETH_CON))
      CALL LIB$PUT_OUTPUT('       padding='//CLV_MODES(ICVETH_PAD))
C
CP.  print port counters
C
      CALL SYS$FAO('Counters:   !5UW seconds since last zeroed',LN,
     +		   CLV_BUF,%VAL(NCVETH_C_ZER))
      IF(NCVETH_C_ZER.EQ.-1) CLV_BUF(12:17)='>65534'
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:43))
      CALL SYS$FAO(
     +'!10UL data blocks rcvd, !10UL multicast blocks rcvd',
     +LN,CLV_BUF,%VAL(ICVETH_C_DBR),%VAL(ICVETH_C_MBL))
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:61))
      CALL SYS$FAO(
     +'!10UL data bytes rcvd,  !10UL multicast bytes rcvd',
     +LN,CLV_BUF,%VAL(ICVETH_C_BRC),%VAL(ICVETH_C_MBY))
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:60))
      CALL SYS$FAO(
     +'!10UW data overruns,    !10UW local buffer errors',
     +LN,CLV_BUF,%VAL(NCVETH_C_OVR),%VAL(NCVETH_C_LBE))
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:59))
      CALL SYS$FAO('!10UW receive failure!%S',LN,CLV_BUF,
     +			%VAL(NCVETH_C_RFL(1)))
      IF(NCVETH_C_RFL(1).NE.0)THEN
	CLV_BUF(LN+1:)=', including:'
	LN=LN+12
	DO I=0,2
	  IF(IAND(NCVETH_C_RFL(0),ISHFT(1,I)).NE.0)THEN
	    CLV_BUF(LN+1:)=' '//CLV_RFL(I)
	    LN=LN+5
	  ENDIF
	ENDDO
      ENDIF
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:LN))
      CALL SYS$FAO(
     +'!10UL data blocks sent, !10UL multicast blocks sent',
     +LN,CLV_BUF,%VAL(ICVETH_C_DBS),%VAL(ICVETH_C_MBS))
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:61))
      CALL SYS$FAO(
     +'!10UL data bytes sent,  !10UL multicast bytes sent',
     +LN,CLV_BUF,%VAL(ICVETH_C_BSN),%VAL(ICVETH_C_MSN))
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:60))
      CALL SYS$FAO(
     +'blocks sent: !7UL mult. collision, !7UL one collision',
     +LN,CLV_BUF,%VAL(ICVETH_C_BSM),%VAL(ICVETH_C_BS1))
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:59))
      CALL SYS$FAO(
     +'             !7UL initially deferred',
     +LN,CLV_BUF,%VAL(ICVETH_C_BID))
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:39))
      CALL SYS$FAO('!10UW send failure!%S',LN,CLV_BUF,
     +			%VAL(NCVETH_C_SFL(1)))
      IF(NCVETH_C_SFL(1).NE.0)THEN
	CLV_BUF(LN+1:)=', including:'
	LN=LN+12
	DO I=0,5
	  IF(IAND(NCVETH_C_SFL(0),ISHFT(1,I)).NE.0)THEN
	    CLV_BUF(LN+1:)=' '//CLV_SFL(I)
	    LN=LN+5
	  ENDIF
	ENDDO
      ENDIF
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:LN))
      CALL SYS$FAO(
     +'!10UW collision detect, !10UW unrecognized frame destination',
     +LN,CLV_BUF,%VAL(NCVETH_C_CDC),%VAL(NCVETH_C_UFD))
      IF(NCVETH_C_CDC.EQ.-1) CLV_BUF(5:10)='>65534'
      IF(NCVETH_C_UFD.EQ.-1) CLV_BUF(34:39)='>65534'
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:70))
      CALL SYS$FAO(
     +'buffer unavailable: !6UW system, !6UW user',
     +LN,CLV_BUF,%VAL(NCVETH_C_SBU),%VAL(NCVETH_C_UBU))
      IF(NCVETH_C_SBU.EQ.-1) CLV_BUF(21:26)='>65534'
      IF(NCVETH_C_UBU.EQ.-1) CLV_BUF(36:41)='>65534'
      CALL LIB$PUT_OUTPUT(CLV_BUF(1:46))
C
CP.  RETURN
C
      SHOW_PORT_STATUS=1
      RETURN
C
CP END SHOW_PORT_STATUS
C
      END
      INTEGER*4 FUNCTION SHOW_PROTOCOL_NAMES(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for SHOW_PROTOCOL_NAMES
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      CHARACTER*4 CLV_SCR
      CHARACTER*80 CLV_LINE
      INTEGER*2 LN
      INTEGER*4 I
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_PROT.INC'
C
CP BEGIN SHOW_PROTOCOL_NAMES
CP.  DO for all defined protocol/name pairs
C
      DO I=1,(ICVPRT_NUMNAMES+1)/2
C
CP.  .  format and print 2 pairs to terminal
C
	CLV_LINE=' '
	IF(I.LE.ICPPRT_MINNAMES) CLV_LINE(1:1)='*'
	CALL OTS$CVT_L_TZ(NCVPRT_PROT_NUM(I),CLV_SCR,
     +			  %VAL(4),%VAL(2))
	CLV_LINE(3:7)=CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	CLV_LINE(9:32)=CCVPRT_PROTNAM(I)(1:NCVPRT_PROTNAMLEN(I))
	IF(I+(ICVPRT_NUMNAMES+1)/2.LE.ICVPRT_NUMNAMES)THEN
	  IF(I+(ICVPRT_NUMNAMES+1)/2.LE.ICPPRT_MINNAMES) CLV_LINE(41:41)='*'
	  CALL OTS$CVT_L_TZ(NCVPRT_PROT_NUM(I+(ICVPRT_NUMNAMES+1)/2),
     +			    CLV_SCR,%VAL(4),%VAL(2))
	  CLV_LINE(43:47)=CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	  CLV_LINE(49:72)=CCVPRT_PROTNAM(I+(ICVPRT_NUMNAMES+1)/2)(1:
     +			   NCVPRT_PROTNAMLEN(I+(ICVPRT_NUMNAMES+1)/2))
	  CALL LIB$PUT_OUTPUT(CLV_LINE(1:48+
     +			NCVPRT_PROTNAMLEN(I+(ICVPRT_NUMNAMES+1)/2)))
	ELSE
	  CALL LIB$PUT_OUTPUT(CLV_LINE(1:8+NCVPRT_PROTNAMLEN(I)))
	ENDIF
C
CP.  ENDDO
C
      ENDDO
C
CP.  report total count of names displayed
C
      CALL SYS$FAO('Total of !UL names defined.',LN,CLV_LINE,
     +			%VAL(ICVPRT_NUMNAMES))
      CALL LIB$PUT_OUTPUT(CLV_LINE(1:LN))
C
CP.  RETURN
C
      SHOW_PROTOCOL_NAMES=1
      RETURN
C
CP END SHOW_PROTOCOL_NAMES
C
      END
      INTEGER*4 FUNCTION SUMMARIZE(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for SUMMARIZE
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local COMMON's
C
      INTEGER*4 ICVDSP_INTERVAL/4/, QCVDSP_INTERVAL(2)/-40000000,-1/
      LOGICAL*1 LCVDSP_EXIT
      COMMON /DISPLAY/ ICVDSP_INTERVAL, QCVDSP_INTERVAL, LCVDSP_EXIT
      INCLUDE 'ETHERMON_ADDR.INC'
      INCLUDE 'ETHERMON_ETH.INC'
      INCLUDE 'ETHERMON_FILTER.INC'
      INCLUDE 'ETHERMON_PROT.INC'
      INCLUDE 'ETHERMON_PRS.INC'
      INCLUDE 'ETHERMON_STATS.INC'
C
C   declare local variables
C
      INTEGER*4 ILP_NUMIP
      PARAMETER(ILP_NUMIP=47)
      CHARACTER*8 CLV_SCR
      CHARACTER*18 CLV_TITLE(7)/'Destinations','Protocols','Sizes',
     +		'Sources','Statistics','???','Internet Protocols'/
      CHARACTER*40 CLV_IPNAM(ILP_NUMIP)
      CHARACTER*80 CLV_LINE
      INTEGER*2 LN, NLV_IPNAMLEN(ILP_NUMIP)
      INTEGER*4 ILV_STATUS, I, J, D, B, ILV_IPPRT(ILP_NUMIP),
     +		IDX(0:ICPADR_MAXADDRS), TIDX, QLV_SYSTIM(2),
     +		QLV_OLDSYSTIM(2), ILV_OLD_C_DBR, ILV_OLD_C_BRC,
     +		ILV_OLD_C_MBY, ILV_OLD_C_MBL, ILV_OLD_C_BS1,
     +		ILV_OLD_C_BSM, ILV_OLD_C_BID, ILV_OLD_C_OVR,
     +		ILV_OLD_C_LBE, ILV_OLD_C_XBU
      DATA ILV_IPPRT(1)/1/, NLV_IPNAMLEN(1)/31/,
     +	   CLV_IPNAM(1)/'ICMP (Internet Control Message)'/
      DATA ILV_IPPRT(2)/2/, NLV_IPNAMLEN(2)/32/,
     +	   CLV_IPNAM(2)/'IGMP (Internet Group Management)'/
      DATA ILV_IPPRT(3)/3/, NLV_IPNAMLEN(3)/24/,
     +	   CLV_IPNAM(3)/'GGP (Gateway-to-Gateway)'/
      DATA ILV_IPPRT(4)/5/, NLV_IPNAMLEN(4)/11/,
     +	   CLV_IPNAM(4)/'ST (Stream)'/
      DATA ILV_IPPRT(5)/6/, NLV_IPNAMLEN(5)/26/,
     +	   CLV_IPNAM(5)/'TCP (Transmission Control)'/
      DATA ILV_IPPRT(6)/7/, NLV_IPNAMLEN(6)/3/,
     +	   CLV_IPNAM(6)/'UCL'/
      DATA ILV_IPPRT(7)/8/, NLV_IPNAMLEN(7)/22/,
     +	   CLV_IPNAM(7)/'EGP (Exterior Gateway)'/
      DATA ILV_IPPRT(8)/9/, NLV_IPNAMLEN(8)/30/,
     +	   CLV_IPNAM(8)/'IGP (private interior gateway)'/
      DATA ILV_IPPRT(9)/10/, NLV_IPNAMLEN(9)/32/,
     +	   CLV_IPNAM(9)/'BBN-RCC-MON (BBN RCC Monitoring)'/
      DATA ILV_IPPRT(10)/11/, NLV_IPNAMLEN(10)/31/,
     +	   CLV_IPNAM(10)/'NVP-II (Network Voice Protocol)'/
      DATA ILV_IPPRT(11)/12/, NLV_IPNAMLEN(11)/3/,
     +	   CLV_IPNAM(11)/'PUP'/
      DATA ILV_IPPRT(12)/13/, NLV_IPNAMLEN(12)/5/,
     +	   CLV_IPNAM(12)/'ARGUS'/
      DATA ILV_IPPRT(13)/14/, NLV_IPNAMLEN(13)/5/,
     +	   CLV_IPNAM(13)/'EMCON'/
      DATA ILV_IPPRT(14)/15/, NLV_IPNAMLEN(14)/25/,
     +	   CLV_IPNAM(14)/'XNET (Cross Net Debugger)'/
      DATA ILV_IPPRT(15)/16/, NLV_IPNAMLEN(15)/5/,
     +	   CLV_IPNAM(15)/'Chaos'/
      DATA ILV_IPPRT(16)/17/, NLV_IPNAMLEN(16)/19/,
     +	   CLV_IPNAM(16)/'UDP (User Datagram)'/
      DATA ILV_IPPRT(17)/18/, NLV_IPNAMLEN(17)/18/,
     +	   CLV_IPNAM(17)/'MUX (Multiplexing)'/
      DATA ILV_IPPRT(18)/19/, NLV_IPNAMLEN(18)/37/,
     +	   CLV_IPNAM(18)/'DCN-MEAS (DCN Measurement Subsystems)'/
      DATA ILV_IPPRT(19)/20/, NLV_IPNAMLEN(19)/30/,
     +	   CLV_IPNAM(19)/'HMP (Host Monitoring Protocol)'/
      DATA ILV_IPPRT(20)/21/, NLV_IPNAMLEN(20)/30/,
     +	   CLV_IPNAM(20)/'PRM (Packet Radio Measurement)'/
      DATA ILV_IPPRT(21)/22/, NLV_IPNAMLEN(21)/22/,
     +	   CLV_IPNAM(21)/'XNS-IDP (XEROX NS IDP)'/
      DATA ILV_IPPRT(22)/23/, NLV_IPNAMLEN(22)/7/,
     +	   CLV_IPNAM(22)/'Trunk-1'/
      DATA ILV_IPPRT(23)/24/, NLV_IPNAMLEN(23)/7/,
     +	   CLV_IPNAM(23)/'Trunk-2'/
      DATA ILV_IPPRT(24)/25/, NLV_IPNAMLEN(24)/6/,
     +	   CLV_IPNAM(24)/'Leaf-1'/
      DATA ILV_IPPRT(25)/26/, NLV_IPNAMLEN(25)/6/,
     +	   CLV_IPNAM(25)/'Leaf-2'/
      DATA ILV_IPPRT(26)/27/, NLV_IPNAMLEN(26)/28/,
     +	   CLV_IPNAM(26)/'RDP (Reliable Data Protocol)'/
      DATA ILV_IPPRT(27)/28/, NLV_IPNAMLEN(27)/36/,
     +	   CLV_IPNAM(27)/'IRTP (Internet Reliable Transaction)'/
      DATA ILV_IPPRT(28)/29/, NLV_IPNAMLEN(28)/40/,
     +	   CLV_IPNAM(28)/'ISO-TP4 (ISO Transport Protocol Class 4)'/
      DATA ILV_IPPRT(29)/30/, NLV_IPNAMLEN(29)/36/,
     +	   CLV_IPNAM(29)/'NETBLT (Bulk Data Transfer protocol)'/
      DATA ILV_IPPRT(30)/31/, NLV_IPNAMLEN(30)/39/,
     +	   CLV_IPNAM(30)/'MFE-NSP (MFE Network Services Protocol)'/
      DATA ILV_IPPRT(31)/32/, NLV_IPNAMLEN(31)/37/,
     +	   CLV_IPNAM(31)/'MERIT-INP (MERIT Internodal Protocol)'/
      DATA ILV_IPPRT(32)/33/, NLV_IPNAMLEN(32)/34/,
     +	   CLV_IPNAM(32)/'SEP (Sequential Exchange Protocol)'/
      DATA ILV_IPPRT(33)/61/, NLV_IPNAMLEN(33)/28/,
     +	   CLV_IPNAM(33)/'(any host internal protocol)'/
      DATA ILV_IPPRT(34)/62/, NLV_IPNAMLEN(34)/4/,
     +	   CLV_IPNAM(34)/'CFTP'/
      DATA ILV_IPPRT(35)/63/, NLV_IPNAMLEN(35)/19/,
     +	   CLV_IPNAM(35)/'(any local network)'/
      DATA ILV_IPPRT(36)/64/, NLV_IPNAMLEN(36)/35/,
     +	   CLV_IPNAM(36)/'SAT-EXPAK (SATNET & Backroom EXPAK)'/
      DATA ILV_IPPRT(37)/65/, NLV_IPNAMLEN(37)/31/,
     +	   CLV_IPNAM(37)/'MIT-SUBNET (MIT Subnet Support)'/
      DATA ILV_IPPRT(38)/66/, NLV_IPNAMLEN(38)/38/,
     +	   CLV_IPNAM(38)/'RVD (MIT Remote Virtual Disk Protocol)'/
      DATA ILV_IPPRT(39)/67/, NLV_IPNAMLEN(39)/36/,
     +	   CLV_IPNAM(39)/'IPPC (Internet Pluribus Packet Core)'/
      DATA ILV_IPPRT(40)/68/, NLV_IPNAMLEN(40)/29/,
     +	   CLV_IPNAM(40)/'(any distributed file system)'/
      DATA ILV_IPPRT(41)/69/, NLV_IPNAMLEN(41)/27/,
     +	   CLV_IPNAM(41)/'SAT-MON (SATNET Monitoring)'/
      DATA ILV_IPPRT(42)/71/, NLV_IPNAMLEN(42)/35/,
     +	   CLV_IPNAM(42)/'IPCV (Internet Packet Core Utility)'/
      DATA ILV_IPPRT(43)/76/, NLV_IPNAMLEN(43)/39/,
     +	   CLV_IPNAM(43)/'BR-SAT-MON (Backroom SATNET Monitoring)'/
      DATA ILV_IPPRT(44)/78/, NLV_IPNAMLEN(44)/28/,
     +	   CLV_IPNAM(44)/'WB-MON (WIDEBAND Monitoring)'/
      DATA ILV_IPPRT(45)/78/, NLV_IPNAMLEN(45)/25/,
     +	   CLV_IPNAM(45)/'WB-EXPAK (WIDEBAND EXPAK)'/
      DATA ILV_IPPRT(46)/0/, NLV_IPNAMLEN(46)/12/,
     +	   CLV_IPNAM(46)/'- reserved -'/
      DATA ILV_IPPRT(47)/255/, NLV_IPNAMLEN(47)/12/,
     +	   CLV_IPNAM(47)/'- reserved -'/
C
CP BEGIN SUMMARIZE
CP.  initialize counters for command
C
      B=ICVSTT_BUFNUM
      DO I=0,ICPADR_MAXADDRS
	ICVADR_ADDRCNT(I,B)=0
      ENDDO
      DO I=0,255
	ICVSTT_PROTCNT(I,B)=0
      ENDDO
      DO I=0,150
	ICVSTT_SIZECNT(I,B)=0
      ENDDO
      ICVSTT_BYTES_S_PRE(B)=0
      ICVSTT_MSGS_S_PRE(B)=0
      ICVSTT_BYTES_S_POST(B)=0
      ICVSTT_MSGS_S_POST(B)=0
C
CP.  IF Internet message type filtering requested THEN
C
      IF(ICVPRS_NUMBER.EQ.7)THEN
C
CP.  .  engage special filter for that protocol type
C
	SCVFLT_FILTERS(0).ENABLE=1
	SCVFLT_FILTERS(0).PROT_ACT=2		! ONLY=PROTOCOL
	SCVFLT_FILTERS(0).W_PROTOCOL='0008'X	! Internet IP (DOD)
	SCVFLT_FILTERS(0).SRC_ACT=0
	SCVFLT_FILTERS(0).DST_ACT=0
C
CP.  ENDIF
C
      ENDIF
C
CP.  store the data processing mode for the Ethernet read routines
C
      CALL GET_PORT_COUNTERS(.FALSE.)
      CALL SYS$GETTIM(QLV_OLDSYSTIM)
      ICVETH_DISPLAYMODE=ICVPRS_NUMBER
      ILV_OLD_C_DBR=ICVETH_C_DBR
      ILV_OLD_C_BRC=ICVETH_C_BRC
      ILV_OLD_C_MBL=ICVETH_C_MBL
      ILV_OLD_C_MBY=ICVETH_C_MBY
      ILV_OLD_C_BS1=ICVETH_C_BS1
      ILV_OLD_C_BSM=ICVETH_C_BSM
      ILV_OLD_C_BID=ICVETH_C_BID
      ILV_OLD_C_OVR=ZEXT(NCVETH_C_OVR)
      ILV_OLD_C_LBE=ZEXT(NCVETH_C_LBE)
      ILV_OLD_C_XBU=ZEXT(NCVETH_C_SBU)+ZEXT(NCVETH_C_UBU)
C
CP.  wait for the user to stop data collection
C
      IF(IAND(NCVPRS_PROTNUM,8).NE.0)THEN
	CALL SYS$SCHDWK(,,BCVPRS_DSTADR,)	! obtained by parser
      ELSE
	CALL SYS$SCHDWK(,,QCVDSP_INTERVAL,)
      ENDIF
      CALL SYS$HIBER
C
CP.  turn off data collection
C
      ICVETH_DISPLAYMODE=0
C
CP.  get current system time
C
      CALL SYS$GETTIM(QLV_SYSTIM)
C
CP.  get current counters
C
      CALL GET_PORT_COUNTERS(.FALSE.)
C
CP.  disengage special filter (if used)
C
      SCVFLT_FILTERS(0).ENABLE=0
C
CP.  print summary title
C
      CLV_LINE=CLV_TITLE(ICVPRS_NUMBER)//' from '
      CALL SYS$ASCTIM(,CLV_LINE(25:47),QLV_OLDSYSTIM,)
      CALL LIB$PUT_OUTPUT(CLV_LINE(1:47))
      CLV_LINE='                   to '
      CALL SYS$ASCTIM(,CLV_LINE(25:47),QLV_SYSTIM,)
      CALL LIB$PUT_OUTPUT(CLV_LINE(1:47))
C
CP.  calculate message loss factor and display it on screen
C
      CLV_LINE='xxx.xx% message loss'
      CALL FOR$CVT_D_TF(DFLOAT(100*((ICVETH_C_DBR-ILV_OLD_C_DBR)-
     +						ICVSTT_MSGS_S_PRE(B)))/
     +				DFLOAT(MAX(ICVETH_C_DBR-ILV_OLD_C_DBR,1)),
     +			  CLV_LINE(1:6),%VAL(2))
C
CP.  write pre-filter and post-filter statistics to screen
C
      CALL SYS$FAO('pre-filter:!10UL bytes !9UL msgs',,
     +		   CLV_LINE(31:),%VAL(ICVSTT_BYTES_S_PRE(B)),
     +		   %VAL(ICVSTT_MSGS_S_PRE(B)))
      CALL LIB$PUT_OUTPUT(CLV_LINE(1:69))
      CLV_LINE=' '
      CALL SYS$FAO('post-filter:!10UL bytes !9UL msgs',,
     +		   CLV_LINE(30:),%VAL(ICVSTT_BYTES_S_POST(B)),
     +		   %VAL(ICVSTT_MSGS_S_POST(B)))
      CALL LIB$PUT_OUTPUT(CLV_LINE(1:69))
C
CP.  DO CASE (displaymode) OF
C
      GOTO (1000,2000,3000,1000,5000,9999,7000), ICVPRS_NUMBER
C
CP.  CASE 1 or 4 (destinations or sources):
C
 1000 CONTINUE
C
CP.  .  sort the address statistics by descending size (indexed shellsort)
C
	DO I=0,ICPADR_MAXADDRS
	  IDX(I)=I
	ENDDO
	D=(ICPADR_MAXADDRS+1)/2
	DO WHILE (D.GT.0)
	  DO I=0,ICPADR_MAXADDRS-D
	    IF(ICVADR_ADDRCNT(IDX(I),B).LT.ICVADR_ADDRCNT(IDX(I+D),B))THEN
	      TIDX=IDX(I+D)
	      IDX(I+D)=IDX(I)
	      IF(I.GT.D)THEN
		J=I-D
		DO WHILE (J.GE.0.AND.
     +			  ICVADR_ADDRCNT(TIDX,B).GT.
     +					ICVADR_ADDRCNT(IDX(J),B))
		  IDX(J+D)=IDX(J)
		  J=J-D
		ENDDO
		IDX(J+D)=TIDX
	      ELSE
		IDX(I)=TIDX
	      ENDIF
	    ENDIF
	  ENDDO
	  D=D/2
	ENDDO
C
CP.  .  DO for all addresses with non-zero counts
C
	D=0
	DO WHILE (D.LE.ICPADR_MAXADDRS.AND.
     +		  ICVADR_ADDRCNT(IDX(D),B).GT.0)
	  D=D+1
	ENDDO
	DO I=0,D-1
	  J=IDX(I)
C
CP.  .  .  format line for display and write to SMG
C
	  CLV_LINE=' '
	  CALL OTS$CVT_L_TI(ICVADR_ADDRCNT(J,B),CLV_LINE(1:10),
     +			    %VAL(1),%VAL(4))
	  CALL OTS$CVT_L_TZ(SCVADR_ADDR(J).LNUM,CLV_SCR,%VAL(8))
	  CLV_LINE(12:22)=CLV_SCR(7:8)//'-'//CLV_SCR(5:6)//
     +			  '-'//CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	  CALL OTS$CVT_L_TZ(SCVADR_ADDR(J).NUM(2),CLV_SCR(1:4),
     +			    %VAL(4),%VAL(2))
	  CLV_LINE(23:28)='-'//CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	  LN=29
	  IF(NCVADR_DECNETNAMLEN(J).NE.0)THEN
	    CLV_LINE(30:)=CCVADR_DECNETNAM(J)(1:NCVADR_DECNETNAMLEN(J))
	    LN=NCVADR_DECNETNAMLEN(J)+29
	  ENDIF
	  IF(NCVADR_INETNAMLEN(J).NE.0)THEN
	    IF(LN.GT.29)THEN
	      LN=LN+1
	      CLV_LINE(LN:LN)=','
	    ENDIF
	    CLV_LINE(LN+1:)=CCVADR_INETNAM(J)(1:NCVADR_INETNAMLEN(J))
	    LN=LN+NCVADR_INETNAMLEN(J)
	  ENDIF
	  IF(NCVADR_MISCNAMLEN(J).NE.0)THEN
	    IF(LN.GT.29)THEN
	      LN=LN+1
	      CLV_LINE(LN:LN)=','
	    ENDIF
	    CLV_LINE(LN+1:)=CCVADR_MISCNAM(J)(1:NCVADR_MISCNAMLEN(J))
	    LN=LN+NCVADR_MISCNAMLEN(J)
	  ENDIF
	  CALL LIB$PUT_OUTPUT(CLV_LINE(1:LN))
C
CP.  .  ENDDO
C
	ENDDO
	GOTO 9999
C
CP.  CASE 2 (protocols):
C
 2000 CONTINUE
C
CP.  .  sort the protocol statistics by descending size (indexed shellsort)
C
	DO I=0,255
	  IDX(I)=I
	ENDDO
	D=128
	DO WHILE (D.GT.0)
	  DO I=0,255-D
	    IF(ICVSTT_PROTCNT(IDX(I),B).LT.ICVSTT_PROTCNT(IDX(I+D),B))THEN
	      TIDX=IDX(I+D)
	      IDX(I+D)=IDX(I)
	      IF(I.GT.D)THEN
		J=I-D
		DO WHILE (J.GE.0.AND.
     +			  ICVSTT_PROTCNT(TIDX,B).GT.
     +					ICVSTT_PROTCNT(IDX(J),B))
		  IDX(J+D)=IDX(J)
		  J=J-D
		ENDDO
		IDX(J+D)=TIDX
	      ELSE
		IDX(I)=TIDX
	      ENDIF
	    ENDIF
	  ENDDO
	  D=D/2
	ENDDO
C
CP.  .  DO for all protocols with non-zero counts
C
	D=0
	DO WHILE (D.LE.255.AND.
     +		  ICVSTT_PROTCNT(IDX(D),B).GT.0)
	  D=D+1
	ENDDO
	DO I=0,D-1
C
CP.  .  .  try to find protocol in protocol name table
C
	  J=1
	  DO WHILE (J.LE.ICVPRT_NUMNAMES.AND.
     +		    NCVSTT_PROT(IDX(I)).NE.NCVPRT_PROT_NUM(J))
	    J=J+1
	  ENDDO
C
CP.  .  .  format line for display and write to SMG
C
	  CLV_LINE=' '
	  CALL OTS$CVT_L_TI(ICVSTT_PROTCNT(IDX(I),B),
     +			    CLV_LINE(1:10),%VAL(1))
	  CALL OTS$CVT_L_TZ(NCVSTT_PROT(IDX(I)),
     +			    CLV_SCR(1:4),%VAL(4),%VAL(2))
	  CLV_LINE(12:16)=CLV_SCR(3:4)//'-'//CLV_SCR(1:2)
	  LN=16
	  IF(J.LE.ICVPRT_NUMNAMES)THEN
	    CLV_LINE(18:)=CCVPRT_PROTNAM(J)(1:NCVPRT_PROTNAMLEN(J))
	    LN=17+NCVPRT_PROTNAMLEN(J)
	  ENDIF
	  CALL LIB$PUT_OUTPUT(CLV_LINE(1:LN))
C
CP.  .  ENDDO
C
	ENDDO
	GOTO 9999
C
CP.  CASE 3 (sizes):
C
 3000 CONTINUE
C
CP.  .  DO for all cell size ranges
C
	DO I=0,14
C
CP.  .  .  format line for display and write to SMG
C
	  CLV_LINE=' '
	  CALL SYS$FAO('!UL - !UL',,CLV_LINE(19:29),%VAL(I*100+1),
     +			 %VAL((I+1)*100))
	  CALL OTS$CVT_L_TI(ICVSTT_SIZECNT(I,B),
     +			    CLV_LINE(32:37),%VAL(1))

C
CP.  .  ENDDO
C
	ENDDO
	GOTO 9999
C
CP.  CASE 5 (general statistics):
C
 5000 CONTINUE
C
CP.  .  display current statistics
C
	CALL FOR$CVT_D_TF(DFLOAT(ICVETH_C_DBR-ILV_OLD_C_DBR),
     +			  CLV_LINE(1:9),%VAL(2))
	CALL FOR$CVT_D_TF(DFLOAT(ICVETH_C_BRC-ILV_OLD_C_BRC),
     +			  CLV_LINE(1:9),%VAL(2))
	CALL FOR$CVT_D_TF(DFLOAT(ICVETH_C_BRC-ILV_OLD_C_BRC)/
     +			  DFLOAT(MIN(ICVETH_C_DBR-ILV_OLD_C_DBR,1)),
     +			  CLV_LINE(1:9),%VAL(2))
	CALL FOR$CVT_D_TF(DFLOAT(ICVETH_C_MBL-ILV_OLD_C_MBL),
     +			  CLV_LINE(1:9),%VAL(2))
	CALL FOR$CVT_D_TF(DFLOAT(ICVETH_C_MBY-ILV_OLD_C_MBY),
     +			  CLV_LINE(1:9),%VAL(2))
	CALL FOR$CVT_D_TF(DFLOAT(ICVETH_C_MBY-ILV_OLD_C_MBY)/
     +			  DFLOAT(MIN(ICVETH_C_MBL-ILV_OLD_C_MBL,1)),
     +			  CLV_LINE(1:9),%VAL(2))
	CALL FOR$CVT_D_TF(DFLOAT(ICVETH_C_BS1-ILV_OLD_C_BS1),
     +			  CLV_LINE(1:9),%VAL(2))
	CALL FOR$CVT_D_TF(DFLOAT(ICVETH_C_BSM-ILV_OLD_C_BSM),
     +			  CLV_LINE(1:9),%VAL(2))
	CALL FOR$CVT_D_TF(DFLOAT(ICVETH_C_BID-ILV_OLD_C_BID),
     +			  CLV_LINE(1:9),%VAL(2))
	CALL FOR$CVT_D_TF(DFLOAT(JZEXT(NCVETH_C_OVR)-ILV_OLD_C_OVR),
     +			  CLV_LINE(1:9),%VAL(2))
	CALL FOR$CVT_D_TF(DFLOAT(JZEXT(NCVETH_C_LBE)-ILV_OLD_C_LBE),
     +			  CLV_LINE(1:9),%VAL(2))
	CALL FOR$CVT_D_TF(DFLOAT(JZEXT(NCVETH_C_SBU)+
     +					JZEXT(NCVETH_C_UBU)-ILV_OLD_C_XBU),
     +			  CLV_LINE(1:9),%VAL(2))
	GOTO 9999
C
CP.  CASE 7 (Internet):
C
 7000 CONTINUE
C
CP.  .  sort the protocol statistics by descending size (indexed shellsort)
C
	DO I=0,255
	  IDX(I)=I
	ENDDO
	D=128
	DO WHILE (D.GT.0)
	  DO I=0,255-D
	    IF(ICVSTT_PROTCNT(IDX(I),B).LT.ICVSTT_PROTCNT(IDX(I+D),B))THEN
	      TIDX=IDX(I+D)
	      IDX(I+D)=IDX(I)
	      IF(I.GT.D)THEN
		J=I-D
		DO WHILE (J.GE.0.AND.
     +			  ICVSTT_PROTCNT(TIDX,B).GT.
     +					ICVSTT_PROTCNT(IDX(J),B))
		  IDX(J+D)=IDX(J)
		  J=J-D
		ENDDO
		IDX(J+D)=TIDX
	      ELSE
		IDX(I)=TIDX
	      ENDIF
	    ENDIF
	  ENDDO
	  D=D/2
	ENDDO
C
CP.  .  DO for all protocols with non-zero counts
C
	D=0
	DO WHILE (D.LE.255.AND.
     +		  ICVSTT_PROTCNT(IDX(D),B).GT.0)
	  D=D+1
	ENDDO
	DO I=0,D-1
C
CP.  .  .  try to find protocol in protocol name table
C
	  J=1
	  DO WHILE (J.LE.ILP_NUMIP.AND.
     +		    IDX(I).NE.ILV_IPPRT(J))
	    J=J+1
	  ENDDO
C
CP.  .  .  format line for display and write to SMG
C
	  CLV_LINE=' '
	  CALL OTS$CVT_L_TI(ICVSTT_PROTCNT(IDX(I),B),
     +			    CLV_LINE(1:10),%VAL(1))
	  CALL OTS$CVT_L_TI(IDX(I),CLV_LINE(12:15),%VAL(1))
	  LN=15
	  IF(J.LE.ILP_NUMIP)THEN
	    CLV_LINE(17:)=CLV_IPNAM(J)(1:NLV_IPNAMLEN(J))
	    LN=16+NLV_IPNAMLEN(J)
	  ENDIF
	  CALL LIB$PUT_OUTPUT(CLV_LINE(1:LN))
C
CP.  .  ENDDO
C
	ENDDO
	GOTO 9999
C
CP.  END CASE
C
 9999 CONTINUE
C
CP.  RETURN
C
      SUMMARIZE=1
      RETURN
C
CP END SUMMARIZE
C
      END
      INTEGER*4 FUNCTION TRACE(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for TRACE
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      CHARACTER*23 CLV_TIME
      INTEGER*4 ILV_MSGVEC(0:8)
C
C   declare local COMMON's
C
      INTEGER*4 ICVDSP_INTERVAL/4/, QCVDSP_INTERVAL(2)/-40000000,-1/
      LOGICAL*1 LCVDSP_EXIT
      COMMON /DISPLAY/ ICVDSP_INTERVAL, QCVDSP_INTERVAL, LCVDSP_EXIT
      INCLUDE 'ETHERMON_ETH.INC'
      INCLUDE 'ETHERMON_PRS.INC'
      INCLUDE 'ETHERMON_STATS.INC'
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__TRACERPT, ETH__TRACERPT2
C
CP BEGIN TRACE
CP.  decide on file storage mode
C
      BCVETH_TRACEFLAG=0
      IF(IAND(NCVPRS_PROTNUM,1).NE.0) BCVETH_TRACEFLAG=1
      IF(IAND(NCVPRS_PROTNUM,2).NE.0) BCVETH_TRACEFLAG=0
C
CP.  open the output file
C
      IF(BCVETH_TRACEFLAG.EQ.0)THEN
	OPEN(UNIT=70,FILE=CCVPRS_FILESPEC(1:NCVPRS_FILESPECLEN),
     +	     STATUS='NEW',DEFAULTFILE='ETHERMON.DAT',
     +	     CARRIAGECONTROL='NONE',FORM='UNFORMATTED',
     +	     RECORDTYPE='VARIABLE',EXTENDSIZE=80)
      ELSE
	OPEN(UNIT=70,FILE=CCVPRS_FILESPEC(1:NCVPRS_FILESPECLEN),
     +	     STATUS='NEW',DEFAULTFILE='ETHERMON.LIS',
     +	     CARRIAGECONTROL='LIST',FORM='FORMATTED',
     +	     RECORDTYPE='VARIABLE',EXTENDSIZE=80)
	CALL SYS$ASCTIM(,CLV_TIME,,)
	WRITE(70,9000) CLV_TIME
 9000	FORMAT('  Ethernet TRACE: starting data collection at ',A23)
      ENDIF
C
CP.  set the "display" mode flag
C
      ICVSTT_BYTES_S_POST(ICVSTT_BUFNUM)=0
      ICVSTT_MSGS_S_POST(ICVSTT_BUFNUM)=0
      ICVSTT_BYTES_S_PRE(ICVSTT_BUFNUM)=0
      ICVSTT_MSGS_S_PRE(ICVSTT_BUFNUM)=0
      ICVETH_DISPLAYMODE=6
C
CP.  wait for the user to stop data collection
C
      IF(IAND(NCVPRS_PROTNUM,8).NE.0)THEN
	CALL SYS$SCHDWK(,,BCVPRS_DSTADR,)	! obtained by parser
      ELSE
	CALL SYS$SCHDWK(,,QCVDSP_INTERVAL,)
      ENDIF
      CALL SYS$HIBER
C
CP.  clear the "display" mode flag
C
      ICVETH_DISPLAYMODE=0
C
CP.  close the output file
C
      IF(BCVETH_TRACEFLAG.EQ.1)THEN
	CALL SYS$ASCTIM(,CLV_TIME,,)
	WRITE(70,9002) CLV_TIME, ICVSTT_BYTES_S_POST(ICVSTT_BUFNUM),
     +			ICVSTT_MSGS_S_POST(ICVSTT_BUFNUM),
     +			ICVSTT_BYTES_S_PRE(ICVSTT_BUFNUM),
     +			ICVSTT_MSGS_S_PRE(ICVSTT_BUFNUM)
 9002	FORMAT('  Ethernet TRACE: ending data collection at ',A23/
     +	       I14,' bytes,',I5,' messages analyzed'/
     +         I14,' bytes,',I5,' messages received before filtering')
      ENDIF
      CLOSE(UNIT=70)
C
CP.  report trace results
C
      ILV_MSGVEC(0)=8
      ILV_MSGVEC(1)=%LOC(ETH__TRACERPT)
      ILV_MSGVEC(2)=2
      ILV_MSGVEC(3)=ICVSTT_BYTES_S_POST(ICVSTT_BUFNUM)
      ILV_MSGVEC(4)=ICVSTT_MSGS_S_POST(ICVSTT_BUFNUM)
      ILV_MSGVEC(5)=%LOC(ETH__TRACERPT2)
      ILV_MSGVEC(6)=2
      ILV_MSGVEC(7)=ICVSTT_BYTES_S_PRE(ICVSTT_BUFNUM)
      ILV_MSGVEC(8)=ICVSTT_MSGS_S_PRE(ICVSTT_BUFNUM)
      CALL SYS$PUTMSG(ILV_MSGVEC,,,)
C
CP.  RETURN
C
      TRACE=1
      RETURN
C
CP END TRACE
C
      END
      INTEGER*4 FUNCTION Check_Internet_num(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for Check_Internet_num
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      INTEGER*4 I
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ADDR.INC'
      INCLUDE 'ETHERMON_PRS.INC'
C
C   declare system symbolic codes
C
      EXTERNAL	SS$_NOSUCHNODE
C
CP BEGIN Check_Internet_num
CP.  assume that no matching Internet address is currently in address table
C
      Check_Internet_num=%LOC(SS$_NOSUCHNODE)
C
CP.  DO WHILE end of address table not reached and valid entry with 
CP		matching Internet address not found
C
      I=0
      DO WHILE (I.LE.ICPADR_MAXADDRS.AND.
     +		(.NOT.LCVADR_USED(I).OR.
     +		 NCVADR_INETNAMLEN(I).EQ.0.OR.
     +		 ICVADR_INETADR(I).NE.ICVPRS_ADDRNUM))
C
CP.  .  step to next entry
C
	I=I+1
C
CP.  ENDDO
C
      ENDDO
C
CP.  IF match found THEN
C
      IF(I.LE.ICPADR_MAXADDRS)THEN
C
CP.  .  copy corresponding Ethernet address to address buffer
C
	ICVPRS_ADDRNUM=SCVADR_ADDR(I).LNUM
	NCVPRS_ADDRNUM(2)=SCVADR_ADDR(I).NUM(2)
C
CP.  .  change return status to success
C
	Check_Internet_num=1
C
CP.  ENDIF
C
      ENDIF
C
CP.  RETURN
C
      RETURN
C
CP END Check_Internet_num
C
      END
      INTEGER*4 FUNCTION check_node_name(ILA_OPTIONS,ILA_STRINGCNT,
     +			ILA_STRINGPTR,ILA_TOKENCNT,ILA_TOKENPTR,
     +			ILA_CHAR,ILA_NUMBER,ILA_PARAM)
C
CP DATA DECLARATIONS for check_node_name
C
      IMPLICIT NONE
C
C   declare subroutine arguments
C
      INTEGER*4 ILA_OPTIONS, ILA_STRINGCNT, ILA_STRINGPTR, ILA_TOKENCNT,
     +		ILA_TOKENPTR, ILA_CHAR, ILA_NUMBER, ILA_PARAM
C
C   declare local variables
C
      INTEGER*4 I, ILV_DESCR(0:1)
      LOGICAL*1 LLV_MATCH
C
C   declare local COMMON's
C
      INCLUDE 'ETHERMON_ADDR.INC'
      INCLUDE 'ETHERMON_PRS.INC'
C
C   declare local symbolic message codes
C
      EXTERNAL	ETH__INVNODSPC
C
C   declare system procedures
C
      INTEGER*4 LIB$INDEX,	STR$COMPARE
      EXTERNAL	LIB$INDEX,	STR$COMPARE
C
C   declare system symbolic codes
C
      EXTERNAL	SS$_NOSUCHNODE
C
CP BEGIN check_node_name
CP.  assume that no matching DECnet node name is currently in address table
C
      check_node_name=%LOC(SS$_NOSUCHNODE)
C
CP.  verify that node name is valid (starts with a letter)
C
      ILV_DESCR(0)=1
      ILV_DESCR(1)=%LOC(ILA_TOKENPTR)
      IF(LIB$INDEX('ABCDEFGHIJKLMNOPQRSTUVWXYZ',ILV_DESCR).EQ.0)THEN
	check_node_name=%LOC(ETH__INVNODSPC)
	RETURN
      ENDIF
      ILV_DESCR(0)=%LOC(ILA_TOKENCNT)
C
CP.  DO WHILE end of address table not reached and valid entry with 
CP		matching DECnet node name not found
C
      I=0
      LLV_MATCH=.FALSE.
      DO WHILE (I.LE.ICPADR_MAXADDRS.AND.
     +		(.NOT.LCVADR_USED(I).OR.
     +		 .NOT.LLV_MATCH))
C
CP.  .  step to next entry
C
	IF(.NOT.LCVADR_USED(I).OR.
     +	   NCVADR_DECNETNAMLEN(I).NE.%LOC(ILA_TOKENCNT))THEN
	  I=I+1
	ELSEIF(STR$COMPARE(ILV_DESCR,
     +		CCVADR_DECNETNAM(I)(1:NCVADR_DECNETNAMLEN(I))))THEN
	  I=I+1
	ELSE
	  LLV_MATCH=.TRUE.
	ENDIF
C
CP.  ENDDO
C
      ENDDO
C
CP.  IF match found THEN
C
      IF(I.LE.ICPADR_MAXADDRS)THEN
C
CP.  .  copy corresponding Ethernet address to address buffer
C
	ICVPRS_ADDRNUM=SCVADR_ADDR(I).LNUM
	NCVPRS_ADDRNUM(2)=SCVADR_ADDR(I).NUM(2)
C
CP.  .  change return status to success
C
	check_node_name=1
C
CP.  ENDIF
C
      ENDIF
C
CP.  RETURN
C
      RETURN
C
CP END check_node_name
C
      END
